! ================================================================================================================================
! MODULE       : stomate_lpj
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Main entry point for daily processes in STOMATE and LPJ (phenology, 
!! allocation, kill, turn, light, establish, crown, cover, lcchange)
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) : None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_lpj

  ! modules used:

  USE ioipsl_para
  USE xios_orchidee
  USE grid
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE lpj_constraints
  USE lpj_pftinout
  USE lpj_kill
  USE lpj_crown
  USE lpj_fire
  USE lpj_gap
  USE lpj_light
  USE lpj_establish
  USE lpj_cover
  USE stomate_prescribe
  USE stomate_phenology
  USE stomate_growth_fun_all
  USE stomate_turnover
  USE stomate_litter
  USE stomate_som_dynamics
  USE stomate_vmax
  USE stomate_lcchange
  USE stomate_phosphorus
!  USE Orch_Write_field_p
!gmjc
  USE grassland_constantes
  USE grassland_management
!end gmjc

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC StomateLpj,StomateLpj_clear

!gmjc
  LOGICAL, SAVE                         :: firstcall = .TRUE.  !!first call
  ! flag that enable grazing
  LOGICAL, SAVE :: enable_grazing
!end gmjc

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : StomateLpj_clear
!!
!>\BRIEF        Re-initialisation of variable
!!
!! DESCRIPTION  : This subroutine reinitializes variables. To be used if we want to relaunch 
!! ORCHIDEE but the routine is not used in current version.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE StomateLpj_clear

    CALL prescribe_clear
    CALL phenology_clear
    CALL turn_clear
    CALL som_dynamics_clear
    CALL constraints_clear
    CALL establish_clear
    CALL fire_clear
    CALL gap_clear
    CALL light_clear
    CALL pftinout_clear
  END SUBROUTINE StomateLpj_clear


!! ================================================================================================================================
!! SUBROUTINE   : StomateLPJ
!!
!>\BRIEF        Main entry point for daily processes in STOMATE and LPJ, structures the call sequence 
!!              to the different processes such as dispersion, establishment, competition and mortality of PFT's.
!! 
!! DESCRIPTION  : This routine is the main entry point to all processes calculated on a 
!! daily time step. Is mainly devoted to call the different STOMATE and LPJ routines 
!! depending of the ok_dgvm (is dynamic veg used) and lpj_constant_mortality (is background mortality used).
!! It also prepares the cumulative 
!! fluxes or pools (e.g TOTAL_M TOTAL_BM_LITTER etc...)
!!
!! This routine makes frequent use of "weekly", "monthly" and "long term" variables. Quotion is used because
!! by default "weekly" denotes 7 days, by default "monthly" denotes 20 days and by default "Long term" denotes
!! 3 years. dtslow refers to 24 hours (1 day).
!!
!!
!! RECENT CHANGE(S) : None
!! 
!! MAIN OUTPUT VARIABLE(S): All variables related to stomate and required for LPJ dynamic vegetation mode.
!!
!! REFERENCE(S) : 
!! - Krinner, G., N. Viovy, N. de Noblet-Ducoudré, J. Ogeé, J. Polcher, P. Friedlingstein, P. Ciais, S. Sitch, 
!! and I. C. Prentice. 2005. A dynamic global vegetation model for studies of the coupled atmosphere-biosphere 
!! system. Global Biogeochemical Cycles 19:GB1015, doi:1010.1029/2003GB002199.
!! - Sitch, S., B. Smith, I. C. Prentice, A. Arneth, A. Bondeau, W. Cramer, J. O. Kaplan, S. Levis, W. Lucht, 
!! M. T. Sykes, K. Thonicke, and S. Venevsky. 2003. Evaluation of ecosystem dynamics, plant geography and 
!! terrestrial carbon cycling in the LPJ dynamic global vegetation model. Global Change Biology 9:161-185.
!!
!! FLOWCHART    : Update with existing flowchart from N Viovy (Jan 19, 2012)
!! \n
!_ ================================================================================================================================
 
  SUBROUTINE StomateLpj (npts, dt_days, &
       neighbours, resolution, &
       herbivores, &
       tsurf_daily, tsoil_daily, t2m_daily, t2m_min_daily, &
       litterhum_daily, soilhum_daily, &
       maxmoiavail_lastyear, minmoiavail_lastyear, &
       gdd0_lastyear, precip_lastyear, &
       moiavail_month, moiavail_week, t2m_longterm, t2m_month, t2m_week, &
       tsoil_month, soilhum_month, &
       gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
       turnover_longterm, gpp_daily, &
!DSGlab_fac
       gpp_week, &
!DSG
       time_hum_min, maxfpc_lastyear, resp_maint_part, &
       PFTpresent, age, fireindex, firelitter, &
       leaf_age, leaf_frac, biomass, ind, adapted, regenerate, &
       senescence, when_growthinit, &
       litter, &
!gmjc
       litter_avail, litter_not_avail, litter_avail_frac, &
!end gmjc
       dead_leaves, som, lignin_struc, &
       lignin_wood, veget_max,veget, veget_max_new, npp_longterm, lm_lastyearmax, veget_lastlight, &
       everywhere, need_adjacent, RIP_time, &
       lai, rprof,npp_daily, turnover_daily, turnover_time,&
       control_moist, control_temp, som_input, &
       co2_to_bm, n_to_bm, p_to_bm,co2_fire, resp_hetero, resp_maint, resp_growth, &
       height, deadleaf_cover, vcmax, &
       nue,bm_to_litter, &
       prod10,prod100,flux10, flux100, &
       convflux,cflux_prod10,cflux_prod100, harvest_above, carb_mass_total, &
       fpc_max, MatrixA, &
       Tseason, Tmin_spring_time, begin_leaves, onset_date, KF, k_latosa_adapt, &
       cn_leaf_min_season, np_leaf_min_season, nstress_season, &
       pstress_season, &
!DSGlaitarget
       lai_target, lai_target_longterm, &
!DSGlaitarget
       moiavail_growingseason,soil_n_min, soil_p_min,rue_longterm, &
       n_uptake_daily, p_uptake_daily, &
!JC Debug MOD005
       record_reserve_grass, &
!End JC Debug MOD005
!gmjc
       wshtotsum, sr_ugb, compt_ugb, nb_ani, grazed_frac, &
       import_yield, sla_age1, t2m_14, sla_calc, snowfall_daily, day_of_year, &
       when_growthinit_cut, nb_grazingdays, &
       EndOfYear, &
       moiavail_daily,tmc_topgrass_daily,fc_grazing,snowmass_daily,&
       after_snow, after_wet, wet1day, wet2day, &
! JCADD 20170421 for CNP merge
       grm_nfert,grm_pfert,BNF_clover_daily, &
       GRM_devstage,agri_fert_save,agri_nfert,agri_pfert)
! END JCADD
!end gmjc
    
  !! 0. Variable and parameter declaration

    !! 0.1 input

    INTEGER(i_std), INTENT(in)                                 :: npts                 !! Domain size (unitless)
    REAL(r_std), INTENT(in)                                    :: dt_days              !! Time step of Stomate (days)
    INTEGER(i_std), DIMENSION(npts,NbNeighb), INTENT(in)       :: neighbours           !! Indices of the 8 neighbours of each grid 
                                                                                       !! point [1=North and then clockwise] 
    REAL(r_std), DIMENSION(npts,2), INTENT(in)                 :: resolution           !! Resolution at each grid point (m)  
                                                                                       !! [1=E-W, 2=N-S] 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: herbivores           !! Time constant of probability of a leaf to 
                                                                                       !! be eaten by a herbivore (days) 
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: tsurf_daily          !! Daily surface temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: tsoil_daily          !! Daily soil temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_daily            !! Daily 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_min_daily        !! Daily minimum 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: litterhum_daily      !! Daily litter humidity (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: soilhum_daily        !! Daily soil humidity (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: maxmoiavail_lastyear !! Last year's maximum moisture availability 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: minmoiavail_lastyear !! Last year's minimum moisture availability 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: gdd0_lastyear        !! Last year's GDD0 (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: precip_lastyear      !! Lastyear's precipitation 
                                                                                       !! @tex $(mm year^{-1})$ @endtex
										       !! to determine if establishment possible
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: moiavail_month       !! "Monthly" moisture availability (0 to 1, 
                                                                                       !! unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: moiavail_week        !! "Weekly" moisture availability 
                                                                                       !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_longterm         !! "Long term" 2 meter reference 
                                                                                       !! temperatures (K) 
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_month            !! "Monthly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_week             !! "Weekly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: Tseason              !! "seasonal" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: Tmin_spring_time     !! Number of days after begin_leaves (leaf onset) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: onset_date           !! Date in the year at when the leaves started to grow(begin_leaves)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: tsoil_month          !! "Monthly" soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)              :: soilhum_month        !! "Monthly" soil humidity
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: gdd_m5_dormance      !! Growing degree days (K), threshold -5 deg 
                                                                                       !! C (for phenology) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: gdd_from_growthinit  !! growing degree days, since growthinit for crops
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: gdd_midwinter        !! Growing degree days (K), since midwinter 
                                                                                       !! (for phenology) - this is written to the history files 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: ncd_dormance         !! Number of chilling days (days), since 
                                                                                       !! leaves were lost (for phenology) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: ngd_minus5           !! Number of growing days (days), threshold 
                                                                                       !! -5 deg C (for phenology) 
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)        :: turnover_longterm    !! "Long term" turnover rate  
                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: gpp_daily            !! Daily gross primary productivity  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: gpp_week             !! Weekly gross primary productivity   
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: time_hum_min         !! Time elapsed since strongest moisture 
                                                                                       !! availability (days) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: maxfpc_lastyear      !! Last year's maximum foliage projected
                                                                                       !! coverage for each natural PFT,
                                                                                       !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)        :: resp_maint_part      !! Maintenance respiration of different 
                                                                                       !! plant parts  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: fpc_max              !! "Maximal" coverage fraction of a PFT (LAI 
                                                                                       !! -> infinity) on ground  
                                                                                       !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)                :: veget_max_new        !! Old timestep "maximal" coverage fraction of a PFT 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: cn_leaf_min_season       !! Seasonal min CN ratio of leaves 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: np_leaf_min_season       !! Seasonal min NP ratio of leaves 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: nstress_season       !! N-related seasonal stress (used for allocation)    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: pstress_season       !! P-related seasonal stress (used for allocation)    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: moiavail_growingseason !! mean growing season moisture availability 

!DSGlaitarget
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: lai_target_longterm    !! seasonal lai target 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: lai_target           !!  lai target 
!DSGlaitarget

  !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: npp_daily            !! Net primary productivity 
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out) :: turnover_daily       !! Turnover rates 
                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: co2_to_bm            !! CO2 taken up from atmosphere when 
                                                                                       !! introducing a new PFT (introduced for 
                                                                                       !! carbon balance closure) 
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: n_to_bm              !! N taken up from ?? when 
                                                                                       !! introducing a new PFT (introduced for 
                                                                                       !! carbon balance closure) 
                                                                                       !! @tex $(gN m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: p_to_bm              !! P taken up from ?? when 
                                                                                       !! introducing a new PFT (introduced for 
                                                                                       !! carbon balance closure) 
                                                                                       !! @tex $(gP m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: co2_fire             !! Carbon emitted into the atmosphere by 
                                                                                       !! fire (living and dead biomass)  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: resp_hetero          !! Heterotrophic respiration
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: resp_maint           !! Maintenance respiration  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: resp_growth          !! Growth respiration  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    
    REAL(r_std), DIMENSION(npts), INTENT(inout)                :: deadleaf_cover       !! Fraction of soil covered by dead leaves 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: vcmax                !! Maximum rate of carboxylation 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out):: bm_to_litter      !! Conversion of biomass to litter 
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    LOGICAL, DIMENSION(npts,nvm), INTENT(out)                  :: begin_leaves         !! signal to start putting leaves on (true/false)
    REAL(r_std),DIMENSION(npts,nvm), INTENT(out)               :: nue                  !! Nitrogen use Efficiency with impact of leaf age (umol CO2 (gN)-1 s-1) 

    !! 0.3 Modified variables
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: height               !! Height of vegetation (m) 
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(inout)          :: control_moist        !! Moisture control of heterotrophic 
                                                                                       !! respiration (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(inout)          :: control_temp         !! Temperature control of heterotrophic 
                                                                                       !! respiration, above and below 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(inout) :: som_input     !! Quantity of carbon going into carbon 
                                                                                       !! pools from litter decomposition  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: lai                  !! Leaf area index OF AN INDIVIDUAL PLANT,
										       !! where a PFT contains n indentical plants
										       !! i.e., using the mean individual approach 
                                                                                       !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: rprof                !! Prescribed root depth (m) 
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                :: PFTpresent           !! Tab indicating which PFTs are present in 
                                                                                       !! each pixel 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: age                  !! Age (years)    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: fireindex            !! Probability of fire (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: firelitter           !! Longer term litter above the ground that 
                                                                                       !! can be burned, @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nleafages), INTENT(inout)  :: leaf_age             !! Leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nleafages), INTENT(inout)  :: leaf_frac            !! Fraction of leaves in leaf age class, 
                                                                                       !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(inout) :: biomass        !! Biomass @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: ind                  !! Density of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: adapted              !! Adaptation of PFT (killed if too cold) 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: regenerate           !! "Fitness": Winter sufficiently cold for 
                                                                                       !! PFT regeneration ? (0 to 1, unitless) 
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                :: senescence           !! Flag for setting senescence stage (only 
                                                                                       !! for deciduous trees) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: when_growthinit      !! How many days ago was the beginning of 
                                                                                       !! the growing season (days) 
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(inout):: litter     !! Metabolic and structural litter, above 
                                                                                       !! and below ground 
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(inout)      :: dead_leaves          !! Dead leaves on ground, per PFT, metabolic 
                                                                                       !! and structural,  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(inout)      :: som               !! Carbon pool: active, slow, or passive, 
                                                                                       !! @tex $(gC m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)      :: lignin_struc         !! Ratio of Lignin/Carbon in structural 
                                                                                       !! litter, above and below ground,  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)      :: lignin_wood          !! Ratio of Lignin/Carbon in woody	                                       !! litter, above and below ground, 	
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: veget_max            !! "Maximal" coverage fraction of a PFT (LAI 
                                                                                       !! -> infinity) on ground 
    REAL(r_std),DIMENSION(npts,nvm), INTENT(in)                :: veget               !! Fractional coverage: actually share of the pixel 
                                                                              !! covered by a PFT (fraction of ground area), 
                                                                              !! taking into account LAI ??(= grid scale fpc)?? 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: npp_longterm         !! "Long term" mean yearly primary 
                                                                                       !! productivity 
                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: lm_lastyearmax       !! Last year's maximum leaf mass, for each 
                                                                                       !! PFT @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: veget_lastlight      !! Vegetation fractions (on ground) after 
                                                                                       !! last light competition  
                                                                                       !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: everywhere           !! Is the PFT everywhere in the grid box or 
                                                                                       !! very localized (after its introduction) 
                                                                                       !! (unitless) 
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                :: need_adjacent        !! In order for this PFT to be introduced, 
                                                                                       !! does it have to be present in an 
                                                                                       !! adjacent grid box? 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: RIP_time             !! How much time ago was the PFT eliminated 
                                                                                       !! for the last time (y) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: turnover_time        !! Turnover_time of leaves for grasses 
                                                                                       !! (days)
    REAL(r_std),DIMENSION(npts,0:10), INTENT(inout)            :: prod10               !! Products remaining in the 10
                                                                                       !! year-turnover pool after the annual 
                                                                                       !! release for each compartment (10
                                                                                       !! + 1 : input from year of land cover 
                                                                                       !! change) @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts,0:100), INTENT(inout)           :: prod100              !! Products remaining in the 100 
                                                                                       !! year-turnover pool after the annual 
                                                                                       !! release for each compartment (100 
                                                                                       !! + 1 : input from year of land cover 
                                                                                       !! change) @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts,10), INTENT(inout)              :: flux10               !! Annual release from the 10
                                                                                       !! year-turnover pool compartments  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts,100), INTENT(inout)             :: flux100              !! Annual release from the 100 
                                                                                       !! year-turnover pool compartments  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts), INTENT(inout)                 :: convflux             !! Release during first year following land 
                                                                                       !! cover change @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts), INTENT(inout)                 :: cflux_prod10         !! Total annual release from the 10 
                                                                                       !! year-turnover pool 
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts), INTENT(inout)                 :: cflux_prod100        !! Total annual release from the 100 
                                                                                       !! year-turnover pool 
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts), INTENT(inout)                :: harvest_above        !! Harvest above ground biomass for 
                                                                                       !! agriculture @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts), INTENT(inout)                :: carb_mass_total      !! Carbon Mass total (soil, litter, veg) 
                                                                                       !! @tex $(gC m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(inout) :: MatrixA         !! Matrix containing the fluxes  
                                                                                       !! between the carbon pools
                                                                                       !! per sechiba time step 
                                                                                       !! @tex $(gC.m^2.day^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: KF                            !! Scaling factor to convert sapwood mass
                                                                                       !! into leaf mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: k_latosa_adapt                !! Leaf to sapwood area adapted for long 
                                                                                       !! term water stress (m)
    REAL(r_std), DIMENSION(npts,nvm,nionspec), INTENT(inout)      :: n_uptake_daily    !! Uptake of soil N by plants  
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: p_uptake_daily    !! Uptake of soil P by plants  
       !! (gN/m**2/day)  
    REAL(r_std), DIMENSION(npts,nvm,nnspec), INTENT(inout)       :: soil_n_min         !! mineral nitrogen in the soil (gN/m**2)  
    REAL(r_std), DIMENSION(npts,nvm,npspec), INTENT(inout)       :: soil_p_min         !! mineral phosphorus in the soil (gP/m**2)  
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: rue_longterm            !! Longterm radiation use efficiency 
!JC Debug MOD005
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)       :: record_reserve_grass  !! record maximum grass reserve pool
!End JC Debug MOD005
                                                                                !! (??units??) 
!gmjc 
    !for grazing litter
    REAL(r_std), DIMENSION(npts,nlitt,nvm), INTENT(out):: litter_avail
    REAL(r_std), DIMENSION(npts,nlitt,nvm) , INTENT(out):: litter_not_avail
    REAL(r_std), DIMENSION(npts,nlitt,nvm), INTENT(in):: litter_avail_frac
    LOGICAL, INTENT(in)                                        :: EndOfYear
   ! snowfall_daily (mm/d?)
    REAL(r_std), DIMENSION(npts), INTENT(in)         :: snowfall_daily
   ! snowmass_daily (kg/m2)
    REAL(r_std), DIMENSION(npts), INTENT(in)         :: snowmass_daily
    ! "14days" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)         ::  t2m_14
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  sla_calc
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  wshtotsum
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  sr_ugb
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  compt_ugb
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  nb_ani
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  grazed_frac
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  import_yield
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  sla_age1
    INTEGER(i_std), INTENT(in)                       ::  day_of_year
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  when_growthinit_cut
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  nb_grazingdays
!    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  ::  resp_hetero_litter_d
!    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(inout)  ::
!    resp_hetero_soil_d
! top 5 layer grassland soil moisture for grazing
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)           :: moiavail_daily
!! Daily moisture availability (0-1, unitless)
    REAL(r_std),DIMENSION (npts), INTENT(in)       :: tmc_topgrass_daily
    REAL(r_std),DIMENSION (npts), INTENT(in)       :: fc_grazing
    REAL(r_std),DIMENSION (npts), INTENT(inout)    :: after_snow
    REAL(r_std),DIMENSION (npts), INTENT(inout)    :: after_wet
    REAL(r_std),DIMENSION (npts), INTENT(inout)    :: wet1day
    REAL(r_std),DIMENSION (npts), INTENT(inout)    :: wet2day
    ! JCADD 20170421 for CNP merge
    REAL(r_std), DIMENSION(npts,nvm,ninput), INTENT(out)   :: grm_nfert !! nfert from GRM module
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)   :: grm_pfert !! pfert from GRM module
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)   :: BNF_clover_daily
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)   :: GRM_devstage
    REAL(r_std), DIMENSION(npts,nvm,4), INTENT(in) :: agri_fert_save
    REAL(r_std), DIMENSION(npts,nvm,ninput), INTENT(out)  :: agri_nfert
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)  :: agri_pfert
    ! END JCADD
!end gmjc

    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)           :: recycling_daily     !! Recycled nutrients from tissue prior shedding

    REAL(r_std), DIMENSION(npts,nvm)                            :: nox_emission        !! NOx emitted into the atmosphere by fire (living and dead biomass) 
                                                                                        !! (in gN/m**2/day) 
                                                                                        !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: tot_bm_to_litter    !! Total conversion of biomass to litter 
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: tot_live_biomass    !! Total living biomass  
                                                                                       !! @tex $(gC m{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)           :: bm_alloc            !! Biomass increase, i.e. NPP per plant part 
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: tot_turnover        !! Total turnover rate  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                            :: tot_litter_soil_carb!! Total soil and litter carbon  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                            :: tot_litter_carb     !! Total litter carbon 
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                            :: tot_soil_carb       !! Total soil carbon  
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts)                                :: carb_mass_variation !! Carbon Mass variation  
                                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                            :: cn_ind              !! Crown area of individuals 
                                                                                       !! @tex $(m^{2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                            :: woodmass_ind        !! Woodmass of individuals (gC) 
    REAL(r_std), DIMENSION(npts,nvm,nparts)                     :: f_alloc             !! Fraction that goes into plant part 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                                :: avail_tree          !! Space availability for trees 
                                                                                       !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                                :: avail_grass         !! Space availability for grasses 
                                                                                       !! (0 to 1, unitless) 
    INTEGER                                                     :: j,k,l
    REAL(r_std),DIMENSION(npts)                                 :: prod10_total        !! Total products remaining in the pool 
                                                                                       !! after the annual release 
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts)                                 :: prod100_total       !! Total products remaining in the pool 
                                                                                       !! after the annual release 
                                                                                       !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts)                                 :: cflux_prod_total    !! Total flux from conflux and the 10/100 
                                                                                       !! year-turnover pool 
                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts)                                 :: nflux_prod_total    !! Total flux from land cover change                                    
       !! @tex $(gN m^{-2} year^{-1})$ @endtex  
    REAL(r_std),DIMENSION(npts,nvm)                             :: veget_max_tmp       !! "Maximal" coverage fraction of a PFT  
                                                                                       !! (LAI-> infinity) on ground (unitless) 
    REAL(r_std), DIMENSION(npts,nvm)                            :: mortality           !! Fraction of individual dying this time 
                                                                                       !! step (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                                :: vartmp              !! Temporary variable used to add history
    REAL(r_std), DIMENSION(npts,nvm)                            :: histvar             !! History variables
    REAL(r_std), DIMENSION(npts,nvm)                 :: use_reserve             !! Mass taken from carbohydrate reserve 
                                                                                !! @tex $(gC m^{-2})$ @endtex
    CHARACTER(LEN=2), DIMENSION(nelements)                     :: element_str         !! string suffix indicating element 
    REAL(r_std), DIMENSION(npts,nvm)                           :: vcmax_new          !! Vmax leafN relationship from Kattge et al. 2009 
    REAL(r_std), DIMENSION(npts,nvm)                           :: vcmax_walker       !! Vcmax leafN leafP relationship from Walker et al. (2014)
    REAL(r_std), DIMENSION(npts,nvm)                           :: vcmax_domingues     !! Vcmax- leafN,leafP,SLA relationship from Domingues et al. 2010
    REAL(r_std), DIMENSION(npts,nvm)                           :: leafN_dummy
    REAL(r_std), DIMENSION(npts,nvm)                           :: leafP_dummy
    REAL(r_std), DIMENSION(npts,nvm)                           :: N_support              !! Nitrogen which is added to the ecosystem to support vegetation growth (only used when IMPOSE_CN .EQ. true) (gC/m2/day)
    REAL(r_std), DIMENSION(npts,nvm)                           :: P_support              !! Phosphorus which is added to the ecosystem to support vegetation growth (only used when IMPOSE_CN .EQ. true) (gP/m2/day)

    ! diagnostic outputs (of the drivers of nutrient acquisation processes) 
    REAL(r_std), DIMENSION(npts,nvm)                           :: plant_cn             !! C to N ratio of labile plant tissue (icarbres, ilabile, ileaf)
    REAL(r_std), DIMENSION(npts,nvm)                           :: plant_np             !! N to P ratio of labile plant tissue (icarbres, ilabile, ileaf)
   
    
    !DSG debug
    INTEGER                                                     ::    dsg1,dsg2,dsg3
    REAL(r_std), DIMENSION(npts,nvm,nelements)                :: mass_before
    REAL(r_std), DIMENSION(npts,nvm,nelements)                :: mass_change

    REAL(r_std), PARAMETER :: sla_conv=.5                                              !! conversion of sla m2/g(C) ->m2/g(DW)
!gmjc lcchange of managed grassland
    ! "maximal" coverage fraction of a PFT (LAI -> infinity) on ground
    INTEGER(i_std)                       :: ier
    LOGICAL                               :: l_error =.FALSE.
      ! variables to get closure carbon cycle for nbp
    REAL(r_std), DIMENSION(npts,nvm)                            :: harvest_gm
    REAL(r_std), DIMENSION(npts,nvm)                            :: ranimal_gm
    REAL(r_std), DIMENSION(npts,nvm)                            :: ch4_pft_gm
    REAL(r_std), DIMENSION(npts)                                :: ch4_gm
    REAL(r_std), DIMENSION(npts,nvm)                            :: cinput_gm
    REAL(r_std), DIMENSION(npts)                                :: co2_gm
    REAL(r_std),DIMENSION(npts,nvm)                             :: veget_max_gm
    REAL(r_std), DIMENSION(npts)                                :: veget_exist_gm
    REAL(r_std),DIMENSION(npts,nvm)                             :: n2o_pft_gm
    REAL(r_std), DIMENSION(npts)                                :: n2o_gm
    REAL(r_std),DIMENSION(npts,nvm,nelements)                   :: grm_masschangecnp
    REAL(r_std),DIMENSION(npts,nvm,nelements)                   :: fert_masschangecnp
!end gmjc
!_ ================================================================================================================================

    ! init mass check variables
    mass_before(:,:,:) = zero
    mass_change(:,:,:) = zero
! JCADD 20170421 for CNP merge
    grm_masschangecnp(:,:,:) = zero
    fert_masschangecnp(:,:,:) = zero
! END JCADD
    IF (printlev>=3) WRITE(numout,*) 'Entering stomate_lpj'
!gmjc
    IF (firstcall) THEN

        firstcall = .FALSE.

        !Config  Key  = GRM_ENABLE_GRAZING
        !Config  Desc = grazing allowed
        !Config  Def  = n
        !Config  Help = flag for choose if you want animals or not.
        !
        enable_grazing = .FALSE.
        CALL getin_p('GRM_ENABLE_GRAZING',enable_grazing)
        WRITE (numout,*) 'enable_grazing',enable_grazing
        WRITE (numout,*) 'manage',is_grassland_manag
        WRITE (numout,*) 'cut',is_grassland_cut
        WRITE (numout,*) 'grazed',is_grassland_grazed

    ENDIF
!end gmjc
 
  !! 1. Initializations
    
    !! 1.1 Initialize variables to zero
    co2_to_bm(:,:) = zero
    n_to_bm(:,:) = zero
    p_to_bm(:,:) = zero
    co2_fire(:,:) = zero
    nox_emission(:,:) = zero
    npp_daily(:,:) = zero
    resp_maint(:,:) = zero
    resp_growth(:,:)      = zero
    harvest_above(:)      = zero
    bm_to_litter(:,:,:,:) = zero
    cn_ind(:,:) = zero
    woodmass_ind(:,:) = zero
    turnover_daily(:,:,:,:) = zero
    recycling_daily(:,:,:,:) = zero
    use_reserve(:,:) = zero
    !! 1.2  Initialize variables to veget_max
    veget_max_tmp(:,:) = veget_max(:,:)
    N_support(:,:) = zero
    P_support(:,:) = zero
!gmjc
    !! Initialize gm variables for nbp to zero
    harvest_gm(:,:) = zero
    ranimal_gm(:,:) = zero
    ch4_pft_gm(:,:) = zero
    cinput_gm(:,:) = zero
    co2_gm(:) = zero
    ch4_gm(:) = zero
    n2o_gm(:) = zero
    n2o_pft_gm(:,:) = zero
    veget_max_gm(:,:) = zero
    veget_exist_gm(:) = zero
!end gmjc

    !! 1.3 Calculate some vegetation characteristics
    
    !! 1.3.1 Calculate some vegetation characteristics 
    !        Calculate cn_ind (individual crown mass) and individual height from
    !        state variables if running DGVM or dynamic mortality in static cover mode
    !??        Explain (maybe in the header once) why you mulitply with veget_max in the DGVM
    !??        and why you don't multiply with veget_max in stomate.
    IF ( ok_dgvm .OR. .NOT.lpj_gap_const_mort) THEN
       IF(ok_dgvm) THEN
          WHERE (ind(:,:).GT.min_stomate)
             woodmass_ind(:,:) = &
                  ((biomass(:,:,isapabove,icarbon)+biomass(:,:,isapbelow,icarbon) &
                  +biomass(:,:,iheartabove,icarbon)+biomass(:,:,iheartbelow,icarbon)) & 
                  *veget_max(:,:))/ind(:,:)
          ENDWHERE
       ELSE
          WHERE (ind(:,:).GT.min_stomate)
             woodmass_ind(:,:) = &
                  (biomass(:,:,isapabove,icarbon)+biomass(:,:,isapbelow,icarbon) &
                  +biomass(:,:,iheartabove,icarbon)+biomass(:,:,iheartbelow,icarbon))/ind(:,:)
          ENDWHERE
       ENDIF

       ! DSG: does not modify biomass
       CALL crown (npts,  PFTpresent, &
            ind, biomass, woodmass_ind, &
            veget_max, cn_ind, height)
    ENDIF

    !! 1.3.2 Prescribe characteristics if the vegetation is not dynamic
    !        IF the DGVM is not activated, the density of individuals and their crown
    !        areas don't matter, but they should be defined for the case we switch on
    !        the DGVM afterwards. At the first call, if the DGVM is not activated, 
    !        impose a minimum biomass for prescribed PFTs and declare them present.
    IF (printlev>=4) WRITE(numout,*) 'before prescribe'
    IF (printlev>=4) WRITE(numout,*) 'ind(test_grid,test_pft)=',ind(test_grid,test_pft)

    !DSG mass conservation ========================================
    mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3)

    CALL prescribe (npts, &
         veget_max, veget, dt_days, PFTpresent, everywhere, when_growthinit, &
         biomass, leaf_frac, ind, co2_to_bm,KF,senescence,age,npp_longterm,&
         lm_lastyearmax,k_latosa_adapt)

    IF(dsg_debug) THEN
       !DSG mass conservation ============================================
       CALL  check_mass(npts,biomass(:,:,:,:),'lpj: after prescribe')
       WRITE (numout,*) 'No mass cons test; prescribe violates mass per design' !:DSG right?
       !DSG mass_change(:,:,icarbon)     = zero
       !DSG mass_change(:,:,initrogen)   = zero
       !DSG mass_change(:,:,iphosphorus) = zero
       !DSG CALL cons_mass(npts, nvm, nelements,     &  ! dimensions
       !DSG        mass_before(:,:,:),               &  ! mass before
       !DSG        SUM(biomass(:,:,:,:),DIM=3),      &  ! mass after
       !DSG        mass_change(:,:,:)                &  ! net of fluxes
       !DSG        )
    ENDIF

    IF (printlev>=4) WRITE(numout,*) 'Leaving prescribe'
    IF (printlev>=4) WRITE(numout,*) 'ind(test_grid,test_pft)=',ind(test_grid,test_pft)
  !! 2. Climatic constraints for PFT presence and regenerativeness

    !   Call this even when DGVM is not activated so that "adapted" and "regenerate"
    !   are kept up to date for the moment when the DGVM is activated.
    CALL constraints (npts, dt_days, &
         t2m_month, t2m_min_daily,when_growthinit, Tseason, &
         adapted, regenerate)

    
  !! 3. Determine introduction and elimination of PTS based on climate criteria
 
    IF ( ok_dgvm ) THEN

       !DSG mass conservation ========================================
       mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3)
      
       !! 3.1 Calculate introduction and elimination
       CALL pftinout (npts, dt_days, adapted, regenerate, &
            neighbours, veget_max, &
            biomass, ind, cn_ind, age, leaf_frac, npp_longterm, lm_lastyearmax, senescence, &
            PFTpresent, everywhere, when_growthinit, need_adjacent, RIP_time, &
            co2_to_bm, n_to_bm, p_to_bm, &
            avail_tree, avail_grass, &
!gmjc
            sla_calc)
!end gmjc
 
       IF (dsg_debug) THEN
          CALL  check_mass(npts,biomass(:,:,:,:), 'lpj: after pftinout')
          !DSG mass conservation ============================================
          WRITE (numout,*) 'MASS CONSERVATON: pftinout is designed to violate against mass conservation'
          WRITE (numout,*) 'but it should be not detectable according to comments'
          mass_change(:,:,icarbon)     = zero
          mass_change(:,:,initrogen)   = zero
          mass_change(:,:,iphosphorus) = zero
          CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
                 mass_before(:,:,:),              &  ! mass before
                 SUM(biomass(:,:,:,:),DIM=3),     &  ! mass after
                 mass_change(:,:,:),              &  ! net of fluxes
                 'lpj: after pftinout' )
       ENDIF

       !! 3.2 Reset attributes for eliminated PFTs.
       !     This also kills PFTs that had 0 leafmass during the last year. The message
       !     "... after pftinout" is misleading in this case.


       !DSG mass conservation ========================================
       mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3)

       CALL kill (npts, 'pftinout  ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, npp_longterm, &
            when_growthinit, everywhere, veget_max, bm_to_litter)

       IF(dsg_debug) THEN
          CALL  check_mass(npts,biomass(:,:,:,:), 'lpj: after kill')
          !DSG mass conservation ============================================
          mass_change(:,:,icarbon)     = SUM(bm_to_litter(:,:,:,icarbon),DIM=3)
          mass_change(:,:,initrogen)   = SUM(bm_to_litter(:,:,:,initrogen),DIM=3)
          mass_change(:,:,iphosphorus) = SUM(bm_to_litter(:,:,:,iphosphorus),DIM=3)
          CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
                 mass_before(:,:,:),               &  ! mass before
                 SUM(biomass(:,:,:,:),DIM=3),      &  ! mass after
                 mass_change(:,:,:),               &  ! net of fluxes
                 'lpj: after kill' )
       ENDIF
       
       !! 3.3 Calculate woodmass of individual tree
       IF(ok_dgvm) THEN
          WHERE ((ind(:,:).GT.min_stomate))
             woodmass_ind(:,:) = &
                  ((biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))*veget_max(:,:))/ind(:,:)
          ENDWHERE
       ELSE
          WHERE ((ind(:,:).GT.min_stomate))
             woodmass_ind(:,:) =(biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))/ind(:,:)
          ENDWHERE
       ENDIF
       
       ! Calculate crown area and diameter for all PFTs (including the newly established)
       CALL crown (npts, PFTpresent, &
            ind, biomass, woodmass_ind, &
            veget_max, cn_ind, height)

    ENDIF
    
  !! 4. Phenology

    !! 4.1 Write values to history file
    !      Current values for ::when_growthinit 
    CALL xios_orchidee_send_field("WHEN_GROWTHINIT",when_growthinit)

    CALL histwrite_p (hist_id_stomate, 'WHEN_GROWTHINIT', itime, when_growthinit, npts*nvm, horipft_index)

    ! Set and write values for ::PFTpresent
    WHERE(PFTpresent)
       histvar=un
    ELSEWHERE
       histvar=zero
    ENDWHERE

    CALL xios_orchidee_send_field("PFTPRESENT",histvar)

    CALL histwrite_p (hist_id_stomate, 'PFTPRESENT', itime, histvar, npts*nvm, horipft_index)

    ! Set and write values for gdd_midwinter
    WHERE(gdd_midwinter.EQ.undef)
       histvar=val_exp
    ELSEWHERE
       histvar=gdd_midwinter
    ENDWHERE

    CALL xios_orchidee_send_field("GDD_MIDWINTER",histvar)

    CALL histwrite_p (hist_id_stomate, 'GDD_MIDWINTER', itime, histvar, npts*nvm, horipft_index)

    ! Set and write values for gdd_m5_dormance
    WHERE(gdd_m5_dormance.EQ.undef)
       histvar=val_exp
    ELSEWHERE
       histvar=gdd_m5_dormance
    ENDWHERE
    
    CALL xios_orchidee_send_field('GDD_M5_DORMANCE',histvar)
    CALL histwrite_p (hist_id_stomate, 'GDD_M5_DORMANCE', itime, histvar, npts*nvm, horipft_index)

    ! Set and write values for ncd_dormance
    WHERE(ncd_dormance.EQ.undef)
       histvar=val_exp
    ELSEWHERE
       histvar=ncd_dormance
    ENDWHERE

    CALL xios_orchidee_send_field("NCD_DORMANCE",histvar)

    CALL histwrite_p (hist_id_stomate, 'NCD_DORMANCE', itime, histvar, npts*nvm, horipft_index)
!JC Debug
    IF (dsg_debug) THEN
                WRITE(numout,*) 'before phenology '
                WRITE(numout,*) 'biomass(icarbon)',biomass(1,10,:,icarbon)
                WRITE(numout,*) 'biomass(initrogen)',biomass(1,10,:,initrogen)
                WRITE(numout,*) 'biomass(iphosphorus)',biomass(1,10,:,iphosphorus)
                WRITE(numout,*) 'N_support',N_support(1,10)
                WRITE(numout,*) 'P_support',P_support(1,10)
    ENDIF
!End JC Debug
    !DSG mass conservation ========================================
    mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3)

       IF (printlev>=3) THEN
          WRITE(numout,*) 'stomateLPJ: before phenology_prognostic'
          WRITE(numout,*) 'biomass(test_grid, test_pft,:,icarbon)',biomass(test_grid, test_pft,:,icarbon)
          WRITE(numout,*) 'biomass(test_grid, test_pft,:,initrogen)',biomass(test_grid, test_pft,:,initrogen)
          WRITE(numout,*) 'biomass(test_grid, test_pft,:,iphosphorus)',biomass(test_grid, test_pft,:,iphosphorus)

          WRITE(numout,*) 'co2_to_bm(test_grid, test_pft)',co2_to_bm(test_grid, test_pft)
          WRITE(numout,*) 'n_to_bm(test_grid, test_pft)',n_to_bm(test_grid, test_pft)
          WRITE(numout,*) 'p_to_bm(test_grid, test_pft)',p_to_bm(test_grid, test_pft)
       ENDIF
      

    !! 4.2 Calculate phenology
    CALL phenology (npts, dt_days, PFTpresent, &
         veget_max, &
         t2m_longterm, t2m_month, t2m_week, gpp_daily, &
         maxmoiavail_lastyear, minmoiavail_lastyear, &
         moiavail_month, moiavail_week, &
         gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
         senescence, time_hum_min, &
         biomass, leaf_frac, leaf_age, &
         when_growthinit, co2_to_bm, n_to_bm, p_to_bm, &
         begin_leaves,ind,KF,N_support, P_support, &
!DSG_cb01
         cn_leaf_min_season, np_leaf_min_season, &
!DSG_cb01
!gmjc
         sla_calc)
!end gmjc
    !DSG: the variable senescene is not used in phenology
    
    IF (dsg_debug) THEN
       !DSG debug: ===================================================
       WRITE(6,*) 'location in code: '
       WRITE(6,*) 'stomateLPJ: after phenology_prognostic'

       IF (printlev>=3) THEN
          WRITE(numout,*) 'stomateLPJ: after phenology_prognostic'
          WRITE(numout,*) 'biomass(test_grid, test_pft,:,icarbon)',biomass(test_grid, test_pft,:,icarbon)
          WRITE(numout,*) 'biomass(test_grid, test_pft,:,initrogen)',biomass(test_grid, test_pft,:,initrogen)
          WRITE(numout,*) 'biomass(test_grid, test_pft,:,iphosphorus)',biomass(test_grid, test_pft,:,iphosphorus)

          WRITE(numout,*) 'co2_to_bm(test_grid, test_pft)',co2_to_bm(test_grid, test_pft)
          WRITE(numout,*) 'n_to_bm(test_grid, test_pft)',n_to_bm(test_grid, test_pft)
          WRITE(numout,*) 'p_to_bm(test_grid, test_pft)',p_to_bm(test_grid, test_pft)
       ENDIF
      
       CALL  check_mass(npts,biomass(:,:,:,:), 'lpj: after phenology_prognostic')
       WRITE (6,*)'no mass conservation check after phenology: as the routine is designed to violate mass'
      !DSG as we sometimes have the case 'There is leaf or root carbon that
      !should not be here, something could have gone' see stomate_phenology.f90
!DSG    !DSG mass conservation ============================================
!DSG    mass_change(:,:,icarbon)     = zero
!DSG    mass_change(:,:,initrogen)   = zero
!DSG    mass_change(:,:,iphosphorus) = zero
!DSG    CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
!DSG           mass_before(:,:,:),               &  ! mass before
!DSG           SUM(biomass(:,:,:,:),DIM=3),      &  ! mass after
!DSG           mass_change(:,:,:)                &  ! net of fluxes
!DSG           )


       IF(printlev>=3)THEN
           WRITE(numout,*)    'biomass(test_grid,test_pft,:,icarbon)',biomass(test_grid,test_pft,:,icarbon)
           WRITE(numout,*)    'biomass(test_grid,test_pft,ilabile,initrogen)',biomass(test_grid,test_pft,ilabile,initrogen)
           WRITE(numout,*)    'gpp_daily(test_grid,test_pft)' ,gpp_daily(test_grid,test_pft)
           WRITE(numout,*)    'npp_daily(test_grid,test_pft)' ,npp_daily(test_grid,test_pft)
           WRITE(numout,*)    'resp_maint(test_grid,test_pft)',resp_maint(test_grid,test_pft)
           WRITE(numout,*)    'resp_growth(test_grid,test_pft)',resp_growth(test_grid,test_pft)
           WRITE(numout,*)    'dt_days',dt_days
        ENDIF

     !DSG mass conservation ========================================

!DSG: I had to revert JC_Debug MOD007
! 1) we need to have separate mass conservation checks for phenology and
! growth_fun_all for all elements, if not, debugging gets to complicated
! 2) if impose_CN or impose_NP mass conservation checks should be disabled; only
! then N_support, P_support >0
! 3) phenology violates mass conservation by design thus a check makes nosense

!DSG !JC Debug MOOD007
!DSG ! mass_before should be calculated before phenology 
!DSG ! because when BEGIN_LEAVES=TRUE, there could be N_support, P_support > 0
!DSG ! and biomass change
     mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3)
!JC MOD038 new crop strategy, there could be N_support and P_support > 0 in phenology
!and allocated to biomass, however, it should not be accounted as mass change
!in growth_fun_all, so, we need to minus it as mass_before
     mass_before(:,:,initrogen) = mass_before(:,:,initrogen) - N_support
     mass_before(:,:,iphosphorus) = mass_before(:,:,iphosphorus) - P_support
!DSG ! HOWEVER, for icarbon, the mass_before should be calculated here,
!DSG ! because the mass_change for icarbon is defined as NPP without accounting for
!DSG ! bm_use
!DSG !     mass_before(:,:,icarbon) = SUM(biomass(:,:,:,icarbon),DIM=3)
!DSG !End JC Debug
    ENDIF
!DSG: revert JC_Debug MOD007
 
! JC MOD024 calculate GRM_devstage
     CALL calc_GRM_devstage(npts, t2m_daily, t2m_week, tsurf_daily, begin_leaves, GRM_devstage)
! End JC MOD024
!   WRITE(numout,*) 'before fun age',leaf_age
!   WRITE(numout,*) 'before fun frac',leaf_frac

     CALL growth_fun_all (npts, dt_days, veget_max, veget, PFTpresent, &
       senescence, when_growthinit, t2m_week, &
       nstress_season, pstress_season,moiavail_growingseason, &
       gpp_daily, resp_maint_part, resp_maint, &
!DSGlabfac
       gpp_week, &
!DSGlabfac
       resp_growth, npp_daily, biomass, age, &
       leaf_age, leaf_frac, use_reserve, &
       ind, rue_longterm, KF, k_latosa_adapt, &
       cn_leaf_min_season, n_uptake_daily, N_support, &
       np_leaf_min_season, p_uptake_daily, P_support, &
!DSGlaitarget
       lai_target, lai_target_longterm, &
!DSGlaitarget
!JC Debug MOD005
       record_reserve_grass, &
!End JC Debug MOD005
!gmjc
       sla_calc, sla_age1, BNF_clover_daily, &
       GRM_devstage, when_growthinit_cut)
!end gmjc
!JC Debug
!   WRITE(numout,*) 'after fun age',leaf_age
!   WRITE(numout,*) 'after fun frac',leaf_frac
!                WRITE(numout,*) 'after growth_fun_all'
!                WRITE(numout,*) 'biomass(icarbon)',biomass(1,10,:,icarbon)
!                WRITE(numout,*) 'biomass(initrogen)',biomass(1,10,:,initrogen)
!                WRITE(numout,*) 'biomass(iphosphorus)',biomass(1,10,:,iphosphorus)
!                WRITE(numout,*) 'N_support',N_support(1,10)
!                WRITE(numout,*) 'P_support',P_support(1,10)
!                WRITE(numout,*) 'N_uptake',n_uptake_daily(1,10,:)
!                WRITE(numout,*) 'P_uptake',p_uptake_daily(1,10)
!                WRITE(numout,*) 'BNF_clover_daily',BNF_clover_daily(1,10)
!End JC Debug
     IF(dsg_debug) THEN 
        !DSG debug: ===================================================
        IF(PRINTLEV>=3) THEN
             WRITE(numout,*) 'location in code: '
             WRITE(numout,*) 'stomateLPJ: after growth_fun'
             WRITE(numout,*)    'SUM(Biomass(test_grid,test_pft,:,icarbon))',  SUM(biomass(test_grid,test_pft,:,icarbon))
             WRITE(numout,*)    'SUM(Biomass(test_grid,test_pft,:,initrogen))',SUM(biomass(test_grid,test_pft,:,initrogen))
             WRITE(numout,*)    'biomass(test_grid,test_pft,:,icarbon)',biomass(test_grid,test_pft,:,icarbon)
             WRITE(numout,*)    'biomass(test_grid,test_pft,:,initrogen)',biomass(test_grid,test_pft,:,initrogen)
             WRITE(numout,*)    'gpp_daily(test_grid,test_pft)' ,gpp_daily(test_grid,test_pft)
             WRITE(numout,*)    'npp_daily(test_grid,test_pft)' ,npp_daily(test_grid,test_pft)
             WRITE(numout,*)    'resp_maint(test_grid,test_pft)',resp_maint(test_grid,test_pft)
             WRITE(numout,*)    'resp_growth(test_grid,test_pft)',resp_growth(test_grid,test_pft)
             WRITE(numout,*)    'SUM(n_uptake_daily(test_grid,test_pft,:))',SUM(n_uptake_daily(test_grid,test_pft,:))
             WRITE(numout,*)    'n_support(test_grid,test_pft)',N_support(test_grid,test_pft)
             WRITE(numout,*)    'BNF_clover_daily(test_grid,test_pft)',BNF_clover_daily(test_grid,test_pft)
             WRITE(numout,*)    'dt_days',dt_days
        ENDIF 
        CALL check_mass(npts,biomass(:,:,:,:), 'lpj: after growth_fun')

        !DSG mass conservation ============================================
        mass_change(:,:,icarbon)     = npp_daily(:,:)*dt_days
        IF (impose_cn) THEN
           mass_change(:,:,initrogen)   = SUM(n_uptake_daily(:,:,:),DIM=3) + N_support(:,:) + &
              BNF_clover_daily(:,:)
        ELSE
! JCADD 20170427 BNF for grassland
!           mass_change(:,:,initrogen)   = SUM(n_uptake_daily(:,:,:),DIM=3)
           mass_change(:,:,initrogen)   = SUM(n_uptake_daily(:,:,:),DIM=3) + &
               BNF_clover_daily(:,:)
! END JCADD
        ENDIF

        IF (impose_cn) THEN
            mass_change(:,:,iphosphorus) = p_uptake_daily(:,:)  + P_support(:,:)
        ELSE
            mass_change(:,:,iphosphorus) = p_uptake_daily(:,:) 
        ENDIF
!JC Debug
                WRITE(numout,*) 'mass_before',mass_before(1,10,:)
                WRITE(numout,*) 'mass_after',SUM(biomass(1,10,:,:),DIM=1)
                WRITE(numout,*) 'mass_change',mass_change(1,10,:)

!End JC Debug

        CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
              mass_before(:,:,:),               &  ! mass before
              SUM(biomass(:,:,:,:),DIM=3),      &  ! mass afte
              mass_change(:,:,:),               &  ! net of fluxes
              'lpj: after growth_fun'  )
     ENDIF

! START: this output is redundant as all pools are in standard output, but it is
! convinent to have the important variable as output
      IF (ok_pcycle) THEN

          ! calculate the N:P ratio of labile plant tissue for diagnosis
           WHERE(( biomass(:,:,ilabile,iphosphorus) + biomass(:,:,icarbres,iphosphorus) &
                     + biomass(:,:,ileaf,iphosphorus)).GT. min_stomate)   

                     plant_np(:,:) = ( biomass(:,:,ilabile,initrogen) &
                                     + biomass(:,:,icarbres,initrogen)+ biomass(:,:,ileaf,initrogen)) / &
                              ( biomass(:,:,ilabile,iphosphorus)+     &
                                biomass(:,:,icarbres,iphosphorus)+ biomass(:,:,ileaf,iphosphorus))  
           ELSEWHERE
              plant_np(:,:) = undef
           ENDWHERE
          CALL xios_orchidee_send_field('PLANT_NP',plant_np(:,:))
          CALL histwrite_p (hist_id_stomate, 'PLANT_NP', itime, &
              plant_np(:,:) , npts*nvm, horipft_index)
      ENDIF


      IF (ok_ncycle) THEN
         ! calculate the C:N ratio of labile plant tissue for diagnosis
          WHERE(( biomass(:,:,ilabile,initrogen) + biomass(:,:,icarbres,initrogen) &
                    + biomass(:,:,ileaf,initrogen)).GT. min_stomate)   

                    plant_cn(:,:) = ( biomass(:,:,ilabile,icarbon) &
                                    + biomass(:,:,icarbres,icarbon)+ biomass(:,:,ileaf,icarbon)) / &
                             ( biomass(:,:,ilabile,initrogen)+     &
                               biomass(:,:,icarbres,initrogen)+ biomass(:,:,ileaf,initrogen))  
          ELSEWHERE
             plant_cn(:,:) = undef
          ENDWHERE
         CALL xios_orchidee_send_field('PLANT_CN',plant_cn(:,:))
         CALL histwrite_p (hist_id_stomate, 'PLANT_CN', itime, &
             plant_cn(:,:) , npts*nvm, horipft_index)
      ENDIF

! END:this output is redundant as all pools are in standard output, but it is
! convinent to have the important variable as output


  !! 6. NPP, maintenance and growth respiration

    !! 6.1 Calculate NPP and respiration terms
!    CALL npp_calc (npts, dt_days, &
!         PFTpresent, &
!         t2m_daily, tsoil_daily, lai, rprof, &
!         gpp_daily, f_alloc, bm_alloc, resp_maint_part,&
!         biomass, leaf_age, leaf_frac, age, &
!         resp_maint, resp_growth, npp_daily)

    !! 6.2 Kill slow growing PFTs in DGVM or STOMATE with constant mortality
    IF ( ok_dgvm .OR. .NOT.lpj_gap_const_mort) THEN

       CALL kill (npts, 'npp       ', lm_lastyearmax,  &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, npp_longterm, &
            when_growthinit, everywhere, veget_max, bm_to_litter)

       !! 6.2.1 Update wood biomass      
       !        For the DGVM
       IF(ok_dgvm) THEN
          WHERE (ind(:,:).GT.min_stomate)
             woodmass_ind(:,:) = &
                  ((biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon)) & 
                  *veget_max(:,:))/ind(:,:)
          ENDWHERE

       ! For all pixels with individuals
       ELSE
          WHERE (ind(:,:).GT.min_stomate)
             woodmass_ind(:,:) = &
                  (biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
                  + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))/ind(:,:)
          ENDWHERE
       ENDIF ! ok_dgvm

       !! 6.2.2 New crown area and maximum vegetation cover after growth
       CALL crown (npts, PFTpresent, &
            ind, biomass, woodmass_ind,&
            veget_max, cn_ind, height)

    ENDIF ! ok_dgvm

!DSGbalance: BUG AREA start
    
  !! 7. fire
!DSG: not possible as we burn also litter    !DSG mass conservation ========================================
!DSG: not possible as we burn also litter    mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3) & !DSG: checked
!DSG: not possible as we burn also litter            WRITE(6,*) 'before fire'
!DSG: not possible as we burn also litter            WRITE(6,*) 'bm_to_litter(test_grid,test_pft,:,icarbon)',bm_to_litter(test_grid,test_pft,:,icarbon)

    !! 7.1. Burn PFTs
    CALL fire (npts, dt_days, &
         litterhum_daily, t2m_daily, lignin_struc, lignin_wood, veget_max, &
         fireindex, firelitter, biomass, ind, &
         litter, dead_leaves, bm_to_litter, &
         co2_fire, nox_emission,MatrixA)
!gmjc
! after fire burning
  litter_avail(:,:,:) = litter(:,:,:,iabove,icarbon) * &
            litter_avail_frac(:,:,:)
  litter_not_avail(:,:,:) = litter(:,:,:,iabove,icarbon) * &
            (1.0 - litter_avail_frac(:,:,:))
!end gmjc

    !! 7.2 Kill PFTs in DGVM
    IF ( ok_dgvm ) THEN
       ! reset attributes for eliminated PFTs
       CALL kill (npts, 'fire      ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, npp_longterm, &
            when_growthinit, everywhere, veget_max, bm_to_litter)
    ENDIF ! ok_dgvm

    IF (dsg_debug) THEN
       WRITE(6,*) 'location in code: '
       WRITE(6,*) 'stomateLPJ: after fire'
       IF(PRINTLEV>=3) THEN
            WRITE(6,*)    'biomass(test_grid,test_pft,:,icarbon)',biomass(test_grid,test_pft,:,icarbon)
            WRITE(6,*)    'biomass(test_grid,test_pft,:,initrogen)',biomass(test_grid,test_pft,:,initrogen)
            WRITE(6,*)    'bm_to_litter(test_grid,test_pft,:,icarbon)',bm_to_litter(test_grid,test_pft,:,icarbon)
            WRITE(6,*)    'bm_to_litter(test_grid,test_pft,:,initrogen)',bm_to_litter(test_grid,test_pft,:,initrogen)
       ENDIF
!DSG as we burn litter    WRITE(6,*) 'bm_to_litter(test_grid,test_pft,:,icarbon)',bm_to_litter(test_grid,test_pft,:,icarbon)
!DSG as we burn litter    WRITE(6,*) 'co2_fire(test_grid,test_pft*dt_daysi=',co2_fire(test_grid,test_pft)*dt_days
!DSG as we burn litter    WRITE(6,*) 'biomass(test_grid,test_pft,:,icarbon)',biomass(test_grid,test_pft,:,icarbon)
       CALL check_mass(npts,biomass(:,:,:,:), 'lpj: after fire')
!DSG as we burn litter          !DSG mass conservation ============================================
!DSG as we burn litter          mass_change(:,:,icarbon)     = -(SUM(bm_to_litter(:,:,:,icarbon),DIM=3) + co2_fire(:,:)*dt_days)
!DSG as we burn litter          mass_change(:,:,initrogen)   = -(SUM(bm_to_litter(:,:,:,initrogen),DIM=3) + nox_emission(:,:)*dt_days)
!DSG as we burn litter          mass_change(:,:,iphosphorus) = -SUM(bm_to_litter(:,:,:,iphosphorus),DIM=3)
!DSG as we burn litter
!DSG as we burn litter          CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
!DSG as we burn litter                mass_before(:,:,:),               &  ! mass before
!DSG as we burn litter                SUM(biomass(:,:,:,:),DIM=3),      &  ! mass afte
!DSG as we burn litter                mass_change(:,:,:)                &  ! net of fluxes
!DSG as we burn litter                )
    ENDIF

  !! 8. Tree mortality
    !DSG mass conservation ========================================
    mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3) + SUM(bm_to_litter(:,:,:,:),DIM=3)

    !IF (dsg_debug) THEN
    !    WRITE(6,*) 'location in code: '
    !    WRITE(6,*) 'stomateLPJ: before gap'
    !    IF(PRINTLEV>=3) THEN
    !    !    WRITE(6,*) 'bm_to_litter(test_grid,test_pft,:,icarbon)',bm_to_litter(test_grid,test_pft,:,icarbon)
    !        WRITE(6,*)'mass_before(test_grid,test_pft,icarbon)',mass_before(test_grid,test_pft,icarbon)
    !        WRITE(6,*)'mass_before(test_grid,test_pft,initrogen)',mass_before(test_grid,test_pft,initrogen)
    !    !    WRITE(6,*) 'bm_to_litter(test_grid,test_pft,:,initrogen)',bm_to_litter(test_grid,test_pft,:,initrogen)
    !    !    WRITE(6,*)'biomass(test_grid,test_pft,:,initrogen)',biomass(test_grid,test_pft,:,initrogen)
    !    !    WRITE(6,*) 'bm_to_litter(test_grid,test_pft,:,iphosphorus)',bm_to_litter(test_grid,test_pft,:,iphosphorus)
    !    !    WRITE(6,*)'biomass(test_grid,test_pft,:,iphosphorus',biomass(test_grid,test_pft,:,iphosphorus)
    !    ENDIF
    !ENDIF

    ! Does not depend on age, therefore does not change crown area.
    CALL gap (npts, dt_days, &
         npp_longterm, turnover_longterm, lm_lastyearmax, &
         PFTpresent, t2m_min_daily, Tmin_spring_time, &
         biomass, ind, bm_to_litter, mortality, &
!gmjc
         sla_calc)
!end gmjc)

    IF ( ok_dgvm ) THEN

       ! reset attributes for eliminated PFTs
       CALL kill (npts, 'gap       ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, npp_longterm, &
            when_growthinit, everywhere, veget_max, bm_to_litter)

    ENDIF

    IF(dsg_debug) THEN
       !DSG debug: ===================================================
       IF(PRINTLEV>=3) THEN
            WRITE(numout,*) 'location in code: '
            WRITE(numout,*) 'stomateLPJ: after gap n kill (1)'
            WRITE(numout,*) 'test_pft',test_pft
            WRITE(numout,*) 'test_grid',test_grid
            WRITE(numout,*) 'biomass(test_grid,test_pft,:,icarbon)',biomass(test_grid,test_pft,:,icarbon)
            WRITE(numout,*) 'biomass(test_grid,test_pft,:,initrogen)',biomass(test_grid,test_pft,:,initrogen)
            WRITE(numout,*) 'bm_to_litter(test_grid,test_pft,:,icarbon)',bm_to_litter(test_grid,test_pft,:,icarbon)
            WRITE(numout,*) 'bm_to_litter(test_grid,test_pft,:,initrogen)',bm_to_litter(test_grid,test_pft,:,initrogen)
       ENDIF
       CALL check_mass(npts,biomass(:,:,:,:), 'lpj: after gap n kill')

       !DSG mass conservation ============================================
       mass_change(:,:,icarbon)     = -SUM(bm_to_litter(:,:,:,icarbon),DIM=3)
       mass_change(:,:,initrogen)   = -SUM(bm_to_litter(:,:,:,initrogen),DIM=3)
       mass_change(:,:,iphosphorus) = -SUM(bm_to_litter(:,:,:,iphosphorus),DIM=3)

       CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
             mass_before(:,:,:),               &  ! mass before
             SUM(biomass(:,:,:,:),DIM=3),      &  ! mass afte
             mass_change(:,:,:),               &  ! net of fluxes
             'lpj: after gap n kill')
     ENDIF


!! =====================================
!! DSG: mass conservation issue: START

  !! 10. Leaf senescence, new lai and other turnover processes
    !DSG mass conservation ========================================
    mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3)  + SUM(bm_to_litter(:,:,:,:),DIM=3)
       WRITE(6,*) 'location in code: '

    CALL turn (npts, dt_days, PFTpresent, &
         herbivores, &
         maxmoiavail_lastyear, minmoiavail_lastyear, &
         moiavail_week,  moiavail_month,t2m_longterm, t2m_month, t2m_week, veget_max, &
         gdd_from_growthinit, leaf_age, leaf_frac, age, lai, biomass, &
         turnover_daily, recycling_daily, senescence,turnover_time, &
!gmjc
         sla_calc,enable_grazing,when_growthinit)
!end gmjc

    !! 10. Light competition
    
    !! If not using constant mortality then kill with light competition
!    IF ( ok_dgvm .OR. .NOT.(lpj_gap_const_mort) ) THEN
    IF ( ok_dgvm ) THEN
 
       !! 10.1 Light competition
       CALL light (npts, dt_days, &
            veget_max, fpc_max, PFTpresent, cn_ind, lai, maxfpc_lastyear, &
            lm_lastyearmax, ind, biomass, veget_lastlight, bm_to_litter, mortality, &
!gmjc
         sla_calc)
!end gmjc
       
       !! 10.2 Reset attributes for eliminated PFTs
       CALL kill (npts, 'light     ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, npp_longterm, &
            when_growthinit, everywhere, veget_max, bm_to_litter)

    ENDIF
    IF(dsg_debug) THEN
       !DSG debug: ===================================================
       IF(PRINTLEV>=3) THEN
          WRITE(numout,*) 'location in code: '
          WRITE(numout,*) 'stomateLPJ: after turn'
          WRITE(numout,*) 'bm_to_litter(test_grid,test_pft,:,icarbon)',bm_to_litter(test_grid,test_pft,:,icarbon)
          WRITE(numout,*)'biomass(test_grid,test_pft,:,icarbon)',biomass(test_grid,test_pft,:,icarbon)
          WRITE(numout,*) 'bm_to_litter(test_grid,test_pft,:,initrogen)',bm_to_litter(test_grid,test_pft,:,initrogen)
          WRITE(numout,*)'biomass(test_grid,test_pft,:,initrogen)',biomass(test_grid,test_pft,:,initrogen)
          WRITE(numout,*) 'bm_to_litter(test_grid,test_pft,:,iphosphorus)',bm_to_litter(test_grid,test_pft,:,iphosphorus)
          WRITE(numout,*)'biomass(test_grid,test_pft,:,iphosphorus',biomass(test_grid,test_pft,:,iphosphorus)
       ENDIF
       CALL  check_mass(npts,biomass(:,:,:,:), 'lpj: after turn')

       !DSG mass conservation ============================================
       mass_change(:,:,icarbon)     = -SUM(turnover_daily(:,:,:,icarbon),DIM=3) &
                                      -SUM(bm_to_litter(:,:,:,icarbon),DIM=3)  ! in case ok_dvgm=true
       mass_change(:,:,initrogen)   = -SUM(turnover_daily(:,:,:,initrogen),DIM=3) &
                                      -SUM(bm_to_litter(:,:,:,initrogen),DIM=3)
       mass_change(:,:,iphosphorus) = -SUM(turnover_daily(:,:,:,iphosphorus),DIM=3) &
                                      -SUM(bm_to_litter(:,:,:,iphosphorus),DIM=3)

       CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
             mass_before(:,:,:),               &  ! mass before
             SUM(biomass(:,:,:,:),DIM=3),      &  ! mass afte
             mass_change(:,:,:),               &  ! net of fluxes
             'lpj: after turn')
    ENDIF

!! DSG: mass conservation issue: END
!! =====================================

!DSGbalance: BUG AREA end
    
  !! 11. Establishment of saplings
    
    IF ( ok_dgvm .OR. .NOT.lpj_gap_const_mort ) THEN
       !DSG mass conservation ========================================
       mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3) + SUM(bm_to_litter(:,:,:,:),DIM=3)

       !! 11.1 Establish new plants
       CALL establish (npts, dt_days, PFTpresent, regenerate, &
            neighbours, resolution, need_adjacent, herbivores, &
            precip_lastyear, gdd0_lastyear, lm_lastyearmax, &
            cn_ind, lai, avail_tree, avail_grass, npp_longterm, &
            leaf_age, leaf_frac, &
            ind, biomass, age, everywhere, co2_to_bm, &
            soil_n_min, n_uptake_daily, nstress_season, pstress_season,veget_max, woodmass_ind, &
            mortality, bm_to_litter, &
!gmjc
         sla_calc)
!end gmjc

       IF (printlev>=3) WRITE (numout,*) 'after establish soil_n_min(test_grid,test_pft,:):',soil_n_min(test_grid,test_pft,:)
       !! 12.2 Calculate new crown area (and maximum vegetation cover)
       CALL crown (npts, PFTpresent, &
            ind, biomass, woodmass_ind, &
            veget_max, cn_ind, height)

       IF(dsg_debug) THEN
          !DSG debug: ===================================================
          CALL  check_mass(npts,biomass(:,:,:,:), 'lpj: after establish')

          !DSG mass conservation ============================================
          mass_change(:,:,icarbon)     = -SUM(bm_to_litter(:,:,:,icarbon),DIM=3)
          mass_change(:,:,initrogen)   = -SUM(bm_to_litter(:,:,:,initrogen),DIM=3)
          mass_change(:,:,iphosphorus) = -SUM(bm_to_litter(:,:,:,iphosphorus),DIM=3)

          CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
                mass_before(:,:,:),               &  ! mass before
                SUM(biomass(:,:,:,:),DIM=3),      &  ! mass afte
                mass_change(:,:,:),               &  ! net of fluxes
                'lpj: after establish')
       ENDIF

    ENDIF

!JC add NPinput
    IF (allow_agri_fert) THEN
     mass_before(:,:,:) = SUM(SUM(litter(:,:,:,:,:),DIM=4),DIM=2)
       
      CALL apply_agri_fert(npts,dt_days,day_of_year, &
             agri_fert_save,litter,agri_nfert,agri_pfert,fert_masschangecnp)  
       IF(dsg_debug) THEN
          !DSG debug: ===================================================

          CALL  check_mass(npts,biomass(:,:,:,:), 'stomateLPJ: after apply_agri_fert')

          !DSG mass conservation ============================================
          !JC NOTE: only solid organic fertilizer is applied to litter
          ! mineral fertilizer and the urine N in organic fertilizer
          ! is output through agri_nfert and agri_pfert
          ! and will be applied directly to soil mineral N P pools next day
          mass_change(:,:,icarbon)     = fert_masschangecnp(:,:,icarbon)
          mass_change(:,:,initrogen)   = fert_masschangecnp(:,:,initrogen)
          mass_change(:,:,iphosphorus) = fert_masschangecnp(:,:,iphosphorus)

          CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
                mass_before(:,:,:),               &  ! mass before
                SUM(SUM(litter(:,:,:,:,:),DIM=4),DIM=2),      &  ! mass afte
                mass_change(:,:,:),                &  ! net of fluxes
                'lpj: after apply_agri_fert')
       ENDIF

    ENDIF 
!End JC add NPinput

!gmjc Grassland_management
     mass_before(:,:,:) = SUM(biomass(:,:,:,:),DIM=3) + &
                                SUM(bm_to_litter(:,:,:,:),DIM=3) + &
                                SUM(SUM(litter(:,:,:,:,:),DIM=4),DIM=2)
    !
    ! 13 calculate grazing by animals or cutting for forage
    !
    IF (enable_grazing) THEN
        CALL main_grassland_management(&
           npts, lalo, neighbours, resolution, contfrac, &
           dt_days        , &
           day_of_year    , &
           t2m_daily      , &
           t2m_min_daily  , &
           t2m_14         , &
           tsurf_daily    , &
           snowfall_daily , &
           biomass        , &
           bm_to_litter   , &
           litter         , &
           litter_avail   , &
           litter_not_avail , &
!           !spitfire
!           fuel_1hr(:,:,:,icarbon), &
!           fuel_10hr(:,:,:,icarbon), &
!           fuel_100hr(:,:,:,icarbon), &
!           fuel_1000hr(:,:,:,icarbon), &
!           !end spitfire
           .TRUE.         , &
           EndOfYear    , &
           when_growthinit_cut, nb_grazingdays, &
           lai,sla_calc,leaf_age,leaf_frac, &
           wshtotsum,sr_ugb,compt_ugb, &
           nb_ani,grazed_frac,import_yield, &
           moiavail_daily,tmc_topgrass_daily,fc_grazing, snowmass_daily, &
           after_snow, after_wet, wet1day, wet2day, &
           harvest_gm, ranimal_gm, ch4_pft_gm, cinput_gm, n2o_pft_gm, &
    ! JCADD 20170421 for CNP merge
           grm_nfert, grm_pfert, &
           grm_masschangecnp,GRM_devstage,when_growthinit)
    ! END JCADD

    ENDIF
       IF(dsg_debug) THEN
          !DSG debug: ===================================================
          CALL  check_mass(npts,biomass(:,:,:,:), 'stomateLPJ: after main_grassland_management')

          !DSG mass conservation ============================================
          ! JC NOTED 
          ! the grassland management has included many fluxes from or to
          ! environment (such as fertilization input, harvested biomass output,
          ! milk output, liveweight output, animal respiration output,
          ! and enteric fermentation ch4 emission)
          ! HOWEVER, it should be noted that grm_nfert (including N in urinecnp) 
          ! and grm_ndep are N fluxes
          ! grm_pfert (including N in urinecnp) are P fluxes
          ! that will directly goes into soil in the next time step, thus they
          ! are not included in the mass balance calculation
          mass_change(:,:,icarbon)     = grm_masschangecnp(:,:,icarbon)
          mass_change(:,:,initrogen)   = grm_masschangecnp(:,:,initrogen)
          mass_change(:,:,iphosphorus) = grm_masschangecnp(:,:,iphosphorus)

          CALL cons_mass(npts, nvm, nelements,    &  ! dimensions
                mass_before(:,:,:),               &  ! mass before
                SUM(biomass(:,:,:,:),DIM=3) + &
                SUM(bm_to_litter(:,:,:,:),DIM=3) + &
                SUM(SUM(litter(:,:,:,:,:),DIM=4),DIM=2),      &  ! mass afte
                mass_change(:,:,:)                &  ! net of fluxes
                )
       ENDIF
!end gmjc


  !! 13. Calculate final LAI and vegetation cover
    
    CALL cover (npts, cn_ind, ind, biomass, &
         veget_max, veget_max_tmp, lai, &
         litter, &
!gmjc
       litter_avail, litter_not_avail, &
!end gmjc
         som, turnover_daily, bm_to_litter, &
         co2_to_bm, co2_fire, resp_hetero, resp_maint, resp_growth, gpp_daily, &
         lignin_struc, lignin_wood, soil_n_min)


    IF (printlev>=3) WRITE (numout,*) 'after cover soil_n_min(test_grid,test_pft,:):',soil_n_min(test_grid,test_pft,:)
  !! 14. Update litter pools to account for harvest
 
    ! the whole litter stuff:
    !    litter update, lignin content, PFT parts, litter decay, 
    !    litter heterotrophic respiration, dead leaf soil cover.
    !    No vertical discretisation in the soil for litter decay.\n
    ! added by shilong for harvest
    IF(harvest_agri) THEN
       CALL harvest(npts, dt_days, veget_max, &
            bm_to_litter, turnover_daily, &
            harvest_above)
    ENDIF

    !! 14. Land cover change if it is time to do so
    !! The flag do_now_lcchange is set in slowproc_main at the same time as the vegetation is read from file.
    !! The vegetation fractions are not updated yet and will be updated in the end of sechiba_main.
    IF (do_now_stomate_lcchange) THEN
       CALL lcchange_main (npts, dt_days, veget_max, veget_max_new, &
            biomass, ind, age, PFTpresent, senescence, when_growthinit, everywhere, &
            co2_to_bm, bm_to_litter, turnover_daily, bm_sapl, cn_ind,flux10,flux100, &
            prod10,prod100,convflux,cflux_prod10,cflux_prod100,nflux_prod_total,leaf_frac,&
            npp_longterm, lm_lastyearmax, litter,  &
!gmjc
       litter_avail, litter_not_avail, &
!end gmjc
       som, soil_n_min, KF, k_latosa_adapt,rue_longterm, &
            lignin_struc, lignin_wood)
       do_now_stomate_lcchange=.FALSE.

       ! Set the flag done_stomate_lcchange to be used in the end of sechiba_main to update the fractions.
       done_stomate_lcchange=.TRUE.
    ENDIF

     IF (printlev>=3) WRITE (numout,*) 'after lcchange soil_n_min(test_grid,test_pft,:):',soil_n_min(test_grid,test_pft,:)

    !! 15. Calculate vcmax 

    CALL vmax (npts, dt_days, &
         leaf_age, leaf_frac, &
         vcmax, nue)

    !MM déplacement pour initialisation correcte des grandeurs cumulées :
    cflux_prod_total(:) = convflux(:) + cflux_prod10(:) + cflux_prod100(:)
    prod10_total(:)=SUM(prod10,dim=2)
    prod100_total(:)=SUM(prod100,dim=2)
    
  !! 16. Total heterotrophic respiration

    tot_soil_carb(:,:) = zero
    tot_litter_carb(:,:) = zero
    DO j=2,nvm

       tot_litter_carb(:,j) = tot_litter_carb(:,j) + (litter(:,istructural,j,iabove,icarbon) + &
            &          litter(:,imetabolic,j,iabove,icarbon) + litter(:,iwoody,j,iabove,icarbon) + &
            &          litter(:,istructural,j,ibelow,icarbon) + litter(:,imetabolic,j,ibelow,icarbon)+ &
            &          litter(:,iwoody,j,ibelow,icarbon))

       tot_soil_carb(:,j) = tot_soil_carb(:,j) + (som(:,iactive,j,icarbon) + &
            &          som(:,islow,j,icarbon)+  som(:,ipassive,j,icarbon)+  som(:,isurface,j,icarbon))

    ENDDO
    tot_litter_soil_carb(:,:) = tot_litter_carb(:,:) + tot_soil_carb(:,:)

!!$     DO k = 1, nelements ! Loop over # elements
!!$        tot_live_biomass(:,:,k) = biomass(:,:,ileaf,k) + biomass(:,:,isapabove,k) + biomass(:,:,isapbelow,k) +&
!!$             &                    biomass(:,:,iheartabove,k) + biomass(:,:,iheartbelow,k) + &
!!$             &                    biomass(:,:,iroot,k) + biomass(:,:,ifruit,k) + biomass(:,:,icarbres,k)
!!$    END DO ! Loop over # elements

    tot_live_biomass(:,:,:) = biomass(:,:,ileaf,:) + biomass(:,:,isapabove,:) + biomass(:,:,isapbelow,:) +&
             &                    biomass(:,:,iheartabove,:) + biomass(:,:,iheartbelow,:) + &
             &                    biomass(:,:,iroot,:) + biomass(:,:,ifruit,:) + biomass(:,:,icarbres,:) + biomass(:,:,ilabile,:)


    tot_turnover(:,:,:) = turnover_daily(:,:,ileaf,:) + turnover_daily(:,:,isapabove,:) + &
         &         turnover_daily(:,:,isapbelow,:) + turnover_daily(:,:,iheartabove,:) + &
         &         turnover_daily(:,:,iheartbelow,:) + turnover_daily(:,:,iroot,:) + &
         &         turnover_daily(:,:,ifruit,:) + turnover_daily(:,:,icarbres,:) + turnover_daily(:,:,ilabile,:) 

    tot_bm_to_litter(:,:,:) = bm_to_litter(:,:,ileaf,:) + bm_to_litter(:,:,isapabove,:) +&
         &             bm_to_litter(:,:,isapbelow,:) + bm_to_litter(:,:,iheartbelow,:) +&
         &             bm_to_litter(:,:,iheartabove,:) + bm_to_litter(:,:,iroot,:) + &
         &             bm_to_litter(:,:,ifruit,:) + bm_to_litter(:,:,icarbres,:) + bm_to_litter(:,:,ilabile,:)

    carb_mass_variation(:)=-carb_mass_total(:)
    carb_mass_total(:)=SUM((tot_live_biomass(:,:,icarbon)+tot_litter_carb+tot_soil_carb)*veget_max,dim=2) + &
         &                 (prod10_total + prod100_total)
    carb_mass_variation(:)=carb_mass_total(:)+carb_mass_variation(:)
    
  !! 17. Write history

    CALL xios_orchidee_send_field("RESOLUTION_X",resolution(:,1))
    CALL xios_orchidee_send_field("RESOLUTION_Y",resolution(:,2))
    CALL xios_orchidee_send_field("CONTFRAC_STOMATE",contfrac(:))
    CALL xios_orchidee_send_field("T2M_MONTH",t2m_month)
    CALL xios_orchidee_send_field("T2M_WEEK",t2m_week)
    CALL xios_orchidee_send_field("TSEASON",Tseason)
    CALL xios_orchidee_send_field("TMIN_SPRING_TIME", Tmin_spring_time)
    CALL xios_orchidee_send_field("ONSET_DATE",onset_date)
    CALL xios_orchidee_send_field("FPC_MAX",fpc_max)
    CALL xios_orchidee_send_field("MAXFPC_LASTYEAR",maxfpc_lastyear)
    CALL xios_orchidee_send_field("HET_RESP",resp_hetero(:,:))
    CALL xios_orchidee_send_field("CO2_FIRE",co2_fire)
    CALL xios_orchidee_send_field("CO2_TAKEN",co2_to_bm)
    IF(ok_ncycle) CALL xios_orchidee_send_field("N_TAKEN",n_to_bm)
    IF(ok_pcycle) CALL xios_orchidee_send_field("P_TAKEN",p_to_bm)
    CALL xios_orchidee_send_field("LAI",lai)
    CALL xios_orchidee_send_field("VEGET_MAX",veget_max)
    CALL xios_orchidee_send_field("NPP_STOMATE",npp_daily)
    CALL xios_orchidee_send_field("GPP",gpp_daily)
    CALL xios_orchidee_send_field("IND",ind)
    CALL xios_orchidee_send_field("CN_IND",cn_ind)
    CALL xios_orchidee_send_field("WOODMASS_IND",woodmass_ind)
    CALL xios_orchidee_send_field("MOISTRESS",moiavail_week)
    CALL xios_orchidee_send_field("MAINT_RESP",resp_maint)
    CALL xios_orchidee_send_field("GROWTH_RESP",resp_growth)

    DO l=1,nelements 
       IF     (l == icarbon) THEN 
          element_str(l) = '_c' 
       ELSEIF (l == initrogen) THEN 
          element_str(l) = '_n' 
       ELSEIF (l == iphosphorus) THEN 
          element_str(l) = '_p' 
       ELSE 
          STOP 'Define element_str' 
       ENDIF 
      
       CALL xios_orchidee_send_field("TOTAL_M"//TRIM(element_str(l)),tot_live_biomass(:,:,l))
       CALL xios_orchidee_send_field("LEAF_M"//TRIM(element_str(l)),biomass(:,:,ileaf,l))
       CALL xios_orchidee_send_field("SAP_M_AB"//TRIM(element_str(l)),biomass(:,:,isapabove,l))
       CALL xios_orchidee_send_field("SAP_M_BE"//TRIM(element_str(l)),biomass(:,:,isapbelow,l))
       CALL xios_orchidee_send_field("HEART_M_AB"//TRIM(element_str(l)),biomass(:,:,iheartabove,l))
       CALL xios_orchidee_send_field("HEART_M_BE"//TRIM(element_str(l)),biomass(:,:,iheartbelow,l))
       CALL xios_orchidee_send_field("ROOT_M"//TRIM(element_str(l)),biomass(:,:,iroot,l))
       CALL xios_orchidee_send_field("FRUIT_M"//TRIM(element_str(l)),biomass(:,:,ifruit,l))
       CALL xios_orchidee_send_field("LABILE_M"//TRIM(element_str(l)),biomass(:,:,ilabile,l))
       CALL xios_orchidee_send_field("RESERVE_M"//TRIM(element_str(l)),biomass(:,:,icarbres,l))
       CALL xios_orchidee_send_field("TOTAL_TURN"//TRIM(element_str(l)),tot_turnover(:,:,l))
       CALL xios_orchidee_send_field("LEAF_TURN"//TRIM(element_str(l)),turnover_daily(:,:,ileaf,l))
       CALL xios_orchidee_send_field("SAP_AB_TURN"//TRIM(element_str(l)),turnover_daily(:,:,isapabove,l))
       CALL xios_orchidee_send_field("ROOT_TURN"//TRIM(element_str(l)),turnover_daily(:,:,iroot,l))
       CALL xios_orchidee_send_field("FRUIT_TURN"//TRIM(element_str(l)),turnover_daily(:,:,ifruit,l))
       IF  ( (l == initrogen).OR. (l == iphosphorus) ) THEN
          CALL xios_orchidee_send_field("LEAF_RECYCLE"//TRIM(element_str(l)),recycling_daily(:,:,ileaf,l))
          CALL xios_orchidee_send_field("ROOT_RECYCLE"//TRIM(element_str(l)),recycling_daily(:,:,iroot,l))
       ENDIF
!DSG: this is not per day, but per timestep!!!
       CALL xios_orchidee_send_field("TOTAL_BM_LITTER"//TRIM(element_str(l)),tot_bm_to_litter(:,:,l))
       CALL xios_orchidee_send_field("LEAF_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,ileaf,l))
       CALL xios_orchidee_send_field("SAP_AB_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,isapabove,l))
       CALL xios_orchidee_send_field("SAP_BE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,isapbelow,l))
       CALL xios_orchidee_send_field("HEART_AB_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,iheartabove,l))
       CALL xios_orchidee_send_field("HEART_BE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,iheartbelow,l))
       CALL xios_orchidee_send_field("ROOT_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,iroot,l))
       CALL xios_orchidee_send_field("FRUIT_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,ifruit,l))
       CALL xios_orchidee_send_field("LABILE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,ilabile,l))
       CALL xios_orchidee_send_field("RESERVE_BM_LITTER"//TRIM(element_str(l)),bm_to_litter(:,:,icarbres,l))
!DSG: this is not per day!!!
       CALL xios_orchidee_send_field("LITTER_STR_AB"//TRIM(element_str(l)),litter(:,istructural,:,iabove,l))
       CALL xios_orchidee_send_field("LITTER_MET_AB"//TRIM(element_str(l)),litter(:,imetabolic,:,iabove,l))
       CALL xios_orchidee_send_field("LITTER_WOD_AB"//TRIM(element_str(l)),litter(:,iwoody,:,iabove,l))
       CALL xios_orchidee_send_field("LITTER_STR_BE"//TRIM(element_str(l)),litter(:,istructural,:,ibelow,l))
       CALL xios_orchidee_send_field("LITTER_MET_BE"//TRIM(element_str(l)),litter(:,imetabolic,:,ibelow,l))
       CALL xios_orchidee_send_field("LITTER_WOD_BE"//TRIM(element_str(l)),litter(:,iwoody,:,ibelow,l))
       CALL xios_orchidee_send_field("SOIL_ACTIVE"//TRIM(element_str(l)),som(:,iactive,:,l))
       CALL xios_orchidee_send_field("SOIL_SLOW"//TRIM(element_str(l)),som(:,islow,:,l))
       CALL xios_orchidee_send_field("SOIL_PASSIVE"//TRIM(element_str(l)),som(:,ipassive,:,l))
       CALL xios_orchidee_send_field("SOIL_SURF"//TRIM(element_str(l)),som(:,isurface,:,l))

    ENDDO

    CALL xios_orchidee_send_field("DEADLEAF_COVER",deadleaf_cover)
    CALL xios_orchidee_send_field("TOTAL_SOIL_CARB",tot_litter_soil_carb)

    CALL xios_orchidee_send_field("LITTERHUM",litterhum_daily)
    CALL xios_orchidee_send_field("TURNOVER_TIME",turnover_time)
    CALL xios_orchidee_send_field("PROD10",prod10)
    CALL xios_orchidee_send_field("FLUX10",flux10)
    CALL xios_orchidee_send_field("PROD100",prod100)
    CALL xios_orchidee_send_field("FLUX100",flux100)
    CALL xios_orchidee_send_field("CONVFLUX",convflux)
    CALL xios_orchidee_send_field("CFLUX_PROD10",cflux_prod10)
    CALL xios_orchidee_send_field("CFLUX_PROD100",cflux_prod100)
    CALL xios_orchidee_send_field("HARVEST_ABOVE",harvest_above)
    CALL xios_orchidee_send_field("GRM_DEVSTAGE",GRM_devstage)
   !DSG vcmax_new=zero
    ! DSG: vcmax new is the nitrogen derived vcmax (which is used instead of
    ! vcmax25)
    ! DSG: it is based on Kattge et al (2009)
    DO j=2,nvm
           !DSG avoid division by zero; not quite sure how LAI>zero and leaf C =zero can occur
           ! - but it does. This might indicate something went wrong earlier.
       WHERE((lai(:,j) .GT. min_stomate) &
          !DSGadded
             .AND.(biomass(:,j,ileaf,icarbon).GT. zero) )
          !DSGadded
           ! use the top leaf N concentration
           vcmax_new(:,j) = nue(:,j)*biomass(:,j,ileaf,initrogen) * ext_coeff_N(j)  &
                          / ( 1. - exp(-ext_coeff_N(j) * sla_calc(:,j)*biomass(:,j,ileaf,icarbon)) ) 
       ELSEWHERE
           vcmax_new(:,j) = zero
       ENDWHERE
    ENDDO
    
    ! DSG: vcmax_walker is the nitrogen and phosphorus derived vcmax 
    !      based on Walker et al (2014) (as well as Kattge et al. (2009)
    ! for very low leaf N concentration where the Walker equation is going crazy)
    DO j=2,nvm
           !DSG avoid division by zero; not quite sure how LAI>zero and leaf C =zero can occur
           ! - but it does. This might indicate something went wrong earlier
       WHERE((lai(:,j) .GT. min_stomate) &
             .AND.(biomass(:,j,ileaf,icarbon).GT. zero) )

           ! use the top leaf N concentration
           leafN_dummy(:,j) = biomass(:,j,ileaf,initrogen) * ext_coeff_N(j)  &
                              / ( 1. - exp(-ext_coeff_N(j) *sla_calc(:,j)*biomass(:,j,ileaf,icarbon)) )
           !DSGdebug ! assuming constant N:P troughout the canopy; get the top leaf P concentration
           !DSGdebug leafP_dummy(:,j) = biomass(:,j,ileaf,iphosphorus) * ext_coeff_N(j)  &
           !DSGdebug                    / ( 1. - exp(-ext_coeff_N(j) *sla(j)*biomass(:,j,ileaf,icarbon)) )

           ! get top leaf P concentration of top leaf from top leaf N concetration assumgin constant N:P
           leafP_dummy(:,j) =leafN_dummy(:,j)*(biomass(:,j,ileaf,iphosphorus)/biomass(:,j,ileaf,initrogen))
       ELSEWHERE 
           leafN_dummy(:,j) = zero
           leafP_dummy(:,j) = zero
       ENDWHERE

       WHERE(leafN_dummy(:,j) .GT. 0.5)  ! below that the Walker equation is not well defined
           vcmax_walker(:,j) = exp(3.946 + 0.921*log(leafN_dummy(:,j)) + 0.121*log(leafP_dummy(:,j)) + 0.282*log(leafN_dummy(:,j))*log(leafP_dummy(:,j)))
       ELSEWHERE  ! not defined, we will use the Kattge NUE as N is the main driver
           vcmax_walker(:,j) = zero
       ENDWHERE
    ENDDO

    !DSG: vcmax_domingues is the nitrogen, phosphorus and SLA derived VCMAX
    !     here the Vcmax is most sensitive to SLA, P derived Vcmax tends to be
    !     lower than N derived Vcmax, overall the resulting Vcmax is much more
    !     closer to Vcmax observation than the Kattge or Walker approach which
    !     heavily overestimate Vcmax  (with the C:N:P ratio we use here)
    !     Parameter values and equation see Mercado et al. (2011)

    DO j=2,nvm
       WHERE((lai(:,j) .GT. min_stomate) &
             .AND.(biomass(:,j,ileaf,icarbon).GT. zero) )
!DSGdebug
           ! first we get the top leaf N concentration assuming a constant sla
           ! throughout the canopy
           leafN_dummy(:,j) = biomass(:,j,ileaf,initrogen) * ext_coeff_N(j)  &
                              / ( 1. - exp(-ext_coeff_N(j) *sla_calc(:,j)*biomass(:,j,ileaf,icarbon)) )

           ! then we get the mg(N)/DW(leaf) 
           leafN_dummy(:,j) = leafN_dummy(:,j) * (sla_calc(:,j)*sla_conv) *1E3
!DSGdebug

           ! get top leaf P concentration of top leaf from top leaf N concetration assumgin constant N:P
           leafP_dummy(:,j) =leafN_dummy(:,j)*(biomass(:,j,ileaf,iphosphorus)/biomass(:,j,ileaf,initrogen))
       ELSEWHERE  
           leafN_dummy(:,j) = zero
           leafP_dummy(:,j) = zero
       ENDWHERE

      !DSG WRITE(6,*) 'leafN_dummy(:,j)             =', leafN_dummy(:,j)
      !DSG WRITE(6,*) 'sla(j)                       =', sla(j)
      !DSG WRITE(6,*) ' biomass(:,j,ileaf,iphosphorus)  =', biomass(:,j,ileaf,iphosphorus)
      !DSG WRITE(6,*) ' biomass(:,j,ileaf,initrogen)=', biomass(:,j,ileaf,initrogen)
      !DSG WRITE(6,*) 'leafP_dummy(:,j)             =', leafP_dummy(:,j)

       WHERE(leafN_dummy(:,j) .GT. zero)  
           ! Vcmax [ umol /g(DW) /s] ; sla in cm2/g instead of m2/g
           vcmax_domingues(:,j) = MIN( 10**(.430*LOG10(leafN_dummy(:,j)) + .368*LOG10(sla_calc(:,j)*sla_conv*1E4) -1.559),  & 
                                       10**(.453*LOG10(leafP_dummy(:,j)) + .25 *LOG10(sla_calc(:,j)*sla_conv*1E4) -.798))     

           !DSG: vcmax_domingues is the nitrogen, phosphorus and SLA derived VCMAX
           ! adjust units to ORCHIDEE standard [umol /m2 /s]
           vcmax_domingues(:,j) = vcmax_domingues(:,j)/(sla_calc(:,j)*sla_conv)

       ELSEWHERE  ! not defined, we will use the Kattge NUE as N is the main driver
           vcmax_domingues(:,j) = zero
       ENDWHERE

    ENDDO


    CALL xios_orchidee_send_field("VCMAX",vcmax)
    CALL xios_orchidee_send_field("VCMAX_KATTGE",vcmax_new)
!DSG    CALL xios_orchidee_send_field("VCMAX_WALKER",vcmax_walker)
!DSG    CALL xios_orchidee_send_field("VCMAX_DOMINGUES",vcmax_domingues)

    CALL xios_orchidee_send_field("AGE",age)
    CALL xios_orchidee_send_field("HEIGHT",height)
    CALL xios_orchidee_send_field("FIREINDEX",fireindex(:,:))
!gmjc
    CALL xios_orchidee_send_field("T2M_14",t2m_14(:))
    CALL xios_orchidee_send_field("LITTER_STR_AVAIL",litter_avail(:,istructural,:))
    CALL xios_orchidee_send_field("LITTER_MET_AVAIL",litter_avail(:,imetabolic,:))
    CALL xios_orchidee_send_field("LITTER_STR_NAVAIL",litter_not_avail(:,istructural,:))
    CALL xios_orchidee_send_field("LITTER_MET_NAVAIL",litter_not_avail(:,imetabolic,:))
    CALL xios_orchidee_send_field("LITTER_STR_AVAILF",litter_avail_frac(:,istructural,:))
    CALL xios_orchidee_send_field("LITTER_MET_AVAILF",litter_avail_frac(:,imetabolic,:))
    !calculate grassland co2 fluxes
    DO j=2,nvm
      IF ((.NOT. is_tree(j)) .AND. natural(j)) THEN
        veget_max_gm(:,j) = veget_max(:,j)
      ENDIF
    END DO ! nvm
    veget_exist_gm(:) = SUM(veget_max_gm,dim=2)
    WHERE (veget_exist_gm(:) .GT. 0.0)
      co2_gm(:) = SUM((gpp_daily-(resp_maint+resp_growth+resp_hetero)-co2_fire &
                    -harvest_gm-ranimal_gm-ch4_pft_gm+cinput_gm) &
                    *veget_max_gm,dim=2)/veget_exist_gm
      ch4_gm(:) = SUM(ch4_pft_gm*veget_max_gm,dim=2)/veget_exist_gm
      n2o_gm(:) = SUM(n2o_pft_gm*veget_max_gm,dim=2)/veget_exist_gm
    ELSEWHERE
      co2_gm(:) = zero
      ch4_gm(:) = zero
      n2o_gm(:) = zero
    ENDWHERE
    CALL xios_orchidee_send_field("CO2_GM",co2_gm)
    CALL xios_orchidee_send_field("CH4_GM",ch4_gm)
    CALL xios_orchidee_send_field("N2O_GM",n2o_gm)
    CALL xios_orchidee_send_field("N2O_PFT_GM",n2o_pft_gm)
!end gmjc

! ipcc history
     CALL xios_orchidee_send_field("cVeg",SUM(tot_live_biomass(:,:,icarbon)*veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cLitter",SUM(tot_litter_carb*veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cSoil",SUM(tot_soil_carb*veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cProduct",(prod10_total + prod100_total)/1e3)
     CALL xios_orchidee_send_field("cMassVariation",carb_mass_variation/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("lai_ipcc",SUM(lai*veget_max,dim=2)*contfrac)
     CALL xios_orchidee_send_field("gpp_ipcc",SUM(gpp_daily*veget_max,dim=2)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("ra",SUM((resp_maint+resp_growth)*veget_max,dim=2)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("npp_ipcc",SUM(npp_daily*veget_max,dim=2)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("rh",SUM(resp_hetero*veget_max,dim=2)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("fFire",SUM(co2_fire*veget_max,dim=2)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("fHarvest",harvest_above/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("fLuc",cflux_prod_total/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("nbp",(SUM((gpp_daily-(resp_maint+resp_growth+resp_hetero)-co2_fire) &
            &        *veget_max,dim=2)-cflux_prod_total-harvest_above)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("fVegLitter",SUM((tot_bm_to_litter(:,:,icarbon) + tot_turnover(:,:,icarbon))*&
          veget_max,dim=2)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("fLitterSoil",SUM(SUM(som_input(:,:,:,icarbon),dim=2)*veget_max,dim=2)/1e3/one_day)
     CALL xios_orchidee_send_field("cLeaf",SUM(biomass(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cWood",SUM((biomass(:,:,isapabove,icarbon)+biomass(:,:,iheartabove,icarbon))*&
          veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cRoot",SUM(( biomass(:,:,iroot,icarbon) + biomass(:,:,isapbelow,icarbon) + &
          biomass(:,:,iheartbelow,icarbon) )*veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cMisc",SUM(( biomass(:,:,icarbres,icarbon) + biomass(:,:,ifruit,icarbon))*&
          veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cLitterAbove",SUM((litter(:,istructural,:,iabove,icarbon)+&
          litter(:,imetabolic,:,iabove,icarbon))*veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cLitterBelow",SUM((litter(:,istructural,:,ibelow,icarbon)+&
          litter(:,imetabolic,:,ibelow,icarbon))*veget_max,dim=2)/1e3*contfrac)
     CALL xios_orchidee_send_field("cSoilFast",SUM((som(:,iactive,:,icarbon)+som(:,isurface,:,icarbon))*veget_max,dim=2)/1e3)
     CALL xios_orchidee_send_field("cSoilMedium",SUM(som(:,islow,:,icarbon)*veget_max,dim=2)/1e3)
     CALL xios_orchidee_send_field("cSoilSlow",SUM(som(:,ipassive,:,icarbon)*veget_max,dim=2)/1e3)
       DO j=1,nvm
          histvar(:,j)=veget_max(:,j)*contfrac(:)*100
       ENDDO
     CALL xios_orchidee_send_field("landCoverFrac",histvar)
       vartmp(:)=zero
       DO j = 2,nvm
          IF (is_deciduous(j)) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
     CALL xios_orchidee_send_field("treeFracPrimDec",vartmp)
       vartmp(:)=zero
       DO j = 2,nvm
          IF (is_evergreen(j)) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
     CALL xios_orchidee_send_field("treeFracPrimEver",vartmp)
       vartmp(:)=zero
       DO j = 2,nvm
          IF ( .NOT.(is_c4(j)) ) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
     CALL xios_orchidee_send_field("c3PftFrac",vartmp)
       vartmp(:)=zero
       DO j = 2,nvm
          IF ( is_c4(j) ) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
     CALL xios_orchidee_send_field("c4PftFrac",vartmp)
     CALL xios_orchidee_send_field("rGrowth",SUM(resp_growth*veget_max,dim=2)/1e3/one_day*contfrac)
     CALL xios_orchidee_send_field("rMaint",SUM(resp_maint*veget_max,dim=2)/1e3/one_day*contfrac)
!DSG: bm_alloc is not even set     CALL xios_orchidee_send_field("nppLeaf",SUM(bm_alloc(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac)
!DSG: bm_alloc is not even set     CALL xios_orchidee_send_field("nppWood",SUM(bm_alloc(:,:,isapabove,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac)
!DSG: bm_alloc is not even set     CALL xios_orchidee_send_field("nppRoot",SUM(( bm_alloc(:,:,isapbelow,icarbon) + bm_alloc(:,:,iroot,icarbon) )*&
!DSG:          veget_max,dim=2)/1e3/one_day*contfrac)     


    CALL histwrite_p (hist_id_stomate, 'RESOLUTION_X', itime, &
         resolution(:,1), npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'RESOLUTION_Y', itime, &
         resolution(:,2), npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'CONTFRAC', itime, &
         contfrac(:), npts, hori_index)

    CALL histwrite_p (hist_id_stomate, 'DEADLEAF_COVER', itime, &
         deadleaf_cover, npts, hori_index)

    CALL histwrite_p (hist_id_stomate, 'TOTAL_SOIL_CARB', itime, &
         tot_litter_soil_carb, npts*nvm, horipft_index)

    CALL histwrite_p (hist_id_stomate, 'T2M_MONTH', itime, &
         t2m_month, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'T2M_WEEK', itime, &
         t2m_week, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'TSEASON', itime, &
         Tseason, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'TMIN_SPRING_TIME', itime, &
         Tmin_spring_time, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'ONSET_DATE', itime, &
         onset_date(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'FPC_MAX', itime, &
         fpc_max, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'MAXFPC_LASTYEAR', itime, &
         maxfpc_lastyear, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HET_RESP', itime, &
         resp_hetero(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'FIREINDEX', itime, &
         fireindex(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTERHUM', itime, &
         litterhum_daily, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'CO2_FIRE', itime, &
         co2_fire, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'CO2_TAKEN', itime, &
         co2_to_bm, npts*nvm, horipft_index)
    IF(ok_ncycle) CALL histwrite_p (hist_id_stomate, 'N_TAKEN', itime, &
         n_to_bm, npts*nvm, horipft_index)
    IF(ok_pcycle) CALL histwrite_p (hist_id_stomate, 'P_TAKEN', itime, &
         p_to_bm, npts*nvm, horipft_index)
    ! land cover change
    CALL histwrite_p (hist_id_stomate, 'CONVFLUX', itime, &
         convflux, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD10', itime, &
         cflux_prod10, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD100', itime, &
         cflux_prod100, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'HARVEST_ABOVE', itime, &
         harvest_above, npts, hori_index)
    CALL histwrite_p (hist_id_stomate, 'GRM_DEVSTAGE', itime, &
         GRM_devstage, npts*nvm, horipft_index)

    CALL histwrite_p (hist_id_stomate, 'LAI', itime, &
         lai, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'VEGET_MAX', itime, &
         veget_max, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NPP', itime, &
         npp_daily, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'GPP', itime, &
         gpp_daily, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'IND', itime, &
         ind, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'CN_IND', itime, &
         cn_ind, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'WOODMASS_IND', itime, &
         woodmass_ind, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'MAINT_RESP', itime, &
         resp_maint, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'GROWTH_RESP', itime, &
         resp_growth, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'AGE', itime, &
         age, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HEIGHT', itime, &
         height, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'MOISTRESS', itime, &
         moiavail_week, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'VCMAX', itime, &
         vcmax, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'VCMAX_KATTGE', itime, &
         vcmax_new, npts*nvm, horipft_index)
    !CALL histwrite_p (hist_id_stomate, 'VCMAX_WALKER', itime, &
    !     vcmax_walker, npts*nvm, horipft_index)
    !CALL histwrite_p (hist_id_stomate, 'VCMAX_DOMINGUES', itime, &
    !     vcmax_domingues, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME', itime, &
         turnover_time, npts*nvm, horipft_index)
    ! land cover change
    CALL histwrite_p (hist_id_stomate, 'PROD10', itime, &
         prod10, npts*11, horip11_index)
    CALL histwrite_p (hist_id_stomate, 'PROD100', itime, &
         prod100, npts*101, horip101_index)
    CALL histwrite_p (hist_id_stomate, 'FLUX10', itime, &
         flux10, npts*10, horip10_index)
    CALL histwrite_p (hist_id_stomate, 'FLUX100', itime, &
         flux100, npts*100, horip100_index)
! gmjc
    CALL histwrite_p(hist_id_stomate ,'T2M_14'   ,itime, &
         t2m_14, npts, hori_index)
!    CALL histwrite (hist_id_stomate, 'LITTER_RESP', itime, &
!         resp_hetero_litter_d(:,:), npts*nvm, horipft_index)
!    CALL histwrite (hist_id_stomate, 'ACTIVE_RESP', itime, &
!         resp_hetero_soil_d(:,iactive,:), npts*nvm, horipft_index)
!    CALL histwrite (hist_id_stomate, 'SLOW_RESP', itime, &
!         resp_hetero_soil_d(:,islow,:), npts*nvm, horipft_index)
!    CALL histwrite (hist_id_stomate, 'PASSIVE_RESP', itime, &
!         resp_hetero_soil_d(:,ipassive,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTER_STR_AVAIL', itime, &
         litter_avail(:,istructural,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTER_MET_AVAIL', itime, &
         litter_avail(:,imetabolic,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTER_STR_NAVAIL', itime, &
         litter_not_avail(:,istructural,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTER_MET_NAVAIL', itime, &
         litter_not_avail(:,imetabolic,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTER_STR_AVAILF', itime, &
         litter_avail_frac(:,istructural,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'LITTER_MET_AVAILF', itime, &
         litter_avail_frac(:,imetabolic,:), npts*nvm, horipft_index)
! end gmjc

    DO l=1,nelements 
       IF     (l == icarbon) THEN 
          element_str(l) = '_c' 
       ELSEIF (l == initrogen) THEN 
          element_str(l) = '_n' 
       ELSEIF (l == iphosphorus) THEN 
          element_str(l) = '_p' 
       ELSE 
          STOP 'Define element_str' 
       ENDIF 
      
       CALL histwrite_p (hist_id_stomate, 'LITTER_STR_AB'//TRIM(element_str(l)), itime, & 
            litter(:,istructural,:,iabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LITTER_MET_AB'//TRIM(element_str(l)), itime, & 
            litter(:,imetabolic,:,iabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LITTER_STR_BE'//TRIM(element_str(l)), itime, & 
            litter(:,istructural,:,ibelow,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LITTER_MET_BE'//TRIM(element_str(l)), itime, & 
            litter(:,imetabolic,:,ibelow,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_AB'//TRIM(element_str(l)), itime, & 
            litter(:,iwoody,:,iabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_BE'//TRIM(element_str(l)), itime, & 
            litter(:,iwoody,:,ibelow,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SOIL_ACTIVE'//TRIM(element_str(l)), itime, & 
            som(:,iactive,:,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SOIL_SLOW'//TRIM(element_str(l)), itime, & 
            som(:,islow,:,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SOIL_PASSIVE'//TRIM(element_str(l)), itime, & 
            som(:,ipassive,:,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SOIL_SURF'//TRIM(element_str(l)), itime, & 
            som(:,isurface,:,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'TOTAL_M'//TRIM(element_str(l)), itime, & 
            tot_live_biomass(:,:,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LEAF_M'//TRIM(element_str(l)), itime, & 
            biomass(:,:,ileaf,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SAP_M_AB'//TRIM(element_str(l)), itime, & 
            biomass(:,:,isapabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SAP_M_BE'//TRIM(element_str(l)), itime, & 
            biomass(:,:,isapbelow,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'HEART_M_AB'//TRIM(element_str(l)), itime, & 
            biomass(:,:,iheartabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'HEART_M_BE'//TRIM(element_str(l)), itime, & 
            biomass(:,:,iheartbelow,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'ROOT_M'//TRIM(element_str(l)), itime, & 
            biomass(:,:,iroot,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'FRUIT_M'//TRIM(element_str(l)), itime, & 
            biomass(:,:,ifruit,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'RESERVE_M'//TRIM(element_str(l)), itime, & 
            biomass(:,:,icarbres,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LABILE_M'//TRIM(element_str(l)), itime, & 
            biomass(:,:,ilabile,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'TOTAL_TURN'//TRIM(element_str(l)), itime, & 
            tot_turnover(:,:,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LEAF_TURN'//TRIM(element_str(l)), itime, & 
            turnover_daily(:,:,ileaf,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SAP_AB_TURN'//TRIM(element_str(l)), itime, & 
            turnover_daily(:,:,isapabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'ROOT_TURN'//TRIM(element_str(l)), itime, & 
            turnover_daily(:,:,iroot,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'FRUIT_TURN'//TRIM(element_str(l)), itime, & 
            turnover_daily(:,:,ifruit,l), npts*nvm, horipft_index) 

       CALL histwrite_p (hist_id_stomate, 'LEAF_RECYCLE'//TRIM(element_str(l)), itime, & 
            recycling_daily(:,:,ileaf,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'ROOT_RECYCLE'//TRIM(element_str(l)), itime, & 
            recycling_daily(:,:,iroot,l), npts*nvm, horipft_index) 
!DSGbugfix: corrected dt-1 vs day-1
! from day to dt: /dt_sechiba*one_day
! not something went wrong!!
       CALL histwrite_p (hist_id_stomate, 'TOTAL_BM_LITTER'//TRIM(element_str(l)), itime, & 
            tot_bm_to_litter(:,:,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LEAF_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,ileaf,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SAP_AB_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,isapabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'SAP_BE_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,isapbelow,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'HEART_AB_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,iheartabove,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'HEART_BE_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,iheartbelow,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'ROOT_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,iroot,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'FRUIT_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,ifruit,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'RESERVE_BM_LITTER'//TRIM(element_str(l)), itime, & 
            bm_to_litter(:,:,icarbres,l), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LABILE_BM_LITTER'//TRIM(element_str(l)), itime, &
            bm_to_litter(:,:,ilabile,l), npts*nvm, horipft_index)
!DSGbugfix: corrected units
    ENDDO

    IF (impose_cn .AND. ok_ncycle) THEN
      CALL xios_orchidee_send_field('N_SUPPORT',N_support(:,:))
      CALL histwrite_p (hist_id_stomate, 'N_SUPPORT', itime, &
         N_support(:,:), npts*nvm, horipft_index)
    ENDIF

    IF (impose_np .AND. ok_pcycle) THEN
      CALL xios_orchidee_send_field('P_SUPPORT',P_support(:,:))
      CALL histwrite_p (hist_id_stomate, 'P_SUPPORT', itime, &
         P_support(:,:), npts*nvm, horipft_index)
    ENDIF

    IF ( hist_id_stomate_IPCC > 0 ) THEN
       vartmp(:)=SUM(tot_live_biomass(:,:,icarbon)*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cVeg", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(tot_litter_carb*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cLitter", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(tot_soil_carb*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cSoil", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=(prod10_total + prod100_total)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cProduct", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=carb_mass_variation/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "cMassVariation", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(lai*veget_max,dim=2)
       CALL histwrite_p (hist_id_stomate_IPCC, "lai", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(gpp_daily*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "gpp", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM((resp_maint+resp_growth)*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "ra", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(npp_daily*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "npp", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(resp_hetero*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "rh", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(co2_fire*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "fFire", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=harvest_above/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "fHarvest", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=cflux_prod_total/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "fLuc", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=(SUM((gpp_daily+co2_to_bm-(resp_maint+resp_growth+resp_hetero)-co2_fire) &
            &        *veget_max,dim=2)-cflux_prod_total-harvest_above)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "nbp", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM((tot_bm_to_litter(:,:,icarbon) + tot_turnover(:,:,icarbon))*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "fVegLitter", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(SUM(som_input(:,:,:,icarbon),dim=2)*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "fLitterSoil", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(biomass(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cLeaf", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM((biomass(:,:,isapabove,icarbon)+biomass(:,:,iheartabove,icarbon))*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cWood", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(( biomass(:,:,iroot,icarbon) + biomass(:,:,isapbelow,icarbon) + biomass(:,:,iheartbelow,icarbon) ) &
            &        *veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cRoot", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(( biomass(:,:,icarbres,icarbon) +  biomass(:,:,ilabile,icarbon) + biomass(:,:,ifruit,icarbon))*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cMisc", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM((litter(:,istructural,:,iabove,icarbon)+litter(:,imetabolic,:,iabove,icarbon))*&
            veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cLitterAbove", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM((litter(:,istructural,:,ibelow,icarbon)+litter(:,imetabolic,:,ibelow,icarbon))*&
            veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cLitterBelow", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM((som(:,iactive,:,icarbon)+som(:,isurface,:,icarbon))*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cSoilFast", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(som(:,islow,:,icarbon)*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cSoilMedium", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(som(:,ipassive,:,icarbon)*veget_max,dim=2)/1e3
       CALL histwrite_p (hist_id_stomate_IPCC, "cSoilSlow", itime, &
            vartmp, npts, hori_index)
       DO j=1,nvm
          histvar(:,j)=veget_max(:,j)*100
       ENDDO
       CALL histwrite_p (hist_id_stomate_IPCC, "landCoverFrac", itime, &
            histvar, npts*nvm, horipft_index)
       !-
       vartmp(:)=zero
       DO j = 2,nvm
          IF (is_deciduous(j)) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*100
          ENDIF
       ENDDO
       CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimDec", itime, &
            vartmp, npts, hori_index)
       !-
       vartmp(:)=zero
       DO j = 2,nvm
          IF (is_evergreen(j)) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*100
          ENDIF
       ENDDO
       CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimEver", itime, &
            vartmp, npts, hori_index)
       !-
       vartmp(:)=zero
       DO j = 2,nvm
          IF ( .NOT.(is_c4(j)) ) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*100
          ENDIF
       ENDDO
       CALL histwrite_p (hist_id_stomate_IPCC, "c3PftFrac", itime, &
            vartmp, npts, hori_index)
       !-
       vartmp(:)=zero
       DO j = 2,nvm
          IF ( is_c4(j) ) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*100
          ENDIF
       ENDDO
       CALL histwrite_p (hist_id_stomate_IPCC, "c4PftFrac", itime, &
            vartmp, npts, hori_index)
       !-
       vartmp(:)=SUM(resp_growth*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "rGrowth", itime, &
            vartmp, npts, hori_index)
       vartmp(:)=SUM(resp_maint*veget_max,dim=2)/1e3/one_day
       CALL histwrite_p (hist_id_stomate_IPCC, "rMaint", itime, &
            vartmp, npts, hori_index)
       !DSG vartmp(:)=SUM(bm_alloc(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3/one_day
       !DSG CALL histwrite_p (hist_id_stomate_IPCC, "nppLeaf", itime, &
       !DSG      vartmp, npts, hori_index)
       !DSG vartmp(:)=SUM(bm_alloc(:,:,isapabove,icarbon)*veget_max,dim=2)/1e3/one_day
       !DSG CALL histwrite_p (hist_id_stomate_IPCC, "nppWood", itime, &
       !DSG      vartmp, npts, hori_index)
       !DSG vartmp(:)=SUM(( bm_alloc(:,:,isapbelow,icarbon) + bm_alloc(:,:,iroot,icarbon) )*veget_max,dim=2)/1e3/one_day
       !DSG CALL histwrite_p (hist_id_stomate_IPCC, "nppRoot", itime, &
       !DSG      vartmp, npts, hori_index)

       CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_X', itime, &
            resolution(:,1), npts, hori_index)
       CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_Y', itime, &
            resolution(:,2), npts, hori_index)
       CALL histwrite_p (hist_id_stomate_IPCC, 'CONTFRAC', itime, &
            contfrac(:), npts, hori_index)

    ENDIF

    IF (printlev>=3) WRITE (numout,*) 'soil_n_min(test_grid,test_pft,:):',soil_n_min(test_grid,test_pft,:)
    IF (printlev>=4) WRITE(numout,*) 'Leaving stomate_lpj'

  END SUBROUTINE StomateLpj


!! ================================================================================================================================
!! SUBROUTINE   : harvest
!!
!>\BRIEF        Harvest of croplands
!!
!! DESCRIPTION  : To take into account biomass harvest from crop (mainly to take 
!! into account for the reduced litter input and then decreased soil carbon. it is a 
!! constant (40\%) fraction of above ground biomass.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): ::harvest_above the harvested biomass
!!
!! REFERENCE(S) :
!! - Piao, S., P. Ciais, P. Friedlingstein, N. de Noblet-Ducoudre, P. Cadule, N. Viovy, and T. Wang. 2009. 
!!   Spatiotemporal patterns of terrestrial carbon cycle during the 20th century. Global Biogeochemical 
!!   Cycles 23:doi:10.1029/2008GB003339.
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE harvest(npts, dt_days, veget_max, &
       bm_to_litter, turnover_daily, &
       harvest_above)

  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER, INTENT(in)                                    :: npts            !! Domain size (unitless) 
    REAL(r_std), INTENT(in)                                :: dt_days         !! Time step (days)                               
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)           :: veget_max       !! new "maximal" coverage fraction of a PFT (LAI -> 
                                                                              !! infinity) on ground @tex $(m^2 m^{-2})$ @endtex 
    
   !! 0.2 Output variables
   
   !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(inout) :: bm_to_litter !! [DISPENSABLE] conversion of biomass to litter 
                                                                                     !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(inout) :: turnover_daily   !! Turnover rates 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts), INTENT(inout)            :: harvest_above    !! harvest above ground biomass for agriculture 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    !! 0.4 Local variables

    INTEGER(i_std)                                         :: i, j, k, l, m    !! indices                       
    REAL(r_std)                                            :: above_old        !! biomass of previous time step 
                                                                               !! @tex $(gC m^{-2})$ @endtex 
    CHARACTER(LEN=2), DIMENSION(nelements)                     :: element_str         !! string suffix indicating element
    REAL(r_std), DIMENSION(npts,nvm,nelements)                 :: harvestcnp
!_ ================================================================================================================================

  !! 1. Yearly initialisation

    above_old             = zero
    harvest_above         = zero
    harvestcnp            = zero

    DO i = 1, npts
       DO j = 1,nvm
          IF (.NOT. natural(j)) THEN
             above_old = turnover_daily(i,j,ileaf,icarbon) + turnover_daily(i,j,isapabove,icarbon) + &
                  &       turnover_daily(i,j,iheartabove,icarbon) + turnover_daily(i,j,ifruit,icarbon) + &
                  &       turnover_daily(i,j,icarbres,icarbon) + turnover_daily(i,j,ilabile,icarbon) + turnover_daily(i,j,isapbelow,icarbon) + &
                  &       turnover_daily(i,j,iheartbelow,icarbon) + turnover_daily(i,j,iroot,icarbon)
!JC MOD037 harvest for icarbon, initrogen and iphosphorus at the same time
! and record harvested cnp
             harvestcnp(i,j,:) = (un - frac_turnover_daily) * &
                 SUM(turnover_daily(i,j,:,:),DIM=1)

             turnover_daily(i,j,ileaf,:) = turnover_daily(i,j,ileaf,:)*frac_turnover_daily
             turnover_daily(i,j,isapabove,:) = turnover_daily(i,j,isapabove,:)*frac_turnover_daily
             turnover_daily(i,j,isapbelow,:) = turnover_daily(i,j,isapbelow,:)*frac_turnover_daily
             turnover_daily(i,j,iheartabove,:) = turnover_daily(i,j,iheartabove,:)*frac_turnover_daily
             turnover_daily(i,j,iheartbelow,:) = turnover_daily(i,j,iheartbelow,:)*frac_turnover_daily
             turnover_daily(i,j,iroot,:) = turnover_daily(i,j,iroot,:)*frac_turnover_daily
             turnover_daily(i,j,ifruit,:) = turnover_daily(i,j,ifruit,:)*frac_turnover_daily
             turnover_daily(i,j,icarbres,:) = turnover_daily(i,j,icarbres,:)*frac_turnover_daily
             turnover_daily(i,j,ilabile,:) = turnover_daily(i,j,ilabile,:)*frac_turnover_daily
             harvest_above(i)  = harvest_above(i) + veget_max(i,j) * above_old *(un - frac_turnover_daily)

          ENDIF
       ENDDO
    ENDDO


    DO l=1,nelements
       IF     (l == icarbon) THEN
          element_str(l) = '_c'
       ELSEIF (l == initrogen) THEN
          element_str(l) = '_n'
       ELSEIF (l == iphosphorus) THEN
          element_str(l) = '_p'
       ELSE
          STOP 'Define element_str'
       ENDIF
       CALL xios_orchidee_send_field("CROP_HARVEST"//TRIM(element_str(l)),harvestcnp(:,:,l))
       CALL histwrite_p (hist_id_stomate, 'CROP_HARVEST'//TRIM(element_str(l)), itime, &
            harvestcnp(:,:,l),npts*nvm, horipft_index)
    END DO

!!$    harvest_above = harvest_above
  END SUBROUTINE harvest

  SUBROUTINE calc_GRM_devstage(npts, &
               t2m_daily, t2m_week, &
               tsurf_daily, begin_leaves, GRM_devstage)

    INTEGER (i_std)                   , INTENT(in)  :: npts
    REAL(r_std), DIMENSION(npts), INTENT(in)  :: t2m_daily
    REAL(r_std), DIMENSION(npts), INTENT(in)  :: t2m_week
    REAL(r_std), DIMENSION(npts), INTENT(in)  :: tsurf_daily
    LOGICAL, DIMENSION(npts,nvm), INTENT(in) :: begin_leaves
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout) :: GRM_devstage

    INTEGER(i_std)                                         :: i, j

    DO j=2,nvm
      DO i=1,npts
        ! reset devstage at the beginning of growing season
        IF (begin_leaves(i,j) .EQ. .TRUE.) THEN
          GRM_devstage(i,j) = MAX(0.0, t2m_daily(i) - tbase)/tasumrep

        ! when tsoil > tbase = 5oC 
        ELSE IF ((GRM_devstage(i,j) .GE. 0.0) .AND. &
                 (tsurf_daily(i) .GT. tbase) .AND. &
                 (GRM_devstage(i,j) .LT. 2.0))  THEN

          GRM_devstage(i,j) = GRM_devstage(i,j) + MAX(0.0, t2m_daily(i) - &
                              tbase)/tasumrep
        ELSE 
          GRM_devstage(i,j) = GRM_devstage(i,j)
        ENDIF

      ENDDO
    ENDDO 

    WHERE (GRM_devstage .GT. 2.0) 
      GRM_devstage(:,:) = 2.0
    ENDWHERE

  END SUBROUTINE calc_GRM_devstage

  SUBROUTINE apply_agri_fert(npts,dt_days,day_of_year, &
               agri_fert_save,litter,agri_nfert,agri_pfert,agri_masschangecnp)

    INTEGER (i_std)                   , INTENT(in)   :: npts
    REAL(r_std)                       , INTENT(in)   :: dt_days
    INTEGER(i_std)                    , INTENT(in)   :: day_of_year
    REAL(r_std), DIMENSION(npts,nvm,4), INTENT(in)   :: agri_fert_save
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(inout)  :: litter
    REAL(r_std), DIMENSION(npts,nvm,ninput), INTENT(out)  :: agri_nfert
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)  :: agri_pfert
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(out)  :: agri_masschangecnp

    INTEGER(i_std)                                         :: i, j
    REAL(r_std), DIMENSION(npts,nvm)        :: tmp_Nmineral
    REAL(r_std), DIMENSION(npts,nvm)        :: tmp_Nmanure
    REAL(r_std), DIMENSION(npts,nvm)        :: tmp_Pmineral
    REAL(r_std), DIMENSION(npts,nvm)        :: tmp_Tfert 
    REAL(r_std), PARAMETER       :: org_furine = 0.9
    REAL(r_std), PARAMETER       :: org_fmetabolic = 0.9
    REAL(r_std), PARAMETER       :: org_cnratio = 10.0
    REAL(r_std), PARAMETER       :: org_pnratio = 0.3
    REAL(r_std), PARAMETER       :: min_famm = 0.5
    ! save fertilizer input to temporary variable
    tmp_Nmineral(:,:) = agri_fert_save(:,:,1)
    tmp_Nmanure(:,:)  = agri_fert_save(:,:,2)
    tmp_Pmineral(:,:) = agri_fert_save(:,:,3)
    tmp_Tfert(:,:)    = agri_fert_save(:,:,4)
    ! initialize output variables
    agri_nfert(:,:,:) = zero
    agri_pfert(:,:) = zero
    agri_masschangecnp(:,:,:) = zero    
    DO j=2,nvm
      DO i=1,npts
        ! it is time to fertilization
        IF ((tmp_Tfert(i,j)-0.5 .LT. REAL(day_of_year)) .AND. &
            (tmp_Tfert(i,j)+0.5 .GT. REAL(day_of_year))) THEN 
          ! first apply mineral N and P
          agri_nfert(i,j,iammonium) = tmp_Nmineral(i,j) * min_famm
          agri_nfert(i,j,initrate)  = tmp_Nmineral(i,j) * (un - min_famm)
          agri_pfert(i,j)           = tmp_Pmineral(i,j)

          ! then apply manure C, N and P to litter
          ! considering typical C:N:P ratio in manure
          ! Here, assumptions were made for the C:N:P ratios, 
          ! C:N ratio of slurry 10:1 was used considering its easy application
          ! Soussana and Lemaire, 2014
          ! N:P ratio we chose 10:3 as mean ratio
          ! Phosphorus excreted via urine by ruminants is usually negligible
          ! because a possible surplus of digested P is recirculated to the
          ! rumen and from there excreted in faeces instead of in urine.
          ! http://www.fao.org/WAIRDOCS/LEAD/X6113E/x6113e06.htm
          ! 10:3 of N:P ratio is assumed
          ! according to US states data
          ! https://www.epa.gov/nutrient-policy-data/estimated-animal-agriculture-nitrogen-and-phosphorus-manure

          ! NEED to revise considering the C:N:P ratio in plant and animal
          ! for metabolic ratio, I now used fvgcmetabolic=0.8 in GRM

          ! NOTE: the slurry contain only part of solid manure N (4-10%) 
          ! Here, we assumed a slurry application for N
          ! use a N in urine as 0.9 = fkanurine 
          ! for the solid part 0.9 = fkacmetabolic in GRM
          ! the urine N part is applied directly to the soil mineral N pool as
          ! mineral ammonium fertilizer
          agri_nfert(i,j,iammonium) = agri_nfert(i,j,iammonium) + &
             tmp_Nmanure(i,j) * org_furine
          ! Urine C and P is negligible
 
          litter(i,imetabolic,j,iabove,icarbon) = litter(i,imetabolic,j,iabove,icarbon) + &
             tmp_Nmanure(i,j) * (un - org_furine) * org_fmetabolic * org_cnratio
          litter(i,imetabolic,j,iabove,initrogen) = litter(i,imetabolic,j,iabove,initrogen) + &
             tmp_Nmanure(i,j) * (un - org_furine) * org_fmetabolic
          litter(i,imetabolic,j,iabove,iphosphorus) = litter(i,imetabolic,j,iabove,iphosphorus) + &
             tmp_Nmanure(i,j) * (un - org_furine) * org_fmetabolic * org_pnratio

          litter(i,istructural,j,iabove,icarbon) = litter(i,istructural,j,iabove,icarbon) + &
             tmp_Nmanure(i,j) * (un - org_furine) * (un-org_fmetabolic) * org_cnratio
          litter(i,istructural,j,iabove,initrogen) = litter(i,istructural,j,iabove,initrogen) + &
             tmp_Nmanure(i,j) * (un - org_furine) * (un-org_fmetabolic)
          litter(i,istructural,j,iabove,iphosphorus) = litter(i,istructural,j,iabove,iphosphorus) + &
             tmp_Nmanure(i,j) * (un - org_furine) * (un-org_fmetabolic) * org_pnratio

          agri_masschangecnp(i,j,icarbon) = tmp_Nmanure(i,j) * (un - org_furine) * org_cnratio
          agri_masschangecnp(i,j,initrogen) = tmp_Nmanure(i,j) * (un - org_furine)
          agri_masschangecnp(i,j,iphosphorus) = tmp_Nmanure(i,j) * (un - org_furine) * org_pnratio
        ENDIF
      ENDDO ! i npts
    ENDDO ! j nvm

      ! write history only organic fert to litter is write here
      ! the mineral part and the urine part is write in nitrogen_dynamics,
      ! and phosphorus_dynamics 
      CALL xios_orchidee_send_field('AGRI_ORGFERT_c',agri_masschangecnp(:,:,icarbon))
      CALL histwrite_p (hist_id_stomate, 'AGRI_ORGFERT_c', itime, &
         agri_masschangecnp(:,:,icarbon), npts*nvm, horipft_index)
      CALL xios_orchidee_send_field('AGRI_ORGFERT_n',agri_masschangecnp(:,:,initrogen))
      CALL histwrite_p (hist_id_stomate, 'AGRI_ORGFERT_n', itime, &
         agri_masschangecnp(:,:,initrogen), npts*nvm, horipft_index)
      CALL xios_orchidee_send_field('AGRI_ORGFERT_p',agri_masschangecnp(:,:,iphosphorus))
      CALL histwrite_p (hist_id_stomate, 'AGRI_ORGFERT_p', itime, &
         agri_masschangecnp(:,:,iphosphorus), npts*nvm, horipft_index)


  END SUBROUTINE apply_agri_fert

END MODULE stomate_lpj
