! =================================================================================================================================
! MODULE       : stomate
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Groups the subroutines that: (1) initialize all variables in 
!! stomate, (2) read and write forcing files of stomate and the soil component,
!! (3) aggregates and convert variables to handle the different time steps 
!! between sechiba and stomate, (4) call subroutines that govern major stomate
!! processes (litter, soil, and vegetation dynamics) and (5) structures these tasks 
!! in stomate_main
!!
!!\n DESCRIPTION : None
!!
!! RECENT CHANGE(S) : None
!!
!! REFERENCE(S)	: None
!!
!! SVN :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate

  ! Modules used:
#ifdef __NAGFOR
  USE F90_UNIX_PROC
  USE F90_UNIX_IO
#endif
  USE function_library,   ONLY : check_biomass_sync, biomass_to_lai
  USE netcdf
  USE defprec
  USE grid
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE sapiens_forestry,   ONLY : read_in_fm_map, read_in_litter_map
  USE structures
  USE stomate_io
  USE stomate_data
  USE stomate_season
  USE stomate_lpj
  USE stomate_assimtemp
  USE stomate_litter
  USE stomate_vmax
  USE stomate_soilcarbon
  USE stomate_resp
  USE mod_orchidee_para
  USE ioipsl_para 
  USE xios_orchidee

  USE matrix_resolution
  
  IMPLICIT NONE

  ! Private & public routines

  PUBLIC stomate_main,stomate_clear,init_forcing,forcing_read


  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_cov_max        !! Maximal fractional coverage: maximum share of a pixel
                                                                         !! taken by a PFT 
!$OMP THREADPRIVATE(veget_cov_max)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_cov_max_new    !! Updated maximal fractional coverage (unitless)
!$OMP THREADPRIVATE(veget_cov_max_new)
  

  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ba                   !! Stand basal area @tex $(m^{-2})$ @endtex
!$OMP THREADPRIVATE(ba)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: age                  !! Age of PFT it normalized by biomass - can increase and
                                                                         !! decrease - (years)
!$OMP THREADPRIVATE(age)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: adapted              !! Winter too cold for PFT to survive (0-1, unitless)
!$OMP THREADPRIVATE(adapted)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: regenerate           !! Winter sufficiently cold to produce viable seeds 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(regenerate)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: everywhere           !! Is the PFT everywhere in the grid box or very localized 
                                                                         !! (after its intoduction)
!$OMP THREADPRIVATE(everywhere)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: fireindex            !! Probability of fire (unitless)
!$OMP THREADPRIVATE(fireindex)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_lastlight      !! Vegetation fractions (on ground) after last light 
                                                                         !! competition (unitless) 
!$OMP THREADPRIVATE(veget_lastlight)
  
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: fpc_max              !! "maximal" coverage fraction of a grid box (LAI -> 
                                                                         !! infinity) on ground. [??CHECK??] It's set to zero here, 
                                                                         !! and then is used once in lpj_light.f90 to test if 
                                                                         !! fpc_nat is greater than it. Something seems missing
!$OMP THREADPRIVATE(fpc_max)
  
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)        :: PFTpresent           !! PFT exists (equivalent to veget > 0 for natural PFTs)
!$OMP THREADPRIVATE(PFTpresent)
  
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)        :: senescence           !! The PFT is senescent
!$OMP THREADPRIVATE(senescence)
  
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)        :: need_adjacent        !! This PFT needs to be in present in an adjacent gridbox 
                                                                         !! if it is to be introduced in a new gridbox
!$OMP THREADPRIVATE(need_adjacent)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_daily         !! Daily plant available water -root profile weighted 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(humrel_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: vir_humrel_daily     !! Virtual daily plant available water -root profile weighted 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(vir_humrel_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: stressed_daily       !! Accumulated proxy for stressed ecosystem functioning
                                                                         !! see variable stressed defined in sechiba
!$OMP THREADPRIVATE(stressed_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: unstressed_daily     !! Accumulated proxy for unstressed ecosystem functioning
                                                                         !! see variable stressed defined in sechiba 
!$OMP THREADPRIVATE(unstressed_daily)
 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: daylight             !! Time sterps dt_radia during daylight
!$OMP THREADPRIVATE(daylight)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: transpir_supply_daily !! Daily supply of water for transpiration @tex $(mm dt^{-1})$ @endtex
! $OMP THREADPRIVATE(transpir_supply_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: vir_transpir_supply_daily !! Daily supply of water for transpiration 
                                                                         !! @tex $(mm dt^{-1})$ @endtex
! $OMP THREADPRIVATE(vir_transpir_supply_daily)                                                                        
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: transpir_daily       !! Daily demand of water for transpiration @tex $(mm dt^{-1})$ @endtex
! $OMP THREADPRIVATE(transpir_daily) 

 REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)     :: humrel_week          !! "Weekly" plant available water -root profile weighted
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(humrel_week)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_month         !! "Monthly" plant available water -root profile weighted
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(humrel_month)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_growingseason !! Mean growing season moisture availability (used for 
                                                                         !! allocation response)
!$OMP THREADPRIVATE(humrel_growingseason)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: vir_humrel_growingseason !! Mean growing season moisture availability (used for 
                                                                         !! allocation response)
!$OMP THREADPRIVATE(vir_humrel_growingseason)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_lastyear   !! Last year's max plant available water -root profile 
                                                                         !! weighted (0-1, unitless)
!$OMP THREADPRIVATE(maxhumrel_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_thisyear   !! This year's max plant available water -root profile 
                                                                         !! weighted (0-1, unitless) 
!$OMP THREADPRIVATE(maxhumrel_thisyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_lastyear   !! Last year's min plant available water -root profile 
                                                                         !! weighted (0-1, unitless)  
!$OMP THREADPRIVATE(minhumrel_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_thisyear   !! This year's minimum plant available water -root profile
                                                                         !! weighted (0-1, unitless)
!$OMP THREADPRIVATE(minhumrel_thisyear)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_daily            !! Daily air temperature at 2 meter (K)
!$OMP THREADPRIVATE(t2m_daily)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_week             !! Mean "weekly" (default 7 days) air temperature at 2 
                                                                         !! meter (K)  
!$OMP THREADPRIVATE(t2m_week)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_month            !! Mean "monthly" (default 20 days) air temperature at 2 
                                                                         !! meter (K)
!$OMP THREADPRIVATE(t2m_month)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_longterm         !! Mean "Long term" (default 3 years) air temperature at 
                                                                         !! 2 meter (K) 
!$OMP THREADPRIVATE(t2m_longterm)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_min_daily        !! Daily minimum air temperature at 2 meter (K)
!$OMP THREADPRIVATE(t2m_min_daily)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: tlong_ref            !! "Long term" (default 3 years) reference temperature at
                                                                         !! 2 meter (K) 
!$OMP THREADPRIVATE(tlong_ref)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: tsurf_daily          !! Daily surface temperatures (K)
!$OMP THREADPRIVATE(tsurf_daily)

  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_daily         !! Daily precipitations sum @tex $(mm day^{-1})$ @endtex
!$OMP THREADPRIVATE(precip_daily)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_lastyear      !! Last year's annual precipitation sum 
                                                                         !! @tex $??(mm year^{-1})$ @endtex
!$OMP THREADPRIVATE(precip_lastyear)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_thisyear      !! This year's annual precipitation sum 
                                                                         !! @tex $??(mm year^{-1})$ @endtex 
!$OMP THREADPRIVATE(precip_thisyear)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_daily        !! Daily soil humidity (0-1, unitless)
!$OMP THREADPRIVATE(soilhum_daily)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_month        !! Soil humidity - integrated over a month (0-1, unitless) 
!$OMP THREADPRIVATE(soilhum_month)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_daily          !! Daily soil temperatures (K)
!$OMP THREADPRIVATE(tsoil_daily)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_month          !! Soil temperatures at each soil layer integrated over a
                                                                         !! month (K) 
!$OMP THREADPRIVATE(tsoil_month)
 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: litterhum_daily      !! Daily litter humidity (0-1, unitless)
!$OMP THREADPRIVATE(litterhum_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: control_moist        !! Moisture control of heterotrophic respiration 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(control_moist)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: control_temp         !! Temperature control of heterotrophic respiration at the
                                                                         !! different soil levels (0-1, unitless)
!$OMP THREADPRIVATE(control_temp)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: control_moist_daily  !! Moisture control of heterotrophic respiration daily 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(control_moist_daily)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: control_temp_daily   !! Temperature control of heterotrophic respiration, above
                                                                         !! and below daily (0-1, unitless)
!$OMP THREADPRIVATE(control_temp_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_init_date        !! inital date for gdd count 
!$OMP THREADPRIVATE(gdd_init_date)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_from_growthinit  !! gdd from beginning of season (C)
!$OMP THREADPRIVATE(gdd_from_growthinit)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_lastyear        !! Last year's annual Growing Degree Days,
                                                                         !! threshold 0 deg C (K) 
!$OMP THREADPRIVATE(gdd0_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_thisyear        !! This year's annual Growing Degree Days,
                                                                         !! threshold 0 deg C (K)
!$OMP THREADPRIVATE(gdd0_thisyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_m5_dormance      !! Growing degree days for onset of growing season, 
                                                                         !! threshold -5 deg C (K)
!$OMP THREADPRIVATE(gdd_m5_dormance)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_midwinter        !! Growing degree days for onset of growing season, 
                                                                         !! since midwinter (K)
!$OMP THREADPRIVATE(gdd_midwinter)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ncd_dormance         !! Number of chilling days since leaves were lost (days) 
!$OMP THREADPRIVATE(ncd_dormance)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ngd_minus5           !! Number of growing days, threshold -5 deg C (days)
!$OMP THREADPRIVATE(ngd_minus5)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: hum_min_dormance     !! Minimum moisture during dormance (0-1, unitless) 
!$OMP THREADPRIVATE(hum_min_dormance)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_daily            !! Daily gross primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(gpp_daily) 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_week             !! Mean "weekly" (default 7 days) GPP  
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(gpp_week)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_lastyear  !! Last year's maximum "weekly" GPP  
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex 
!$OMP THREADPRIVATE(maxgppweek_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_thisyear  !! This year's maximum "weekly" GPP  
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex  
!$OMP THREADPRIVATE(maxgppweek_thisyear)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_daily            !! Daily net primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex 
!$OMP THREADPRIVATE(npp_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_longterm         !! "Long term" (default 3 years) net primary productivity 
                                                                         !! per ground area  
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex   
!$OMP THREADPRIVATE(npp_longterm)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_equil            !! Equilibrium NPP written to forcesoil 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(npp_equil)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: npp_tot              !! Total NPP written to forcesoil 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex 
!$OMP THREADPRIVATE(npp_tot)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: resp_maint_part_radia!! Maintenance respiration of different plant parts per 
                                                                         !! total ground area at Sechiba time step  
                                                                         !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_part_radia)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: resp_maint_part      !! Maintenance respiration of different plant parts per
                                                                         !! total ground area at Stomate time step 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_part)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_maint_radia     !! Maintenance respiration per ground area at Sechiba time
                                                                         !! step   
                                                                         !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_radia)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_maint_d         !! Maintenance respiration per ground area at Stomate time 
                                                                         !! step  
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_growth_d        !! Growth respiration per ground area 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_growth_d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_hetero_d        !! Heterotrophic respiration per ground area 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_hetero_d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_hetero_radia    !! Heterothrophic respiration per ground area at Sechiba
                                                                         !! time step 
                                                                         !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex 
!$OMP THREADPRIVATE(resp_hetero_radia)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)   :: turnover_time       !! Turnover time of grasses 
                                                                         !! @tex $(dt_slow^{-1})$ @endtex 
!$OMP THREADPRIVATE(turnover_time)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: turnover_daily      !! Senescence-driven turnover (better: mortality) of 
                                                                         !! leaves and roots  
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(turnover_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: turnover_littercalc !! Senescence-driven turnover (better: mortality) of 
                                                                         !! leaves and roots at Sechiba time step 
                                                                         !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex 
!$OMP THREADPRIVATE(turnover_littercalc)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: turnover_longterm   !! "Long term" (default 3 years) senescence-driven 
                                                                         !! turnover (better: mortality) of leaves and roots 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(turnover_longterm)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: bm_to_litter        !! Background (not senescence-driven) mortality of biomass
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
!$OMP THREADPRIVATE(bm_to_litter)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: bm_to_littercalc    !! conversion of biomass to litter per ground area at 
                                                                         !! Sechiba time step 
                                                                         !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex 
!$OMP THREADPRIVATE(bm_to_littercalc)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)   :: dead_leaves         !! Metabolic and structural pools of dead leaves on ground
                                                                         !! per PFT @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(dead_leaves)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:,:):: litter             !! Above and below ground metabolic and structural litter 
                                                                         !! per ground area 
                                                                         !! @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(litter)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: litterpart           !! Fraction of litter above the ground belonging to 
                                                                         !! different litter pools (unitless)
!$OMP THREADPRIVATE(litterpart)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: firelitter           !! Total litter above the ground that could potentially 
                                                                         !! burn @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(firelitter)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:):: soilcarbon_input     !! Quantity of carbon going into carbon pools from litter
                                                                         !! decomposition per ground area  at Sechiba time step 
                                                                         !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex 
!$OMP THREADPRIVATE(soilcarbon_input)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: soilcarbon_input_daily !! Daily quantity of carbon going into carbon pools from
                                                                           !! litter decomposition per ground area 
                                                                           !! @tex $(gC m^{-2} day^{-1})$ @endtex 
!$OMP THREADPRIVATE(soilcarbon_input_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: carbon               !! Soil carbon pools per ground area: active, slow, or 
                                                                         !! passive, @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(carbon)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: lignin_struc         !! Ratio Lignine/Carbon in structural litter for above and
                                                                         !! below ground compartments (unitless)

!$OMP THREADPRIVATE(lignin_struc)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: lignin_wood          !! Ratio Lignine/Carbon in woody litter for above and
                                                                         !! below ground compartments (unitless)

!$OMP THREADPRIVATE(lignin_wood)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: black_carbon         !! Black carbon on the ground 
                                                                         !! @tex $(gC m^{-2})$ @endtex   
!$OMP THREADPRIVATE(black_carbon)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_lastyearmax       !! Last year's maximum leaf mass per ground area for each
                                                                         !! PFT @tex $(gC m^{-2})$ @endtex  
!$OMP THREADPRIVATE(lm_lastyearmax)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_thisyearmax       !! This year's maximum leaf mass per ground area for each
                                                                         !! PFT @tex $(gC m^{-2})$ @endtex  
!$OMP THREADPRIVATE(lm_thisyearmax)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_lastyear      !! Last year's maximum fpc for each natural PFT, on ground
                                                                         !! [??CHECK] fpc but this ones look ok (computed in 
                                                                         !! season, used in light)?? 
!$OMP THREADPRIVATE(maxfpc_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_thisyear      !! This year's maximum fpc for each PFT, on ground (see 
                                                                         !! stomate_season), [??CHECK] fpc but this ones look ok 
                                                                         !! (computed in season, used in light)??
!$OMP THREADPRIVATE(maxfpc_thisyear)

  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:,:) :: leaf_age             !! Age of different leaf classes (days)
!$OMP THREADPRIVATE(leaf_age)
  
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:,:) :: leaf_frac            !! PFT fraction of leaf mass in leaf age class (0-1, 
                                                                         !! unitless) 
!$OMP THREADPRIVATE(leaf_frac)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: when_growthinit      !! Days since beginning of growing season (days)
!$OMP THREADPRIVATE(when_growthinit)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: herbivores           !! Time constant of probability of a leaf to be eaten by a
                                                                         !! herbivore (days)
!$OMP THREADPRIVATE(herbivores)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: RIP_time             !! How much time ago was the PFT eliminated for the last 
                                                                         !! time (year)
!$OMP THREADPRIVATE(RIP_time)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: time_lowgpp          !! Duration of dormance (days)
!$OMP THREADPRIVATE(time_lowgpp)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: time_hum_min         !! Time elapsed since strongest moisture limitation (days) 
!$OMP THREADPRIVATE(time_hum_min)

  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: clay_fm              !! Soil clay content (0-1, unitless), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: clay_fm_g            !! Soil clay content (0-1, unitless), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: precip_fm            !! Daily precipitations sum @tex $(mm day^{-1})$ @endtex,
                                                                         !! parallel computing
   
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: precip_fm_g          !! Daily precipitations sum @tex $(mm day^{-1})$ @endtex,
                                                                         !! parallel computing
   
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: litterhum_daily_fm   !! Daily relative humidity of litter (0-1, unitless), 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: litterhum_daily_fm_g !! Daily relative humidity of litter (0-1, unitless), 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_daily_fm         !! Daily air temperature at 2 meter (K), parallel 
                                                                         !! computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_daily_fm_g       !! Daily air temperature at 2 meter (K), parallel 
                                                                         !! computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_min_daily_fm     !! Daily minimum air temperature at 2 meter (K), 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_min_daily_fm_g   !! Daily minimum air temperature at 2 meter (K), 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: tsurf_daily_fm       !! Daily surface temperatures (K), parallel 
                                                                         !! computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: tsurf_daily_fm_g     !! Daily surface temperatures (K), parallel 
                                                                         !! computing 
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: tsoil_daily_fm       !! Daily soil temperatures (K), parallel computing 
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: tsoil_daily_fm_g     !! Daily soil temperatures (K), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: soilhum_daily_fm     !! Daily soil humidity (0-1, unitless), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: soilhum_daily_fm_g   !! Daily soil humidity (0-1, unitless), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: humrel_daily_fm      !! Daily relative humidity of atmosphere (0-1, unitless), 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: humrel_daily_fm_g    !! Daily relative humidity of atmosphere (0-1, unitless), 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: gpp_daily_fm         !! Daily gross primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex, 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: gpp_daily_fm_g       !! Daily gross primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} day^{-1})$ @endtex, 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_fm             !! Vegetation coverage taking into account non-biological
                                                                         !! coverage (unitless), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_fm_g           !! Vegetation coverage taking into account non-biological
                                                                         !! coverage (unitless), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_max_fm         !! Maximum vegetation coverage taking into account 
                                                                         !! non-biological coverage (unitless), parallel computing
   
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_max_fm_g       !! Maximum vegetation coverage taking into account none 
                                                                         !! biological coverage (unitless), parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: lai_fm               !! Leaf area index @tex $(m^2 m^{-2})$ @endtex 
                                                                         !! parallel computing
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: lai_fm_g             !! Leaf area index @tex $(m^2 m^{-2})$ @endtex 
                                                                         !! parallel computing

  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: senescence_days      !! days since the beginning of senescence (days)
  
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: rue_longterm         !! Longterm radiation use efficiency (??units??)
  
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: t_photo_stress       !! Temperature stress for CEXCHANGE photosynthesis (0-1)
  
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: lab_fac              !! Activity of labile pool factor (??units??)
  
!!$  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: lai_target           !! Target lai in functional allocation
!!$                                                                         !! @tex $(m^{2} m^{-2})$ @endtex

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: co2_fire             !! Carbon emitted to the atmosphere by burning living 
                                                                         !! and dead biomass 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex 
!$OMP THREADPRIVATE(co2_fire)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: co2_to_bm_dgvm       !! Psuedo-photosynthesis,C used to provide seedlings with
                                                                         !! an initial biomass, arbitrarily removed from the 
                                                                         !! atmosphere  
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex 
!$OMP THREADPRIVATE(co2_to_bm_dgvm)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: co2_flux_daily       !! Daily net CO2 flux between atmosphere and biosphere 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
                                                                         !! [??CHECK] sign convention?
!$OMP THREADPRIVATE(co2_flux_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: co2_flux_monthly     !! Monthly net CO2 flux between atmosphere and biosphere 
                                                                         !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
                                                                         !! [??CHECK] sign convention? 
!$OMP THREADPRIVATE(co2_flux_monthly)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: prod_s               !! Wood products remaining in the 1 year-turnover pool 
                                                                         !! after the annual release for each compartment 
                                                                         !! @tex $(gC m^{-2})$ @endtex    
                                                                         !! (0:1 input from year of land cover change),
                                                                         !! dimension(#pixels,0:1 years
!$OMP THREADPRIVATE(prod_s)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: prod_m               !! Wood products remaining in the 10 year-turnover pool 
                                                                         !! after the annual release for each compartment 
                                                                         !! @tex $(gC m^{-2})$ @endtex    
                                                                         !! (0:10 input from year of land cover change),
                                                                         !! dimension(#pixels,0:10 years
!$OMP THREADPRIVATE(prod_m)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: prod_l              !! Wood products remaining in the 100 year-turnover pool
                                                                         !! after the annual release for each compartment
                                                                         !! @tex $(gC m^{-2})$ @endtex  
                                                                         !! (0:100 input from year of land cover change), 
                                                                         !! dimension(#pixels,0:100 years)
!$OMP THREADPRIVATE(prod_l)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: flux_s               !! Wood decomposition from the 1 year-turnover pool 
                                                                         !! compartments 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex 
                                                                         !! dimension(#pixels,0:1)  
!$OMP THREADPRIVATE(flux_s)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: flux_m               !! Wood decomposition from the 10 year-turnover pool 
                                                                         !! compartments 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex 
                                                                         !! dimension(#pixels,0:10)  
!$OMP THREADPRIVATE(flux_m)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: flux_l              !! Wood decomposition from the 100 year-turnover pool 
                                                                         !! compartments 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
                                                                         !! dimension(#pixels,0:100)
!$OMP THREADPRIVATE(flux_l)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: cflux_prod_s         !! Release during first year following land cover change 
                                                                         !! (paper, burned, etc...) 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex  
!$OMP THREADPRIVATE(cflux_prod_s)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: cflux_prod_m         !! Total annual release from the 10 year-turnover pool
                                                                         !! sum of flux_m  
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(cflux_prod_m)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: cflux_prod_l         !! Total annual release from the 100 year-turnover pool 
                                                                         !! sum of flux_l 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(cflux_prod_l)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: carb_mass_total      !! Total on-site and off-site C pool 
                                                                         !! @tex $(??gC m^{-2})$ @endtex                        
!$OMP THREADPRIVATE(carb_mass_total)
!---
  REAL(r_std),SAVE                               :: dt_days=zero         !! Time step of STOMATE (days) 
!$OMP THREADPRIVATE(dt_days)
  REAL(r_std),SAVE                               :: day_counter=zero     !! Time step each day sechiba (dtradia)   
!$OMP THREADPRIVATE(day_counter)
  INTEGER(i_std),SAVE                            :: date=0               !! Date (days) 
!$OMP THREADPRIVATE(date)
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:)   :: nforce               !! Number of states calculated for the soil forcing 
                                                                         !! variables (unitless), dimension(::nparan*::nbyear) both 
                                                                         !! given in the run definition file    
!$OMP THREADPRIVATE(nforce)
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:)   :: isf                  !! Index for number of time steps that can be stored in 
                                                                         !! memory (unitless), dimension (#nsfm)
!$OMP THREADPRIVATE(isf)
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:)   :: nf_cumul             !! Number of years over which the average is calculated in
                                                                         !! forcesoil when cumul flag is set, dimension (#nsft)
                                                                         !! [??CHECK] definition the dimension is number of 
                                                                         !! timesteps in a year?
!$OMP THREADPRIVATE(nf_cumul)
  INTEGER,PARAMETER                              :: r_typ = nf90_real4   !! Specify data format (server dependent)
  
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:)          :: nf_written           !! Flag indicating whether the forcing data have been 
                                                                         !! written
!$OMP THREADPRIVATE(nf_written)

  LOGICAL, SAVE                                  :: do_slow=.FALSE.      !! Flag that determines whether stomate_accu calculates
                                                                         !! the sum(do_slow=.FALSE.) or the mean 
                                                                         !! (do_slow=.TRUE.)
!$OMP THREADPRIVATE(do_slow)
  LOGICAL, SAVE                                  :: EndOfYear=.FALSE.    !! Update annual variables? This variable must be 
                                                                         !! .TRUE. once a year
!$OMP THREADPRIVATE(EndOfYear)
  LOGICAL, SAVE                                  :: EndOfMonth=.FALSE.   !! Update monthly variables? This variable must be 
                                                                         !!.TRUE. once a month 
!$OMP THREADPRIVATE(EndOfMonth)
  
  LOGICAL, SAVE                                  :: l_first_stomate = .TRUE.!! Is this the first call of stomate?
!$OMP THREADPRIVATE(l_first_stomate)
  
  LOGICAL, SAVE                                  :: cumul_forcing=.FALSE.!! flag for cumul of forcing if teststomate
!$OMP THREADPRIVATE(cumul_forcing)
  
  LOGICAL, SAVE                                  :: cumul_Cforcing=.FALSE.  !! Flag, if internal parameter cumul_Cforcing is 
                                                                         !! TRUE then ::nbyear (defined in run definition 
                                                                         !! file will be forced to 1 later in this module. If 
                                                                         !! FALSE the mean over ::nbyear is written in forcesoil
!$OMP THREADPRIVATE(cumul_Cforcing)
  
  REAL(r_std),DIMENSION(:),ALLOCATABLE,SAVE      :: circ_class_dist   !! When the circumference class distribution
                                                                      !! is redone due to empty classes, this is the
                                                                      !! tree distribution used.  Notice that this distribution
                                                                      !! is normalized after being read in.
!$OMP THREADPRIVATE(circ_class_dist)
  REAL(r_std),DIMENSION(:),ALLOCATABLE,SAVE      :: st_dist           !! During self-thinning, we need to decide which
                                                                      !! circumference classes to kill trees in.  This
                                                                      !! is the distribution that tells us this.  Notice that 
                                                                      !! it is normalized after being read in.
!$OMP THREADPRIVATE(st_class_dist)
 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: harvest_above_monthly!! [??CHECK] post-processing - should be removed?
!$OMP THREADPRIVATE(harvest_above_monthly)
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: cflux_prod_monthly   !! [??CHECK] post-processing - should be removed?
!$OMP THREADPRIVATE(cflux_prod_monthly)

  INTEGER(i_std), SAVE                           :: global_years         !! Global counter of years (year)
!$OMP THREADPRIVATE(global_years)
  
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:)       :: ok_equilibrium       !! Logical array marking the points where the resolution is ok 
                                                                         !! (true/false)
!$OMP THREADPRIVATE(ok_equilibrium)
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:)       :: ok_equilibrium_g     !! Logical array marking the points where the resolution is ok 
                                                                         !! (for parallelisation) (true/false)
!$OMP THREADPRIVATE(ok_equilibrium_g)
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:)       :: carbon_eq            !! Logical array to mark the carbon pools at equilibrium ? 
                                                                         !! If true, the job stops. (true/false)
!$OMP THREADPRIVATE(carbon_eq)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)   :: nbp_accu             !! Accumulated Net Biospheric Production over the year (gC.m^2 )
!$OMP THREADPRIVATE(nbp_accu)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)   :: nbp_flux             !! Net Biospheric Production (gC.m^2.day^{-1})
!$OMP THREADPRIVATE(nbp_flux)
  
  REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:,:)   :: matrixA              !! Matrix containing the fluxes between the carbon pools
                                                                         !! per sechiba time step 
                                                                         !! @tex $(gC.m^2.day^{-1})$ @endtex
!$OMP THREADPRIVATE(matrixA)
  
  REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:)     :: vectorB              !! Vector containing the litter increase per sechiba time step
                                                                         !! @tex $(gC m^{-2})$ @endtex
!$OMP THREADPRIVATE(vectorB)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: MatrixV          !! Matrix containing the accumulated values of matrixA 
!$OMP THREADPRIVATE(MatrixV)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: VectorU          !! Matrix containing the accumulated values of VectorB
!$OMP THREADPRIVATE(VectorU)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: MatrixW          !! Matrix containing the opposite of matrixA
!$OMP THREADPRIVATE(MatrixW)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: previous_stock   !! Array containing the carbon stock calculated by the analytical
                                                                         !! method in the previous resolution
!$OMP THREADPRIVATE(previous_stock)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: current_stock    !! Array containing the carbon stock calculated by the analytical
                                                                         !! method in the current resolution 
!$OMP THREADPRIVATE(current_stock)
  REAL(r_std), SAVE                                  :: eps_carbon       !! Stopping criterion for carbon pools (unitless,0-1)
!$OMP THREADPRIVATE(eps_carbon)

  REAL(r_std), ALLOCATABLE, DIMENSION(:,:)           :: sigma            !! Threshold for indivudal tree growth (m, trees whose
                                                                         !! circumference is smaller than sigma don't grow much)
!$OMP THREADPRIVATE(sigma)

  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: age_stand        !! Age of stand (years)
!$OMP THREADPRIVATE(age_stand)
  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: rotation_n       !! Rotation number (number of rotation since pft is managed)
!$OMP THREADPRIVATE(rotation_n)
  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: last_cut         !! Years since last thinning (years)
!$OMP THREADPRIVATE(last_cut)
  
!!$  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: av_height        !! Average height in the stand (m)
!!$!$OMP THREADPRIVATE(av_height)
  
!!$  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: lai_max_calc     !! lai_max_calc
!!$!$OMP THREADPRIVATE(lai_max_calc)
    
!++++CHECK+++
! I had to move these into sechiba, since they need to be passed to sechiba
!!$  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: circ_class_n     !! Number of trees within a circumference class
!!$!$OMP THREADPRIVATE(circ_class_n)
!!$  
!!$  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:,:) :: circ_class_biomass !! Biomass components of the model tree within a circ class 
!!$                                                                         !! @tex $(gC ind^{-1@)$ @endtex
!!$!$OMP THREADPRIVATE(circ_class_biomass)
!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:):: biomass              !! Biomass per ground area @tex $(gC m^{-2})$ @endtex 
!!$OMP THREADPRIVATE(biomass)
!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ind                  !! Vegetation density, number of individuals per unit 
                                                                            !! ground area @tex $(m^{-2})$ @endtex 
!!$OMP THREADPRIVATE(ind)

!!$  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: biomass_lastyear !! biomass of last year  @tex $(gC m^{-2})$ @endtex 
!!$!$OMP THREADPRIVATE(biomass_lastyear)
  
!!$  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: lai_max_lastyear !! max lai of last year 
!!$!$OMP THREADPRIVATE(lai_max_lastyear)
  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: forest_managed   !! forest management flag (is the forest being managed?)
                                                                         !! (0-4,unitless)
!$OMP THREADPRIVATE(forest_managed)
  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: forest_managed_lastyear !! forest management flag for the previous year
                                                                         !! (0-4,unitless)
!$OMP THREADPRIVATE(forest_managed_lastyear)
  
  ! Functional Allocation
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: KF               !! Scaling factor to convert sapwood mass
                                                                         !! into leaf mass (m). The initial value is calculated
                                                                         !! in prescribe and updated during allocation
!$OMP THREADPRIVATE(KF)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: k_latosa_adapt   !! Leaf to sapwood area adapted for waterstress. 
                                                                         !! Adaptation takes place at the end of the year
                                                                         !! (m)
!$OMP THREADPRIVATE(k_latosa_adapt)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: harvest_pool     !! The pool which records the quantity of
                                                                         !! wood harvested and thinned due to forest
                                                                         !! management and LCC.  
                                                                         !! @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(harvest_pool)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: harvest_type     !! Type of management that resulted
                                                                         !! in the harvest (unitless)
!$OMP THREADPRIVATE(harvest_type)  

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: harvest_cut      !! Type of cutting that was used for the harvest
                                                                         !! (unitless)
!$OMP THREADPRIVATE(harvest_cut)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: harvest_area     !! Harvested area (m^{2})
!$OMP THREADPRIVATE(harvest_area)  

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)       :: harvest_pool_bound       !! The boundaries of the diameter classes
                                                                                 !! in the wood harvest pools
                                                                                 !! @tex $(m)$ @endtex 
!$OMP THREADPRIVATE(harvest_pool_bound)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: mai              !! The mean annual increment used in
                                                                         !! forestry.  It is the average change
                                                                         !! in the wood volume of of the trunk
                                                                         !! over the lifetime of the forest.
                                                                         !! @tex $(m**3 / m**2 / year)$ @endtex 
!$OMP THREADPRIVATE(mai)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: pai              !! The period annual increment used in
                                                                         !! forestry.  It is the average change
                                                                         !! in the wood volume of of the trunk
                                                                         !! over the past n_pai years of the forest,
                                                                         !! where n_pai is defined in constants.f90.
                                                                         !! @tex $(m**3 / m**2 / year)$ @endtex 
!$OMP THREADPRIVATE(pai)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: previous_wood_volume !! The volume of the tree trunks
                                                                         !! in a stand for the previous year.
                                                                         !! @tex $(m**3 / m**2 )$ @endtex 
!$OMP THREADPRIVATE(previous_wood_volume)

  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: mai_count        !! The number of times we've
                                                                         !! calculated the volume increment
                                                                         !! for a stand
!$OMP THREADPRIVATE(mai_count)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: bm_sync          !! The difference betweeen the
                                                                         !! biomass in the circ_classes and
                                                                         !! the total biomass 
                                                                         !! @tex $(gC m^{-2})$ @endtex
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: coppice_dens     !! The density of a coppice at the first
                                                                         !! cutting.
                                                                         !! @tex $( 1 / m**2 )$ @endtex 
!$OMP THREADPRIVATE(coppice_dens)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)       :: litter_demand    !! The amount of litter which will
                                                                         !! be moved from forest to crop pools
                                                                         !! at the end of the year.
                                                                         !! @tex $( gC / year )$ @endtex 
!$OMP THREADPRIVATE(litter_demand)

  REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)      :: wstress_season   !! Water stress factor, based on hum_rel_daily
                                                                         !! (unitless, 0-1)
!$OMP THREADPRIVATE(wstress_season)

  REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)      :: wstress_month    !! Water stress factor, based on hum_rel_daily
                                                                         !! (unitless, 0-1)
!$OMP THREADPRIVATE(wstress_month)

  REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)      :: vir_wstress_fac  !! Water stress factor, based on hum_rel_daily
                                                                         !! (unitless, 0-1)
!$OMP THREADPRIVATE(vir_wstress_fac)
  REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)      :: c_buried         !! Carbon that is buried when land cover changes
                                                                         !! to a nobio type.  For example, in urbanization
                                                                         !! the soil carbon is assumed to be locked under
                                                                         !! the concrete and cannot decompose.  Nothing is
                                                                         !! done with this at the moment, it's just to make
                                                                         !! sure we account for all the carbon in the mass
                                                                         !! balance checks.
                                                                         !! @tex $( gC )$ @endtex 
!$OMP THREADPRIVATE(c_buried)

  

PUBLIC clay_fm, humrel_daily_fm, litterhum_daily_fm, t2m_daily_fm, &
   & t2m_min_daily_fm, tsurf_daily_fm, tsoil_daily_fm, soilhum_daily_fm, &
   & precip_fm, gpp_daily_fm, veget_fm, veget_max_fm, lai_fm
PUBLIC  dt_days, day_counter, date, do_slow, EndOfYear
PUBLIC isf, nf_written

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_main
!!
!>\BRIEF        Manages variable initialisation, reading and writing forcing 
!! files, aggregating data at stomate's time step (dt_slow), aggregating data
!! at longer time scale (i.e. for phenology) and uses these forcing to calculate
!! CO2 fluxes (NPP and respirations) and C-pools (litter, soil, biomass, ...)
!!
!! DESCRIPTION  : The subroutine manages 
!! divers tasks:
!! (1) Initializing all variables of stomate (first call)
!! (2) Reading and writing forcing data (last call)
!! (3) Adding CO2 fluxes to the IPCC history files
!! (4) Converting the time steps of variables to maintain consistency between
!! sechiba and stomate
!! (5) Use these variables to call stomate_lpj, maint_respiration, littercalc,
!! soilcarbon. The called subroutines handle: climate constraints 
!! for PFTs, PFT dynamics, Phenology, Allocation, NPP (based on GPP and
!! authothropic respiration), fire, mortality, vmax, assimilation temperatures,
!! all turnover processes, light competition, sapling establishment, lai,  
!! land cover change and litter and soil dynamics.
!! (6) Use the spin-up method developed by Lardy (2011)(only if SPINUP_ANALYTIC 
!! is set to TRUE).
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): deadleaf_cover, assim_param, lai, height, veget, 
!! veget_max, veget_max_new, totfrac_nobio_new, resp_maint, 
!! resp_hetero,resp_growth, co2_flux, fco2_lu.
!!
!! REFERENCES	: 
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!! Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!!
!! FLOWCHART    : 
!! \latexonly 
!! \includegraphics[scale=0.5]{stomatemainflow.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_main &
       & (kjit, kjpij, npts, dtradia, dt_slow, &
       ldrestart_write, ldforcing_write, ldcarbon_write, index, &
       lalo, neighbours, resolution, contfrac, totfrac_nobio, &
       clay, t2m, t2m_min, temp_sol, stempdiag, &
       humrel, shumdiag, litterhumdiag, precip_rain, precip_snow, &
       gpp, deadleaf_cover, assim_param, lai, frac_age, &
       veget, veget_max, veget_max_new, totfrac_nobio_new, &
       rest_id_stom, hist_id_stom, hist_id_stom_IPCC, &
       co2_flux, fco2_lu, resp_maint, resp_hetero, resp_growth, temp_growth, &
       sinang, circ_class_biomass, circ_class_n, &
       transpir, stressed, unstressed, transpir_supply, &
       biomass, ind, lai_per_level, laieff_fit, &
       vir_transpir_supply, Isotrop_Tran_Tot_p)

    IMPLICIT NONE


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)                   :: kjit                !! Time step number (unitless)
    INTEGER(i_std),INTENT(in)                   :: npts                !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std),INTENT(in)                   :: kjpij               !! Total size of the un-compressed grid (unitless)
    REAL(r_std),INTENT(in)                      :: dtradia             !! Time step of SECHIBA (seconds)
    REAL(r_std),INTENT(in)                      :: dt_slow             !! Time step of STOMATE (days)
!!$    LOGICAL,INTENT(in)                          :: ldrestart_read      !! Logical for _restart_ file to read
    LOGICAL,INTENT(in)                          :: ldrestart_write     !! Flag to write restart file for stomate
    LOGICAL,INTENT(in)                          :: ldforcing_write     !! Flag to write forcing file for main processes in 
                                                                       !! stomate 
    LOGICAL,INTENT(in)                          :: ldcarbon_write      !! Flag to write forcing file for soil processes in 
                                                                       !! stomate 
    INTEGER(i_std),INTENT(in)                   :: rest_id_stom        !! STOMATE's _Restart_ file identifier (unitless)
    INTEGER(i_std),INTENT(in)                   :: hist_id_stom        !! STOMATE's _history_ file identifier (unitless)
    INTEGER(i_std),INTENT(in)                   :: hist_id_stom_IPCC   !! STOMATE's IPCC _history_ file identifier 
                                                                       !! (unitless) 
    INTEGER(i_std),DIMENSION(npts),INTENT(in)   :: index               !! Indices of the pixels on the map. Stomate uses a 
                                                                       !! reduced grid excluding oceans. ::index contains 
                                                                       !! the indices of the terrestrial pixels only 
                                                                       !! (unitless) 
    INTEGER(i_std),DIMENSION(npts,8),INTENT(in) :: neighbours          !! Neighoring grid points if land for the DGVM 
                                                                       !! (unitless) 
    REAL(r_std),DIMENSION(npts,2),INTENT(in)    :: lalo                !! Geographical coordinates (latitude,longitude) 
                                                                       !! for pixels (degrees) 
    REAL(r_std),DIMENSION(npts,2),INTENT(in)    :: resolution          !! Size in x an y of the grid (m) - surface area of 
                                                                       !! the gridbox 
    REAL(r_std),DIMENSION (npts), INTENT (in)   :: contfrac            !! Fraction of continent in the grid cell (unitless)
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: totfrac_nobio       !! Fraction of grid cell covered by lakes, land 
                                                                       !! ice, cities, ... (unitless) 
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: clay                !! Clay fraction of soil (0-1, unitless)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: humrel              !! Relative humidity ("moisture availability") 
                                                                       !! (0-1, unitless) 
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: t2m                 !! 2 m air temperature (K)
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: t2m_min             !! Minimum 2 m air temp. during forcing time step 
                                                                       !! (K) 
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: temp_sol            !! Surface temperature (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: stempdiag           !! Soil temperature (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: shumdiag            !! Relative soil moisture (0-1, unitless)
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: litterhumdiag       !! Litter humidity (0-1, unitless)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: transpir            !! transpiration @tex $(kg m^{-2} timestep^{-1})$
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: transpir_supply     !! Supply of water for transpiration @tex $(mm dt^{-1})$ @endtex
    REAL(r_std),DIMENSION(:,:),INTENT (in)      :: vir_transpir_supply !! Virtual supply of water for transpiration to deal
                                                                       !! with water stress when PFT1 becomes vegetated in LCC
                                                                       !! @tex $(mm dt^{-1})$ @endtex
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: precip_rain         !! Rain precipitation  
                                                                       !! @tex $(mm dt_slow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: precip_snow         !! Snow precipitation  
                                                                       !! @tex $(mm dt_slow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(:,:),INTENT(in)       :: gpp                 !! GPP of total ground area  
                                                                       !! @tex $(gC m^{-2} time step^{-1})$ @endtex 
                                                                       !! Calculated in sechiba, account for vegetation 
                                                                       !! cover and effective time step to obtain ::gpp_d 
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: totfrac_nobio_new   !! New fraction of grid cell covered by lakes, land 
    REAL(r_std),DIMENSION(npts), INTENT(in)     :: sinang              !! the sine of the solar angle from the horizon (unitless)  
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: veget               !! Fraction of vegetation type including 
                                                                       !! non-biological fraction (unitless) 
    REAL(r_std),DIMENSION (:,:,:), INTENT (in)  :: Isotrop_Tran_Tot_p  !! Transmitted radiation per level (unitless)

    !! 0.2 Output variables
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: co2_flux            !! CO2 flux between atmosphere and biosphere per 
                                                                       !! average ground area 
                                                                       !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex  
                                                                       !! [??CHECK] sign convention? 
    REAL(r_std),DIMENSION(npts),INTENT(out)     :: fco2_lu             !! CO2 flux between atmosphere and biosphere from 
                                                                       !! land-use (without forest management)  
                                                                       !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex  
                                                                       !! [??CHECK] sign convention? 
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_maint          !! Maitenance component of autotrophic respiration in 
                                                                       !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_growth         !! Growth component of autotrophic respiration in 
                                                                       !! @tex ($gC m^{-2} dt_slow^{-1}$) @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_hetero         !! Heterotrophic respiration in  
                                                                       !! @tiex $(gC m^{-2} dt_slow^{-1})$ @endtex  


    !! 0.3 Modified
    REAL(r_std),DIMENSION(npts,nvm),INTENT(inout) :: lai               !! Leaf area inex @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(inout) :: veget_max         !! "maximal" coverage fraction of a PFT on the ground
                                                                       !! May sum to
                                                                       !! less than unity if the pixel has
                                                                       !! nobio area. (unitless, 0-1)
    REAL(r_std),DIMENSION(npts,nvm,npco2), &
                                    INTENT(inout) :: assim_param       !! min+max+opt temperatures (K) & vmax for 
                                                                       !! photosynthesis  
                                                                       !! @tex $(\mu mol m^{-2}s^{-1})$ @endtex  
    REAL(r_std),DIMENSION(npts),INTENT(inout)     :: deadleaf_cover    !! Fraction of soil covered by dead leaves 
                                                                       !! (unitless) 
    REAL(r_std),DIMENSION(npts,nvm,nleafages),&
                                    INTENT(inout) :: frac_age          !! Age efficacity from STOMATE 
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &
                                    INTENT(inout) :: circ_class_biomass !! Stem diameter 
                                                                       !! @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc), &
         INTENT(inout)                            :: circ_class_n      !! Number of trees within each circumference
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &               !! Biomass per PFT
         INTENT(inout)                            :: biomass           !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout) :: ind             !! Number of individuals at the stand level
                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(inout) :: veget_max_new     !! New "maximal" coverage fraction of a PFT (LAI -> 
                                                                       !! infinity) on ground only if EndOfYear is 
                                                                       !! activated. Including nobio fraction. (unitless)  
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)  :: lai_per_level     !! This is the LAI per vertical level
                                                                       !! @tex $(m^{2} m^{-2})$
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(inout) &
                                                  :: laieff_fit        !! Fitted parameters for the effective LAI
    REAL(r_std),DIMENSION(npts),INTENT(inout)     :: temp_growth       !! Growth temperature (°C) 
                                                                       !! Is equal to t2m_month
    REAL(r_std),DIMENSION(npts,nvm),INTENT(inout) :: stressed          !! adjusted ecosystem functioning. Takes the unit of the variable
                                                                       !! used as a proxy for waterstress (assigned in sechiba).
    REAL(r_std),DIMENSION(npts,nvm),INTENT(inout) :: unstressed        !! initial ecosystem functioning after the first calculation and 
                                                                       !! before any recalculations. Takes the unit of the variable used
                                                                       !! as a proxy for unstressed (assigned in sechiba).


    !! 0.4 local variables

    REAL(r_std)                                :: day_counter_read     !! Day read in restart file (days)
    REAL(r_std)                                :: dt_days_read         !! STOMATE time step read in restart file (days)
    INTEGER(i_std)                             :: k, ji, jv, i, j, m   !! indices
    INTEGER(i_std)                             :: ivm, ipts, icir      !! indices    
    INTEGER(i_std)                             :: iele, ipar, imbc     !! Indices
    INTEGER(i_std)                             :: ilitt, ilevs, icarb  !! Indices 
    INTEGER(i_std)                             :: date_read            !! STOMATE date read in restart file (days)
    REAL(r_std),PARAMETER                      :: max_dt_days = 5.     !! Maximum STOMATE time step (days)
    REAL(r_std),DIMENSION(0:nbdl)              :: z_soil               !! Variable to store depth of the different soil 
                                                                       !! layers (m) 
    REAL(r_std),DIMENSION(npts,nvm)            :: rprof                !! Coefficient of the exponential functions that 
                                                                       !! relates root density to soil depth (unitless) 
    REAL(r_std),DIMENSION(npts)                :: precip               !! Total liquid and solid precipitation  
                                                                       !! @tex $(??mm dt_slow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts,nvm)            :: gpp_d                !! Gross primary productivity per ground area 
                                                                       !! @tex $(??gC m^{-2} dt_slow^{-1})$ @endtex  
    REAL(r_std),DIMENSION(npts,nvm)            :: gpp_daily_x          !! "Daily" gpp for teststomate  
                                                                       !! @tex $(??gC m^{-2} dt_slow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts,nvm)            :: resp_hetero_litter   !! Litter heterotrophic respiration per ground area 
                                                                       !! @tex $(gC m^{-2} day^{-1})$ @endtex  
                                                                       !! ??Same variable is also used to 
                                                                       !! store heterotrophic respiration per ground area 
                                                                       !! over ::dtradia?? 
    REAL(r_std),DIMENSION(npts,nvm)            :: resp_hetero_soil     !! soil heterotrophic respiration  
                                                                       !! @tex $(gC m^{-2} day^{-1})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm)            :: veget_cov            !! Fractional coverage: actually share of the pixel 
                                                                       !! covered by a PFT (fraction of ground area), 
                                                                       !! taking into account LAI ??(= grid scale fpc)?? 
    REAL(r_std),DIMENSION(npts,nvm)            :: vcmax                !! Maximum rate of carboxylation
                                                                       !! @tex $(\mumol m^{-2} s^{-1})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm)            :: t_photo_min          !! Min temperature for photosynthesis (K)
    REAL(r_std),DIMENSION(npts,nvm)            :: t_photo_opt          !! Opt temperature for photosynthesis (K)
    REAL(r_std),DIMENSION(npts,nvm)            :: t_photo_max          !! Max temperature for photosynthesis (K)
    REAL(r_std),DIMENSION(npts,nlevs)          :: control_moist_inst   !! Moisture control of heterotrophic respiration 
                                                                       !! (0-1, unitless) 
    REAL(r_std),DIMENSION(npts,nlevs)          :: control_temp_inst    !! Temperature control of heterotrophic 
                                                                       !! respiration, above and below (0-1, unitless) 
    REAL(r_std),DIMENSION(npts,ncarb,nvm)      :: soilcarbon_input_inst!! Quantity of carbon going into carbon pools from 
                                                                       !! litter decomposition 
                                                                       !! @tex $(gC m^{-2} day^{-1})$ @endtex
    INTEGER(i_std)                             :: ier                  !! Check errors in netcdf call (unitless)
    REAL(r_std),SAVE                           :: dt_forcesoil         !! Time step of soil forcing file (days)
    !$OMP THREADPRIVATE(dt_forcesoil)
    INTEGER(i_std),PARAMETER                   :: nparanmax=366        !! Maximum number of time steps per year for 
                                                                       !! forcesoil 
    INTEGER(i_std),SAVE                        :: nparan               !! Number of time steps per year for forcesoil - 
                                                                       !! read from run definition (unitless) 
    !$OMP THREADPRIVATE(nparan)
    INTEGER(i_std),SAVE                        :: nbyear=1             !! Number of years saved for forcesoil - read from 
                                                                       !! run definition (unitless) 
    !$OMP THREADPRIVATE(nbyear)
    INTEGER(i_std),SAVE                        :: iatt                 !! Time step of forcing of soil processes (iatt = 1 
                                                                       !! to ::nparan*::nbyear) 
    !$OMP THREADPRIVATE(iatt)
    INTEGER(i_std),SAVE                        :: iatt_old=1           !! Previous ::iatt
    !$OMP THREADPRIVATE(iatt_old)
    REAL(r_std)                                :: sf_time              !! Intermediate variable to calculate current time 
                                                                       !! step 
    INTEGER(i_std)                             :: max_totsize          !! Memory management - maximum memory size (Mb)
    INTEGER(i_std)                             :: totsize_1step        !! Memory management - memory required to store one 
                                                                       !! time step on one processor (Mb) 
    INTEGER(i_std)                             :: totsize_tmp          !! Memory management - memory required to store one 
                                                                       !! time step on all processors(Mb) 
    REAL(r_std)                                :: xn                   !! How many times have we treated in this forcing 
                                                                       !! state 
    INTEGER(i_std),SAVE                        :: nsfm                 !! Number of time steps that can be stored in 
                                                                       !! memory (unitless) 
    !$OMP THREADPRIVATE(nsfm)
    INTEGER(i_std),SAVE                        :: nsft                 !! Number of time steps in a year (unitless)
    !$OMP THREADPRIVATE(nsft)
    INTEGER(i_std),SAVE                         :: iisf                !! Current pointer for teststomate (unitless)
    !$OMP THREADPRIVATE(iisf)
    REAL(r_std), DIMENSION(npts)               :: vartmp               !! Temporary variable
    CHARACTER(LEN=100), SAVE                   :: forcing_name         !! Name of forcing file 1
    !$OMP THREADPRIVATE(forcing_name)
    CHARACTER(LEN=100), SAVE                   :: Cforcing_name        !! Name of forcing file 2
    !$OMP THREADPRIVATE(Cforcing_name)
    INTEGER(i_std),SAVE                        :: Cforcing_id          !! File identifer of file 2
    !$OMP THREADPRIVATE(Cforcing_id)    
    INTEGER(i_std),PARAMETER                   :: ndm = 10             !! Maximum number of dimensions (unitless)
    INTEGER(i_std)                             :: vid                  !! Variable identifer of netCDF (unitless)
    INTEGER(i_std)                             :: nneigh               !! Number of neighbouring pixels
    INTEGER(i_std)                             :: direct               !! ??
    INTEGER(i_std),DIMENSION(ndm)              :: d_id                 !! ??

    REAL(r_std),DIMENSION(npts,nvm)            :: t_root               !! ??[DISPENSABLE] Root temperature (K) convolution 
                                                                       !! of root and soil temperature profiles 
    REAL(r_std),DIMENSION(npts,nparts)         :: t_maint_radia        !! ??[DISPENSABLE] Temperature which is pertinent 
                                                                       !! for maintenance respiration (K) 
    REAL(r_std),DIMENSION(npts)                :: rpc                  !! ??[DISPENSABLE] Integration constant for root 
                                                                       !! profile (unitless) 
    REAL(r_std),DIMENSION(npts)                :: tl                   !! ??[DISPENSABLE] long term annual mean 
                                                                       !! temperature, C 
    REAL(r_std),DIMENSION(npts)                :: slope                !! ??[DISPENSABLE] slope of maintenance respiration 
                                                                       !! coefficient (1/K) 
    !    INTEGER(i_std)                            :: iyear            !! ??[DISPENSABLE]
    REAL(r_std)                                :: net_co2_flux_monthly !! ??[DISPENSABLE]
    REAL(r_std)                                :: net_co2_flux_monthly_sum !! ??[DISPENSABLE]
    !    INTEGER                                   :: ios              !! ??[DISPENSABLE]
    REAL(r_std)                                :: trans_veg            !! ??[DISPENSABLE]
    REAL(r_std)                                :: tmp_day(1)           !! ??[DISPENSABLE]
!!$    INTEGER(i_std),SAVE                       :: lcanop                !! ??[DISPENSABLE] soil level used for LAI
!!$   !$OMP THREADPRIVATE(lcanop)
    REAL(r_std),DIMENSION(nbp_glo)             :: clay_g               !! Clay fraction of soil (0-1, unitless), parallel 
                                                                       !! computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:,:) :: soilcarbon_input_g   !! Quantity of carbon going into carbon pools from 
                                                                       !! litter decomposition  
                                                                       !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex, parallel 
                                                                       !! computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: control_moist_g      !! Moisture control of heterotrophic respiration 
                                                                       !! (0-1, unitless), parallel computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: control_temp_g       !! Temperature control of heterotrophic respiration 
                                                                       !! (0-1, unitless), parallel computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: npp_equil_g          !! Equilibrium NPP written to forcesoil 
                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex, parallel 
                                                                       !! computing 

    REAL(r_std)                                :: net_cflux_prod_monthly_sum    !! AR5 output?? gC m2 month-1 (one variable for 
                                                                       !! reduce_sum and one for bcast??), parallel 
                                                                       !! computing 
    REAL(r_std)                                :: net_cflux_prod_monthly_tot    !! AR5 output?? gC m2 month-1 (one variable for 
                                                                       !! reduce_sum and one for bcast??), parallel 
                                                                       !! computing 
    REAL(r_std)                                :: net_harvest_above_monthly_sum !! AR5 output?? gC m2 month-1 (one variable for 
                                                                       !! reduce_sum and one for bcast??), parallel 
                                                                       !! computing 
    REAL(r_std)                                :: net_harvest_above_monthly_tot !! AR5 output?? gC m2 month-1 (one variable for 
                                                                       !! reduce_sum and one for bcast??), parallel 
                                                                       !! computing 
    REAL(r_std)                                :: net_biosp_prod_monthly_sum    !! AR5 output?? gC m2 month-1 (one variable for 
                                                                       !! reduce_sum and one for bcast??), parallel 
                                                                       !! computing 
    REAL(r_std)                                :: net_biosp_prod_monthly_tot    !! AR5 output?? gC m2 month-1 (one variable for 
                                                                       !! reduce_sum and one for bcast??), parallel 
                                                                       !! computing 
    REAL(r_std), DIMENSION(npts,nvm,nbpools)   :: carbon_stock         !! Array containing the carbon stock for each pool
                                                                       !! used by ORCHIDEE
    INTEGER(i_std), SAVE                       :: spinup_period        !! Period of years used to calculate the 
                                                                       !! resolution of the system for the spinup analytic. 
                                                                       !! This period correspond in most cases to the 
                                                                       !! period of years of forcing data used
    INTEGER(i_std)                             :: forest_managed_temp  !! Temporary variable to affect forest_managed(1,6)
    REAL(r_std), DIMENSION(npts)               :: laieff_solar_angle_array !! a temporary array
    LOGICAL                                    :: l_error              !! error flag
    REAL(r_std)                                :: temp_total           !! Used for renormalizing
    INTEGER(i_std)                             :: ldnight              !! Flag to only accumulate day values in stomate_accu
    LOGICAL                                    :: lsync                !! returns false if the biomass
                                                                       !! is not synced... do we need this?  
                                                                       !! We could equal just handle it in the 
                                                                       !! check_biomass_sync
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements) &
                                               :: check_intern         !! Contains the components of the internal
                                                                       !! mass balance chech for this routine
                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements) :: closure_intern       !! Check closure of internal mass balance
                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements) :: pool_start           !! Start and end pool of this routine 
                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements) :: pool_end             !! Start and end pool of this routine 
                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts)               :: pixel_area           !! surface area of the pixel @tex ($m^{2}$)
    REAL(r_std), DIMENSION(npts,nvm,norphans,nelements) &
                                               :: orphan_flux          !! Storage for fluxes of PFTs that no longer
                                                                       !! exist. Following the total destruction of 
                                                                       !! a PFT in LCC (veget_max_new = 0), a flux 
                                                                       !! from before LCC needs a storage
                                                                       !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    LOGICAL                                    :: lread_fm_map         !! A logical flag determining if we read
                                                                       !! in the forest management strategy from a map.
    REAL(r_std), DIMENSION (npts,1)            :: temp_array
    REAL(r_std), DIMENSION (npts,1)            :: temp_array_daily
    
    REAL(r_std), DIMENSION(npts,nvm)           :: tau_eff_root         !! Effective root turnover time that accounts
                                                                       !! waterstress (days)
    REAL(r_std), DIMENSION(npts,nvm)           :: tau_eff_sap          !! Effective sapwood turnover time that accounts
                                                                       !! waterstress (days)
    REAL(r_std), DIMENSION(npts,nvm)           :: tau_eff_leaf         !! Effective leaf turnover time that accounts
                                                                       !! waterstress (days)
    REAL(r_std), DIMENSION(npts,nvm)           :: wstress_adapt        !! Factor to account for a long acclimation of
                                                                       !! of the PFT to the long-term waterstress in
                                                                       !! the pixel
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:)  &
                                               :: fm_map_temp          !! A temporary variable to hold the forest
                                                                       !! management map which is read in from a file
                                                                       !! (0-4,unitless)

!_ ================================================================================================================================

    IF (bavard >= 4) THEN
       WRITE(numout,*) 'Entering stomate bavard',l_first_stomate,control%ok_stomate
    ENDIF
    !! 1. Initialize variables
    ! This is an INTENT(OUT), and otherwise some times does not get set.
    co2_flux(:,:) = zero
    

    !! 1.1 Store current time step in a common variable
    itime = kjit

    !![DISPENSABLE] 1.2 Copy the depth of the different soil layers from diaglev specified in slow_proc

    !! 1.3 PFT rooting depth across pixels, humescte is pre-defined 
    ! (constantes_veg.f90). It is defined as the coefficient of an exponential 
    ! function relating root density to depth 
    DO j=1,nvm
       rprof(:,j) = 1./humcste(j)     
    ENDDO

    !! 1.4 Initialize first call
    ! Set growth respiration to zero
    resp_growth(:,:) = zero
 
    IF (l_first_stomate) THEN
       IF (long_print) THEN
          WRITE (numout,*) 'l_first_stomate : call stomate_init'
       ENDIF

       !! 1.4.0 Parameters for spinup
       !
       eps_carbon = 0.01
       !Config Key   = EPS_CARBON
       !Config Desc  = Allowed error on carbon stock
       !Config If    = SPINUP_ANALYTIC
       !Config Def   = 0.01
       !Config Help  = 
       !Config Units = [%]   
       CALL getin_p('EPS_CARBON',eps_carbon)       

       !Config Key   = SPINUP_PERIOD
       !Config Desc  = Period to calulcate equilibrium during spinup analytic
       !Config If    = SPINUP_ANALYTIC
       !Config Def   = -1
       !Config Help  = Period corresponds in most cases to the number of years of forcing data used in the spinup.
       !Config Units = [years]   
       spinup_period = -1
       CALL getin_p('SPINUP_PERIOD',spinup_period)       

       ! Check spinup_period values. 
       ! For periods uptil 6 years, to obtain equilibrium, a bigger period have to be used 
       ! and therefore spinup_period is adjusted to 10 years. 
       IF (spinup_analytic) THEN
          IF (spinup_period <= 0) THEN
             WRITE(numout,*) 'Error in parameter spinup_period. This parameter must be > 0 : spinup_period=',spinup_period
             CALL ipslerr_p (3,'stomate_main', &
                  'Parameter spinup_period must be set to a positive integer.', &
                  'Set this parameter to the number of years of forcing data used for the spinup.', &
                  '')
          ELSE IF (spinup_period <= 6) THEN
             ! Adjust to bigger period. The period must be a multiple of the original period.
             WRITE(numout,*) 'Initial spinup_period =',spinup_period,' will be adjusted.'
             spinup_period = spinup_period*(INT(9/spinup_period)+1)
          END IF
          WRITE(numout,*) 'Spinup analytic is activated using eps_carbon=',eps_carbon, ' and spinup_period=',spinup_period
       END IF

       !! 1.4.0 Initialization of PFT specific parameters 
       ! Initialization of PFT specific parameters that have no value
       ! for the bare soil PFT i.e. fire resistance, flamability, maximum lai,
       ! settings for growing degree days (GDD), settings for senescence, 
       ! respiration coefficients, photosynthesis, etc.
       ! [DISPENSABLE]

       !! 1.4.1 Allocate memory for all variables in stomate
       ! Allocate memory for all variables in stomate, build new index
       ! tables accounting for the PFTs, read and check flags and set file
       ! identifier for restart and history files.
       CALL stomate_init (kjpij, npts, index, ldforcing_write, lalo, &
            rest_id_stom, hist_id_stom, hist_id_stom_IPCC)

!       WRITE(numout,*) 'returned from stomate_init'

       !! 1.4.2 Initialization of PFT specific parameters
       ! Initialization of PFT specific parameters i.e. sla from leaf life, 
       ! sapling characteristics (biomass), migration speed, critical diameter,
       ! coldest tolerable temperature, critical values for phenology, maximum
       ! life time of leaves, respiration coefficients and photosynthesis.
       ! The subroutine also communicates settings read by stomate_constant_init.

       CALL data
       !       CALL data (npts, lalo)

       !! 1.4.3 Initial conditions

       !! 1.4.3.1 Read initial values for STOMATE's variables from the _restart_ file
       ! ??Shouldn't this be included in stomate_init?? Looks like an initialization!
       fco2_lu(:) = zero

       !! 1.4.3.2 Read the management status of (i,j) (has to be done before readrestart)
       ! this is the first call in this loop, so why is this check here?  It should just
       ! be initialised...otherwise the check is over uninitialized values,

       forest_managed_lastyear(:,:) = 0

       !Config Key   = FOREST_MANAGED
       !Config Desc  =
       !Config If    = OK_STOMATE
       !Config Def   = 0
       !Config Help  =
       !Config Units =
       !  Here we use different defaults for the case of functional
       ! allocation and resource limited allocation, since using
       ! functional allocation means that forestry is always actived.
       ! However, a setting of 1 means don't do any human management.
       IF(control%ok_functional_allocation)THEN

          forest_managed_temp=1

       ELSE

          forest_managed_temp=0

       ENDIF
       CALL getin_p('FOREST_MANAGED_FORCED',forest_managed_temp)
       forest_managed(:,:) = forest_managed_temp

       ! We need to have the option to read the forest management
       ! strategy from a map (NetCDF file).  If this option is
       ! equal to Y, we will overwrite the forest_managed_forced 
       ! option above, so you should be careful to only use one
       ! or the other.  
       !Config Key   = READ_FM_MAP
       !Config Desc  =
       !Config If    = OK_STOMATE
       !Config Def   = N
       !Config Help  =
       !Config Units =
       lread_fm_map=.FALSE.
       CALL getin_p('READ_FM_MAP',lread_fm_map)
       IF(lread_fm_map)THEN
          
          ! If we are using age classes, we read in the map in the same
          ! way but then we change it a bit to account for age classes.
          l_error = .FALSE.
          ALLOCATE(fm_map_temp(npts,nvm),stat=ier)
          l_error = l_error .OR. (ier /= 0)
          IF (l_error) THEN
             WRITE(numout,*) 'Problem with memory allocation: temporary FM map ',npts,nvmap
             STOP 'stomate_main'
          ENDIF

          CALL read_in_fm_map(npts, lalo, neighbours, resolution, &
               contfrac, fm_map_temp)

          IF(nagec .GT. 1)THEN
             ! All age classes of the same PFT will have the same
             ! management
             DO ivm = 1,nvm
                forest_managed(:,ivm)=fm_map_temp(:,agec_group(ivm))
             ENDDO

          ELSE
             forest_managed(:,:)=fm_map_temp(:,:)
          ENDIF

          DEALLOCATE(fm_map_temp)


       ENDIF

       ! If we are doing litter raking, we need a map which
       ! tells us how much litter is required at every pixel.
       ! This number is independent of the PFTs.  
       IF(use_litter_raking)THEN
          
          CALL read_in_litter_map(npts, lalo, neighbours, resolution, &
               contfrac, litter_demand)

       ENDIF

       ! Get values from _restart_ file. Note that only ::npts, ::index, ::lalo 
       ! and ::resolution are input variables, all others are output variables.
       CALL readrestart (npts, index, lalo, resolution, neighbours, contfrac, &
            day_counter_read, dt_days_read, date_read, ind, & 
            adapted, regenerate, humrel_daily, gdd_init_date, &
            litterhum_daily, t2m_daily, t2m_min_daily, tsurf_daily, &
            tsoil_daily, soilhum_daily, precip_daily, gpp_daily, &
            npp_daily, turnover_daily, humrel_month, humrel_week, &
            humrel_growingseason, t2m_longterm, tlong_ref, t2m_month, &
            t2m_week, tsoil_month, soilhum_month, fireindex, &
            firelitter, maxhumrel_lastyear, maxhumrel_thisyear, minhumrel_lastyear, &
            minhumrel_thisyear, maxgppweek_lastyear, maxgppweek_thisyear, gdd0_lastyear, &
            gdd0_thisyear, precip_lastyear, precip_thisyear, gdd_m5_dormance, &
            gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
            PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
            maxfpc_lastyear, maxfpc_thisyear, turnover_longterm, gpp_week, &
            biomass, resp_maint_part, leaf_age, leaf_frac, &
            senescence, senescence_days, when_growthinit, age, &
            resp_hetero_d, resp_maint_d, resp_growth_d, co2_fire, &
            co2_to_bm_dgvm, veget_lastlight, everywhere, need_adjacent, &
            RIP_time, time_lowgpp, time_hum_min, hum_min_dormance, &
            litterpart, litter, dead_leaves, carbon, &
            black_carbon, lignin_struc, lignin_wood, turnover_time, &
            prod_s, prod_m, prod_l, flux_s, flux_m, flux_l, &
            cflux_prod_s, cflux_prod_m, cflux_prod_l, bm_to_litter, &
            carb_mass_total, &
            age_stand, rotation_n, last_cut, &
            forest_managed, forest_managed_lastyear, global_years, &
            ok_equilibrium, nbp_accu, nbp_flux, MatrixV, &
            VectorU, previous_stock, current_stock, rue_longterm, &
            fpc_max, circ_class_n, circ_class_biomass, KF, &
            mai, pai, previous_wood_volume, mai_count, coppice_dens, &
            vir_humrel_growingseason, lai_per_level, laieff_fit, temp_growth, &
            wstress_season, wstress_month, vir_wstress_fac, c_buried, &
            k_latosa_adapt)

       !---TEMP---
       CALL check_biomass_sync("stomate.f90: After readstart", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)            

       !! 1.4.4 If the vegetation is dynamic
       ! If the vegetation is dynamic, long-term reference temperature was 
       ! read by the call above. If vegetation is static then the long-term 
       ! reference temperature is a boundary condition and read here.  
       CALL readbc (npts, lalo, neighbours, resolution, contfrac, tlong_ref)

       !! 1.4.5 Check time step

       !! 1.4.5.1 Allow STOMATE's time step to change although this is dangerous
       IF (dt_days /= dt_days_read) THEN
          WRITE(numout,*) 'slow_processes: STOMATE time step changes:', &
               & dt_days_read,' -> ',dt_days
       ENDIF

       !! 1.4.5.2 Time step has to be a multiple of a full day
       IF ( ( dt_days-REAL(NINT(dt_days),r_std) ) > min_stomate ) THEN
          WRITE(numout,*) 'slow_processes: STOMATE time step is not a mutiple of a full day:', &
               & dt_days,' days.'
          STOP
       ENDIF

       !! 1.4.5.3 upper limit to STOMATE's time step
       IF ( dt_days > max_dt_days ) THEN
          WRITE(numout,*) 'slow_processes: STOMATE time step exceeds the maximum value:', &
               & dt_days,' days > ', max_dt_days, ' days.'  
          STOP
       ENDIF

       !! 1.4.5.4 STOMATE time step must not be less than the forcing time step
       IF ( dtradia > dt_days*one_day ) THEN
          WRITE(numout,*) &
               & 'slow_processes: STOMATE time step ::dt_days smaller than forcing time step ::dtradia'
          STOP
       ENDIF

       !! 1.4.5.5 For teststomate : test day_counter
       IF ( abs(day_counter - day_counter_read) > min_stomate ) THEN
          WRITE(numout,*) 'slow_processes: STOMATE day counter changes:', &
               day_counter_read,' -> ',day_counter
       ENDIF

       !! 1.4.5.6 Date check
       IF (date /= date_read) THEN
          WRITE(numout,*) 'Slow_processes: STOMATE date changes:', &
               date_read,' -> ',date
       ENDIF

       !! 1.4.5.6 Final message on time step
       WRITE(numout,*) 'Slow_processes, STOMATE time step (days): ', dt_days

       !! 1.4.6 Write forcing file for stomate?
       IF (ldforcing_write) THEN

          !Config Key   = STOMATE_FORCING_NAME
          !Config Desc  = Name of STOMATE's forcing file
          !Config If    = OK_STOMATE
          !Config Def   = NONE
          !Config Help  = Name that will be given
          !Config         to STOMATE's offline forcing file
          !Config         Compatible with Nicolas Viovy's driver
          !Config Units = [FILE]
          forcing_name = stomate_forcing_name
          CALL getin_p('STOMATE_FORCING_NAME',forcing_name)

          IF ( TRIM(forcing_name) /= 'NONE' ) THEN

             !! 1.4.6.1 Calculate steps that can be stored in memory
             ! Action for the root processor only (parallel computing)  
             IF (is_root_prc) CALL SYSTEM ('rm -f '//TRIM(forcing_name))
             WRITE(numout,*) 'writing a forcing file for STOMATE.'

             !Config Key   = STOMATE_FORCING_MEMSIZE
             !Config Desc  = Size of STOMATE forcing data in memory 
             !Config If    = OK_STOMATE
             !Config Def   = 50
             !Config Help  = This variable determines how many
             !Config         forcing states will be kept in memory.
             !Config         Must be a compromise between memory
             !Config         use and frequeny of disk access.
             !Config Units = [MegaBytes]
             max_totsize = 50
             CALL getin_p('STOMATE_FORCING_MEMSIZE', max_totsize)      
             max_totsize = max_totsize*1000000

             totsize_1step = &
                  &      SIZE(clay)*KIND(clay) &
                  &           +SIZE(humrel_daily)*KIND(humrel_daily) &
                  &     +SIZE(litterhum_daily)*KIND(litterhum_daily) &
                  &     +SIZE(t2m_daily)*KIND(t2m_daily) &
                  &     +SIZE(t2m_min_daily)*KIND(t2m_min_daily) &
                  &     +SIZE(tsurf_daily)*KIND(tsurf_daily) &
                  &     +SIZE(tsoil_daily)*KIND(tsoil_daily) &
                  &     +SIZE(soilhum_daily)*KIND(soilhum_daily) &
                  &     +SIZE(precip_daily)*KIND(precip_daily) &
                  &     +SIZE(gpp_daily_x)*KIND(gpp_daily_x) &
                  &     +SIZE(veget)*KIND(veget) &
                  &     +SIZE(veget_max)*KIND(veget_max) &
                  &     +SIZE(lai)*KIND(lai)

             ! Totsize_1step is the size on a single processor, sum
             ! all processors and send to all processors
             CALL reduce_sum(totsize_1step,totsize_tmp)
             CALL bcast(totsize_tmp)
             totsize_1step=totsize_tmp

             ! Total number of forcing steps
             nsft = INT(one_year/(dt_slow/one_day))

             ! Number of forcing steps in memory
             nsfm = MIN(nsft, &
                  &       MAX(1,NINT( REAL(max_totsize,r_std) &
                  &                  /REAL(totsize_1step,r_std))))


	     !! 1.6.4.2 Allocate memory for variables containing forcing data  
             ! and initialize variables (set to zero).
             CALL init_forcing (npts,nsfm,nsft)

             ! Indexing for writing forcing file
             isf(:) = (/ (i,i=1,nsfm) /)
             nf_written(:) = .FALSE.
             nf_cumul(:) = 0
             iisf = 0

             !! 1.6.4.3 Create netcdf file
             ! Create, define and populate a netcdf file containing the forcing data.
             ! For the root processor only (parallel computing). NF90_ are functions
             ! from and external library.  
             IF (is_root_prc) THEN

                ! Create new netCDF dataset
                ier = NF90_CREATE (TRIM(forcing_name),NF90_SHARE,forcing_id)

                ! Add variable attribute
                ! Note ::iim_g and ::jjm_g are dimensions of the global field and 
                ! ::nbp_glo is the number of global continental points
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL,'dtradia',dtradia)
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL,'dt_slow',dt_slow)
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                     & 'nsft',REAL(nsft,r_std))
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                     & 'kjpij',REAL(iim_g*jjm_g,r_std))
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                     & 'npts',REAL(nbp_glo,r_std))

                ! Add new dimension
                ier = NF90_DEF_DIM (forcing_id,'points',nbp_glo,d_id(1))
                ier = NF90_DEF_DIM (forcing_id,'layers',nbdl,d_id(2))
                ier = NF90_DEF_DIM (forcing_id,'pft',nvm,d_id(3))
                direct=2
                ier = NF90_DEF_DIM (forcing_id,'direction',direct,d_id(4))
                nneigh=8
                ier = NF90_DEF_DIM (forcing_id,'nneigh',nneigh,d_id(5))
                ier = NF90_DEF_DIM (forcing_id,'time',NF90_UNLIMITED,d_id(6))
                ier = NF90_DEF_DIM (forcing_id,'nbparts',nparts,d_id(7))

                ! Add new variable
                ier = NF90_DEF_VAR (forcing_id,'points',    r_typ,d_id(1),vid)
                ier = NF90_DEF_VAR (forcing_id,'layers',    r_typ,d_id(2),vid)
                ier = NF90_DEF_VAR (forcing_id,'pft',       r_typ,d_id(3),vid)
                ier = NF90_DEF_VAR (forcing_id,'direction', r_typ,d_id(4),vid)
                ier = NF90_DEF_VAR (forcing_id,'nneigh',    r_typ,d_id(5),vid)
                ier = NF90_DEF_VAR (forcing_id,'time',      r_typ,d_id(6),vid)
                ier = NF90_DEF_VAR (forcing_id,'nbparts',   r_typ,d_id(7),vid)
                ier = NF90_DEF_VAR (forcing_id,'index',     r_typ,d_id(1),vid)
                ier = NF90_DEF_VAR (forcing_id,'contfrac',  r_typ,d_id(1),vid) 
                ier = NF90_DEF_VAR (forcing_id,'lalo', &
                     & r_typ,(/ d_id(1),d_id(4) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'neighbours', &
                     & r_typ,(/ d_id(1),d_id(5) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'resolution', &
                     & r_typ,(/ d_id(1),d_id(4) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'clay', &
                     & r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'humrel', &
                     & r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'litterhum', &
                     & r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'t2m', &
                     & r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'t2m_min', &
                     & r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'tsurf', &
                     & r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'tsoil', &
                     & r_typ,(/ d_id(1),d_id(2),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'soilhum', &
                     & r_typ,(/ d_id(1),d_id(2),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'precip', &
                     & r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'gpp', &
                     & r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'veget', &
                     & r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'veget_max', &
                     & r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'lai', &
                     & r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'sinang', &
                     & r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_ENDDEF (forcing_id)

		! Given the name of a varaible, nf90_inq_varid finds the variable 
                ! ID (::vid). Put data value(s) into variable ::vid
                ier = NF90_INQ_VARID (forcing_id,'points',vid)
                ier = NF90_PUT_VAR (forcing_id,vid, &
                     & (/(REAL(i,r_std),i=1,nbp_glo) /))
                ier = NF90_INQ_VARID (forcing_id,'layers',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nbdl)/))
                ier = NF90_INQ_VARID (forcing_id,'pft',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nvm)/))
                ier = NF90_INQ_VARID (forcing_id,'direction',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,2)/))
                ier = NF90_INQ_VARID (forcing_id,'nneigh',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,8)/))
                ier = NF90_INQ_VARID (forcing_id,'time',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nsft)/))
                ier = NF90_INQ_VARID (forcing_id,'nbparts',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nparts)/))
                ier = NF90_INQ_VARID (forcing_id,'index',vid)  
                ier = NF90_PUT_VAR (forcing_id,vid,REAL(index_g,r_std))
                ier = NF90_INQ_VARID (forcing_id,'contfrac',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,REAL(contfrac_g,r_std))
                ier = NF90_INQ_VARID (forcing_id,'lalo',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,lalo_g)
                !ym attention a neighbours, a modifier plus tard      
                ier = NF90_INQ_VARID (forcing_id,'neighbours',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,REAL(neighbours_g,r_std))
                ier = NF90_INQ_VARID (forcing_id,'resolution',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,resolution_g)
             ENDIF ! is_root_prc
          ENDIF ! (forcing_name) /= 'NONE'
       ENDIF ! ldforcing_write =.TRUE.

       !! 1.4.7 write forcing file for the soil?
       IF (ldcarbon_write) THEN

          !! 1.4.7.1 Initialize
          !Config Key   = STOMATE_CFORCING_NAME
          !Config Desc  = Name of STOMATE's carbon forcing file
          !Config If    = OK_STOMATE
          !Config Def   = NONE
          !Config Help  = Name that will be given to STOMATE's carbon
          !Config         offline forcing file
          !Config         Compatible with Nicolas Viovy's driver
          !Config Units = [FILE]
          Cforcing_name = stomate_Cforcing_name
          CALL getin_p('STOMATE_CFORCING_NAME',Cforcing_name)

          IF ( TRIM(Cforcing_name) /= 'NONE' ) THEN

             ! Time step of forcesoil
             !Config Key   = FORCESOIL_STEP_PER_YEAR
             !Config Desc  = Number of time steps per year for carbon spinup.
             !Config If    = OK_STOMATE
             !Config Def   = 365
             !Config Help  = Number of time steps per year for carbon spinup.
             !Config Units = [days, months, year]
             nparan = 365
             CALL getin_p('FORCESOIL_STEP_PER_YEAR', nparan)

             ! Correct if setting is out of bounds 
             IF ( nparan < 1 ) nparan = 1

             !Config Key   = FORCESOIL_NB_YEAR
             !Config Desc  = Number of years saved for carbon spinup.
             !Config If    = OK_STOMATE
             !Config Def   = 1
             !Config Help  = Number of years saved for carbon spinup. If internal parameter cumul_Cforcing is TRUE in stomate.f90
             !Config         Then this parameter is forced to one.
             !Config Units = [years]
             CALL getin_p('FORCESOIL_NB_YEAR', nbyear)

             ! Set ::nbyear to 1. if ::cumul_Cforcing=.TRUE.
             IF ( cumul_Cforcing ) THEN
                CALL ipslerr_p (1,'stomate', &
                     &          'Internal parameter cumul_Cforcing is TRUE in stomate.f90', &
                     &          'Parameter FORCESOIL_NB_YEAR is therefore forced to 1.', &
                     &          '::nbyear is thus set to 1.')
                nbyear=1
             ENDIF

             ! Make use of ::nparan to calculate ::dt_forcesoil
             dt_forcesoil = zero
             nparan = nparan+1
             DO WHILE ( dt_forcesoil < dt_slow/one_day )
                nparan = nparan-1
                IF ( nparan < 1 ) THEN
                   STOP 'Problem with number of soil forcing time steps ::nparan < 1.'
                ENDIF
                dt_forcesoil = one_year/REAL(nparan,r_std)
             ENDDO
             IF ( nparan > nparanmax ) THEN
                STOP 'Problem with number of soil forcing time steps ::nparan > ::nparanmax'
             ENDIF
             WRITE(numout,*) 'Time step of soil forcing (d): ',dt_forcesoil

             ! Allocate memory for the forcing variables of soil dynamics
             ALLOCATE( nforce(nparan*nbyear))
             nforce(:) = 0
             ALLOCATE(control_moist(npts,nlevs,nparan*nbyear))
             ALLOCATE(npp_equil(npts,nparan*nbyear))
             ALLOCATE(npp_tot(npts))
             ALLOCATE(control_temp(npts,nlevs,nparan*nbyear))
             ALLOCATE(soilcarbon_input(npts,ncarb,nvm,nparan*nbyear)) 

             ! Initialize variables, set to zero
             control_moist(:,:,:) = zero
             npp_equil(:,:) = zero
             npp_tot(:) = zero
             control_temp(:,:,:) = zero
             soilcarbon_input(:,:,:,:) = zero

          ENDIF ! Cforcing_name) /= 'NONE'
       ENDIF ! ::ldcarbon_write!.TRUE.

       !! 1.4.8 Calculate STOMATE's vegetation fractions from veget, veget_max
       DO j=1,nvm
          WHERE ((1.-totfrac_nobio(:)) > min_sechiba)       
             ! Pixels with vegetation
             veget_cov(:,j) = veget(:,j)/( 1.-totfrac_nobio(:) )
             veget_cov_max(:,j) = veget_max(:,j)/( 1.-totfrac_nobio(:) )
          ELSEWHERE
             ! Pixels without vegetation
             veget_cov(:,j) = zero
             veget_cov_max(:,j) = zero
          ENDWHERE
       ENDDO ! Loop over PFTs
       IF ( EndOfYear ) THEN
          DO j=1,nvm
             WHERE ((1.-totfrac_nobio_new(:)) > min_sechiba)
                ! Update pixels with vegetation
                veget_cov_max_new(:,j) = veget_max_new(:,j)/( 1.-totfrac_nobio_new(:) )
             ELSEWHERE
                ! Update pixels without vegetation
                veget_cov_max_new(:,j) = zero
             ENDWHERE
          ENDDO
       ENDIF

       !! 1.4.9 Initialize non-zero variables
       IF (control%ok_stomate) THEN
          CALL stomate_var_init &
               &         (npts, veget_cov_max, leaf_age, leaf_frac, &
               &          tlong_ref, t2m_month, dead_leaves, &
               &          forest_managed, age_stand, &
               &          deadleaf_cover, assim_param)


          !! Calculate the initial effective lai
          !  Convert the 3-D physiological 'true' lai to an effective lai for the 1-D
          !  two stream radiation model...this only needs to be done if we are
          !  using the two stream model
!!!! this cannot be done here because the diameter has not yet been calculated
!!! in stomatelpj

          SELECTCASE (albedo_type)
          CASE ('pinty')
!             CALL effective_lai (npts, lai, biomass, ind, laieff_isotrop)

             ! This is now down in the restart
             !CALL laieff_type_init(npts, nlevels_tot, laieff_fit)
             !             circ_class_biomass_out=circ_class_biomass
             !             circ_class_n_out=circ_class_n
          END SELECT

          ! This is a variable read in from the input file.  The problem is
          ! that is depends on ncirc, which is also read in from the input
          ! file, so I can't allocate it in the usual place (pft_parameters.f90).
          ! So I will allocate it and deallocate it here, since it is only
          ! used by stomate.
          l_error = .FALSE.

          ALLOCATE(circ_class_dist(ncirc),stat=ier)
          l_error = l_error .OR. (ier /= 0)
          IF (l_error) THEN
             WRITE(numout,*) ' Memory allocation error for circ_class_dist. ' // &
                  'We stop. We need ncirc words = ',ncirc
             STOP 'stomate'
          END IF
          !Config Key   = CIRC_CLASS_DIST
          !Config Desc  = The tree distribution to target if we have to redistribute.
          !Config if    = OK_STOMATE
          !Config Def   = 1
          !Config Help  =
          !Config Units = [-]
          circ_class_dist(:)=un
          CALL getin_p('CIRC_CLASS_DIST',circ_class_dist)

          ! Now we normalize the distribution
          temp_total=SUM(circ_class_dist(:))
          circ_class_dist(:)=circ_class_dist(:)/temp_total
          WRITE(numout,*) 'Target distribution for renormalization: ',circ_class_dist(:)

          ALLOCATE(st_dist(ncirc),stat=ier)
          l_error = l_error .OR. (ier /= 0)
          IF (l_error) THEN
             WRITE(numout,*) ' Memory allocation error for st_dist. ' // &
                  'We stop. We need ncirc words = ',ncirc
             STOP 'stomate'
          END IF
          !Config Key   = ST_DIST
          !Config Desc  = The distribution for killing trees in self-thinning.
          !Config if    = OK_STOMATE
          !Config Def   = circ_class_dist
          !Config Help  =
          !Config Units = [-]
          st_dist(:)=circ_class_dist(:)
          CALL getin_p('ST_DIST',st_dist)
 
          ! Now we normalize the distribution
          temp_total=SUM(st_dist(:))
          st_dist(:)=st_dist(:)/temp_total
          WRITE(numout,*) 'Target distribution for self-thinning: ',st_dist(:)


       ENDIF

       !! 1.4.10 Update flag
       l_first_stomate = .FALSE.

       RETURN ! Out of stomate main
    ENDIF  ! First call

    IF (bavard >= 0 .AND. ld_date) THEN
       WRITE(numout,*) 'stomate_main: date=',date,' ymds=', year, month, day, sec, ' itime=', itime, ' do_slow=',do_slow
    ENDIF


    !! 2. Prepares restart file for the next simulation

    !! 2.1 Write restart file for stomate
    IF (ldrestart_write) THEN
       IF (long_print) THEN
          WRITE (numout,*) &
               &      ' Write a restart file for STOMATE'
       ENDIF

      CALL writerestart (npts, index, day_counter, dt_days, &
            date, ind, adapted, regenerate, &
            humrel_daily, gdd_init_date, litterhum_daily, t2m_daily, &
            t2m_min_daily, tsurf_daily, tsoil_daily, soilhum_daily, &
            precip_daily, gpp_daily, npp_daily, turnover_daily, &
            humrel_month, humrel_week, humrel_growingseason, t2m_longterm, &
            tlong_ref, t2m_month, t2m_week, tsoil_month, &
            soilhum_month, fireindex, firelitter, maxhumrel_lastyear, &
            maxhumrel_thisyear, minhumrel_lastyear, minhumrel_thisyear, maxgppweek_lastyear, &
            maxgppweek_thisyear, gdd0_lastyear, gdd0_thisyear, precip_lastyear, &
            precip_thisyear, gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, &
            ncd_dormance, ngd_minus5, PFTpresent, npp_longterm, &
            lm_lastyearmax, lm_thisyearmax, maxfpc_lastyear, maxfpc_thisyear, &
            turnover_longterm, gpp_week, biomass, resp_maint_part, &
            leaf_age, leaf_frac, senescence, senescence_days, &
            when_growthinit, age, resp_hetero_d, resp_maint_d, &
            resp_growth_d, co2_fire, co2_to_bm_dgvm, veget_lastlight, &
            everywhere, need_adjacent, RIP_time, time_lowgpp, &
            time_hum_min, hum_min_dormance, litterpart, litter, &
            dead_leaves, carbon, black_carbon, lignin_struc, & 
            lignin_wood, turnover_time, prod_s, prod_m, prod_l, &
            flux_s, flux_m, flux_l, cflux_prod_s, cflux_prod_m, &
            cflux_prod_l, bm_to_litter, carb_mass_total, &
            age_stand, &
            rotation_n, last_cut, forest_managed, forest_managed_lastyear, &
            global_years, ok_equilibrium, nbp_accu, &
            nbp_flux, MatrixV, VectorU, previous_stock, &
            current_stock, rue_longterm, fpc_max, &
            circ_class_n, circ_class_biomass, KF, &
            mai, pai, previous_wood_volume, &
            mai_count, coppice_dens, vir_humrel_growingseason, lai_per_level,&
            laieff_fit, temp_growth, wstress_season, wstress_month, &
            vir_wstress_fac, c_buried, k_latosa_adapt)

       !! 2.2 Write file with variables that force general processes in stomate
       IF (ldforcing_write .AND. TRIM(forcing_name) /= 'NONE' ) THEN  
          CALL forcing_write(forcing_id,1,iisf)
          ! Close forcing file
          IF (is_root_prc) ier = NF90_CLOSE (forcing_id)
          forcing_id=-1
       ENDIF

       !! 2.3 Collect variables that force the soil processes in stomate
       IF (ldcarbon_write .AND. TRIM(Cforcing_name) /= 'NONE' ) THEN 

          !! 2.3.1 Collect variables 
          WRITE(numout,*) &
               &      'stomate: writing the forcing file for carbon spinup'
          DO iatt = 1, nparan*nbyear
             IF ( nforce(iatt) > 0 ) THEN
                soilcarbon_input(:,:,:,iatt) = &
                     & soilcarbon_input(:,:,:,iatt)/REAL(nforce(iatt),r_std)
                control_moist(:,:,iatt) = &
                     & control_moist(:,:,iatt)/REAL(nforce(iatt),r_std)
                control_temp(:,:,iatt) = &
                     & control_temp(:,:,iatt)/REAL(nforce(iatt),r_std)
                npp_equil(:,iatt) = &
                     & npp_equil(:,iatt)/REAL(nforce(iatt),r_std)
             ELSE
                WRITE(numout,*) &
                     &         'We have no soil carbon forcing data for this time step:', &
                     &         iatt
                WRITE(numout,*) ' -> we set them to zero'
                soilcarbon_input(:,:,:,iatt) = zero
                control_moist(:,:,iatt) = zero
                control_temp(:,:,iatt) = zero
                npp_equil(:,iatt) = zero
             ENDIF
          ENDDO

          ! Allocate memory for parallel computing
          IF (is_root_prc) THEN
             ALLOCATE(soilcarbon_input_g(nbp_glo,ncarb,nvm,nparan*nbyear))
             ALLOCATE(control_moist_g(nbp_glo,nlevs,nparan*nbyear))
             ALLOCATE(control_temp_g(nbp_glo,nlevs,nparan*nbyear))
             ALLOCATE(npp_equil_g(nbp_glo,nparan*nbyear))
          ENDIF

          ! Gather distributed variables
          CALL gather(clay,clay_g)
          CALL gather(soilcarbon_input,soilcarbon_input_g)
          CALL gather(control_moist,control_moist_g)
          CALL gather(control_temp,control_temp_g)
          CALL gather(npp_equil,npp_equil_g)

          !! 2.3.2 Create netcdf
          ! Create, define and populate a netcdf file containing the forcing data.
          ! For the root processor only (parallel computing). NF90_ are functions
          ! from and external library.  
          IF (is_root_prc) THEN
             WRITE (numout,*) 'Create Cforcing file : ',TRIM(Cforcing_name)
             ! Create new netCDF dataset
             ier = NF90_CREATE (TRIM(Cforcing_name),NF90_64BIT_OFFSET ,Cforcing_id)
             IF (ier /= NF90_NOERR) THEN
                WRITE (numout,*) 'Error in creating Cforcing file : ',TRIM(Cforcing_name)
                CALL ipslerr_p (3,'stomate_main', &
                     &        'PROBLEM creating Cforcing file', &
                     &        NF90_STRERROR(ier),'')
             END IF

             ! Add variable attribute
             ! Note ::nbp_glo is the number of global continental points
             ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
                  &                        'npts',REAL(nbp_glo,r_std))
             ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
                  &                        'nparan',REAL(nparan,r_std))
             ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
                  &                        'nbyear',REAL(nbyear,r_std))

             ! Add new dimension
             ier = NF90_DEF_DIM (Cforcing_id,'points',nbp_glo,d_id(1))
             ier = NF90_DEF_DIM (Cforcing_id,'carbtype',ncarb,d_id(2))
             ier = NF90_DEF_DIM (Cforcing_id,'vegtype',nvm,d_id(3))
             ier = NF90_DEF_DIM (Cforcing_id,'level',nlevs,d_id(4))
             ier = NF90_DEF_DIM (Cforcing_id,'time_step',NF90_UNLIMITED,d_id(5))

             ! Add new variable
             ier = NF90_DEF_VAR (Cforcing_id,'points',    r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'carbtype',  r_typ,d_id(2),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'vegtype',   r_typ,d_id(3),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'level',     r_typ,d_id(4),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'time_step', r_typ,d_id(5),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'index',     r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'clay',      r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'soilcarbon_input',r_typ, &
                  &                        (/ d_id(1),d_id(2),d_id(3),d_id(5) /),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'control_moist',r_typ, &
                  &                        (/ d_id(1),d_id(4),d_id(5) /),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'control_temp',r_typ, &
                  &                        (/ d_id(1),d_id(4),d_id(5) /),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'npp_equil',r_typ, &
                  &                        (/ d_id(1),d_id(5) /),vid)
             ier = NF90_ENDDEF (Cforcing_id)

             ! Given the name of a varaible, nf90_inq_varid finds the variable 
             ! ID (::vid). Put data value(s) into variable ::vid 
             ier = NF90_INQ_VARID (Cforcing_id,'points',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                          (/(REAL(i,r_std),i=1,nbp_glo)/))
             ier = NF90_INQ_VARID (Cforcing_id,'carbtype',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                        (/(REAL(i,r_std),i=1,ncarb)/))
             ier = NF90_INQ_VARID (Cforcing_id,'vegtype',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                            (/(REAL(i,r_std),i=1,nvm)/))
             ier = NF90_INQ_VARID (Cforcing_id,'level',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                          (/(REAL(i,r_std),i=1,nlevs)/))
             ier = NF90_INQ_VARID (Cforcing_id,'time_step',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                          (/(REAL(i,r_std),i=1,nparan*nbyear)/))
             ier = NF90_INQ_VARID (Cforcing_id,'index',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, REAL(index_g,r_std) )
             ier = NF90_INQ_VARID (Cforcing_id,'clay',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, clay_g )
             ier = NF90_INQ_VARID (Cforcing_id,'soilcarbon_input',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, soilcarbon_input_g )
             ier = NF90_INQ_VARID (Cforcing_id,'control_moist',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, control_moist_g )
             ier = NF90_INQ_VARID (Cforcing_id,'control_temp',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, control_temp_g )
             ier = NF90_INQ_VARID (Cforcing_id,'npp_equil',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, npp_equil_g )

             ! Close netCDF
             ier = NF90_CLOSE (Cforcing_id)
             IF (ier /= NF90_NOERR) THEN
                CALL ipslerr_p (3,'stomate_main', &
                     &        'PROBLEM in closing Cforcing file', &
                     &        NF90_STRERROR(ier),'')
             END IF

             Cforcing_id = -1
          ENDIF

          ! Clear memory
          IF (is_root_prc) THEN
             DEALLOCATE(soilcarbon_input_g)
             DEALLOCATE(control_moist_g)
             DEALLOCATE(control_temp_g)
             DEALLOCATE(npp_equil_g)
          ENDIF

       ENDIF ! ldcarbon_write=.TRUE.
       RETURN ! This routine is only called at the last time step to write a netcdf for forcesoil, 
       ! the rest of the code does not need to be executed.

    ENDIF  ! write restart-files


!! 3. Special treatment for some input arrays.
    
    !! 3.1 Sum of liquid and solid precipitation
    precip(:) = ( precip_rain(:) + precip_snow(:) )*one_day/dtradia

    !! 3.2 Calculate STOMATE's vegetation fractions from veget and veget_max
    DO j=1,nvm 
       WHERE ((1.-totfrac_nobio(:)) > min_sechiba)
          ! Pixels with vegetation
          veget_cov(:,j) = veget(:,j)/( 1.-totfrac_nobio(:) )
          veget_cov_max(:,j) = veget_max(:,j)/( 1.-totfrac_nobio(:) )
       ELSEWHERE
          ! Pixels without vegetation
          veget_cov(:,j) = zero
          veget_cov_max(:,j) = zero
       ENDWHERE
    ENDDO
    IF ( EndOfYear ) THEN
       DO j=1,nvm
          WHERE ((1.-totfrac_nobio(:)) > min_sechiba)
             ! Pixels with vegetation
             veget_cov_max_new(:,j)=veget_max_new(:,j)/( 1.-totfrac_nobio(:) )
          ELSEWHERE
             ! Pixels without vegetation
             veget_cov_max_new(:,j) = zero
          ENDWHERE
       ENDDO
    ENDIF

    !! 3.3 Adjust time step of GPP 
    ! No GPP for bare soil
    gpp_d(:,1) = zero
    ! GPP per PFT
    DO j = 2,nvm   
       WHERE (veget_cov_max(:,j) > min_stomate)
          ! The PFT is available on the pixel
          gpp_d(:,j) =  gpp(:,j)/ veget_cov_max(:,j)* one_day/dtradia  
       ELSEWHERE
          ! The PFT is absent on the pixel
          gpp_d(:,j) = zero
       ENDWHERE
    ENDDO

    !! 3.4 The first time step of the first day of the month  
    ! implies that the month is over
    IF ( day == 1 .AND. sec .LT. dtradia ) THEN
       EndOfMonth=.TRUE.
    ELSE
       EndOfMonth=.FALSE.
    ENDIF

  
    !! 4. Calculate variables for dt_slow (i.e. "daily")

    ! Note: If dt_days /= 1, then variables 'xx_daily' (eg. half-daily or bi-daily) are by definition
    ! not expressed on a daily basis. This is not a problem but could be
    ! confusing

    !! 4.1. Calculate water stress accounting for hydraulic architecture
    !  If hydraulic architecture is used, humrel_daily (water stress) for use in stomate is 
    !  not determined from humrel as calculated in hydrol.f90, but it is calculated as the 
    !  ratio between a proxy for stressed and unstressed ecosystem functioning. 
    !  Nightvalues are exclude. On the first day of the simulation (no biomass)
    !  the stressed and unstressed proxies are probably both equal to the initilazed value (=zero, 
    !  initialized in sechiba.f90), to avoid numerical issues in this case we set humrel_daily 
    !  to 1. If hydraulic architecture is not used humrel_daily is calculated from humrel as 
    !  determined in hydrol. 
    IF (control%ok_hydrol_arch) THEN
     
       ! Accumulate the half hourly values into a daily value. Acculate the
       ! stressed and unstressed proxy first and then take the average. We 
       ! first accumulate and then take the ratio because that way we better 
       ! account for the night time values. If we do it in the other order 
       ! we need to assign a value to the ratio during the night. Whether we
       ! take zero or one, this will bias our waterstress number because 
       ! the number of half hours during the night is different for the 
       ! different pixels. So although water stress could be higher in the 
       ! south than in the north. During the growing season, day are shorter
       ! in the south so if we set the ratio to 1 during the night, our daily 
       ! water stress in the south may be less than in the north because we
       ! more 1's in the daily time series.

       ! The order of the calculation may depend on the proxy used. For example, 
       ! Accumulate transpir_supply and transpir first and then calculate ratio
       ! By doing this water stress is buffered (it is assumed that if 
       ! ::transpir_supply is larger than ::transpir at one timestep it can buffer 
       ! potential water stress in the next timestep. In reality this is not the 
       ! case: transpir_supply is a potential value not a realized one.
       DO ipts = 1,npts

          ! Sinang is the sin of the angle from the horizon this is equal 
          ! to the cos of the zenith angle, which is what we need to 
          ! identify night values.
          IF (sinang(ipts) .LT. min_stomate) THEN
           
             ! Redundant because the value will never be used
             ! For all pixels
             stressed(ipts,:) = zero
             unstressed(ipts,:) = zero
             
             ! For PFT1 under LCC
             ! vir_stressed = zero
             ! vir_unstressed = zero
          
          ELSE

             ! No vegetation present so ecosystem functioning
             ! was not defined
             WHERE (veget_max(ipts,:) .LT. min_stomate)

                ! To avoid uninitialized values
                stressed(ipts,:) = zero
                unstressed(ipts,:) = zero
                
                ! Update the values to avoid uninitialized fields
                ! in ::stressed_daily and ::unstressed_daily
                stressed_daily(ipts,:) = stressed_daily(ipts,:) + &
                     stressed(ipts,:)
                unstressed_daily(ipts,:) = unstressed_daily(ipts,:) + &
                     unstressed(ipts,:)
                daylight(ipts,:) = daylight(ipts,:) + 1
                
                ! For PFT1 under LCC
                ! vir_stressed = zero
                ! vir_unstressed = zero
             
             ELSEWHERE

                ! The pixel and pft contain vegetation and their
                ! is daylight, store values to calculate the stress
                stressed_daily(ipts,:) = stressed_daily(ipts,:) + &
                     stressed(ipts,:)
                unstressed_daily(ipts,:) = unstressed_daily(ipts,:) + &
                     unstressed(ipts,:)
                daylight(ipts,:) = daylight(ipts,:) + 1

                ! For PFT1 under LCC
                ! vir_stressed = zero
                ! vir_unstressed = zero

             ENDWHERE

          ENDIF

       ENDDO
        
       !---TEMP---
       IF(ld_wstress .OR. ld_hydrolarch) THEN
          WRITE(numout,*) 'dt_slow, ', do_slow
          DO ipts=1,npts
             DO ivm=1,nvm
                IF(ipts == test_grid .AND. ivm == test_pft)THEN
                   WRITE(numout,*) 'stressed, ',stressed(test_grid,test_pft)
                   WRITE(numout,*) 'stressed_daily, ',stressed_daily(test_grid,test_pft)
                   WRITE(numout,*) 'unstressed, ',stressed(test_grid,test_pft)
                   WRITE(numout,*) 'unstressed_daily, ',stressed_daily(test_grid,test_pft)
                   WRITE(numout,*) 'daylight, ', daylight(test_grid,test_pft)
                ENDIF
             END DO
          ENDDO
       ENDIF
       !----------
       
       ! Calculate the mean waterstress value at the end of each day
       IF (do_slow) THEN

          ! Calculate the ratio
          WHERE (unstressed_daily(:,:) .LT. min_stomate &
               .OR. stressed_daily(:,:) .LT. min_stomate)
             
             ! No ecosystem function thus no data, we assume
             ! there is no water stress
             humrel_daily(:,:) = un

          ELSEWHERE

             ! Calculate water stress. If we first calculate
             ! the daily sum and then the ratio there is no need
             ! to divide by daylight. If we accumulate ratios
             ! we will have to divide by daylight. This is
             ! arbitrary. To do this properly we should calculate
             ! the turgor in the cells and calculate growth
             ! based on that see i.e. Fatichi et al 2013, New
             ! Phytologist. We use a simple numerical construct
             ! (the sqrt of the ratio) to overcome that complexity.
             humrel_daily(:,:) = stressed_daily(:,:) / &
                  unstressed_daily(:,:)

             !+++CHECK+++
             ! vir_humrel_daily will be needed when accounting for the 
             ! water columns following a land cover change. It is partly 
             ! calculated in sechiba hydrol_arch but it is not passed all
             ! the way to here. Check and fix. For the moment LCC does
             ! not account for the water columns so ::humrel_daily and
             ! ::vir_humrel_daily are equal to each other
             vir_humrel_daily(:,:) = humrel_daily(:,:)
             !+++++++++++
        
          ENDWHERE

          ! Set to zero to start accumulating for the next day
          stressed_daily(:,:) = zero
          unstressed_daily(:,:) = zero
          daylight(:,:) = zero

       ELSE
          
          ! Set to a large value so that it is easy to detect problems
          ! This value should never be used. It should only be passed
          ! to other routine at the end of the day when do_slow is false
          humrel_daily(:,:) = large_value
          vir_humrel_daily(:,:) = large_value

       ENDIF

       !---TEMP---
       IF(ld_wstress .OR. ld_hydrolarch) THEN
          IF (do_slow) THEN
             WRITE(numout,*) 'humrel_daily, ',humrel_daily(:,:)
          ENDIF
       ENDIF
       !----------

    ! No hydrological architecture
    ELSE

       CALL stomate_accu (npts, nvm, dt_slow, dtradia, &
            & do_slow, humrel, humrel_daily)
       ! The variable ::vir_humrel_arch was not defined for this case
       ! Assume no stress
!!$        CALL stomate_accu (npts, nvm, dt_slow, dtradia, &
!!$            & do_slow, vir_humrel_arch, vir_humrel_daily)
       vir_humrel_daily = un
      
    ENDIF
    
    !---TEMP---
    IF (ld_hydrolarch .OR. ld_wstress) THEN
!!$       WRITE(numout,*) 'CHECK: humrel_arch after accumulation',humrel_daily(:,:)
!!$       WRITE(numout,*) 'CHECK: vir_humrel_arch after accumulation',vir_humrel_daily(:,:)
    ENDIF
    !----------

    !! 4.2 Accumulate instantaneous variables (do_slow=.FALSE.) 
    !  Accumulate instantaneous variables (do_slow=.FALSE.) and eventually 
    !  calculate daily mean value (do_slow=.TRUE.)

    ! Technically, we need to pass arrays with two dimensions to
    ! stomate_accu.  Some of these arrays have a single dimension,
    ! though, so let's copy them to a two dimensional array.  
    temp_array(:,1)=litterhumdiag(:)
    temp_array_daily(:,1)=litterhum_daily(:)

    CALL stomate_accu (npts,    1, dt_slow, dtradia, &
         & do_slow, temp_array, temp_array_daily)

    litterhum_daily(:)=temp_array_daily(:,1)

    temp_array(:,1)=t2m(:)
    temp_array_daily(:,1)=t2m_daily(:)

    CALL stomate_accu (npts,    1, dt_slow, dtradia, &
         & do_slow, temp_array, temp_array_daily)
    
    t2m_daily(:)=temp_array_daily(:,1)

    temp_array(:,1)=temp_sol(:)
    temp_array_daily(:,1)=tsurf_daily(:)

    CALL stomate_accu (npts,    1, dt_slow, dtradia, &
         & do_slow, temp_array, temp_array_daily)
    
    tsurf_daily(:)=temp_array_daily(:,1)

    CALL stomate_accu (npts, nbdl, dt_slow, dtradia, &
         & do_slow, stempdiag, tsoil_daily)
    CALL stomate_accu (npts, nbdl, dt_slow, dtradia, &
         & do_slow, shumdiag, soilhum_daily)
    
    temp_array(:,1)=precip(:)
    temp_array_daily(:,1)=precip_daily(:)

    CALL stomate_accu (npts,    1, dt_slow, dtradia, &
         & do_slow,  temp_array, temp_array_daily)
    
    precip_daily(:)=temp_array_daily(:,1)

    CALL stomate_accu (npts, nvm, dt_slow, dtradia, &
         & do_slow, gpp_d, gpp_daily)

    !! 4.2 Daily minimum temperature
    t2m_min_daily(:) = MIN( t2m_min(:), t2m_min_daily(:) )

    !! 4.3 Calculate maintenance respiration
    ! 
    ! The trunk passes LAI out.  We do not, since LAI is
    ! not a prognostic variable.  They say they do it
    ! to overcome previous problems with natural and agricultural 
    ! vegetation types.
    CALL maint_respiration (npts, dtradia, t2m, tlong_ref, &
         stempdiag, gpp, gpp_week, lab_fac, &
         veget_cov_max, rprof, biomass, resp_maint_part_radia, &
         circ_class_n, circ_class_biomass)

    !---TEMP---
    CALL check_biomass_sync("stomate.f90: After maint_respiration", npts, biomass, &
         circ_class_biomass, circ_class_n, ind, &
         lsync, bm_sync)
 

    ! Aggregate maintenance respiration across the different plant parts 
    resp_maint_radia(:,:) = zero
    DO j=2,nvm
       DO k= 1, nparts
          resp_maint_radia(:,j) = resp_maint_radia(:,j) &
               & + resp_maint_part_radia(:,j,k)
       ENDDO
    ENDDO

    ! Maintenance respiration separated by plant parts
    resp_maint_part(:,:,:) = resp_maint_part(:,:,:) &
         & + resp_maint_part_radia(:,:,:)


    !!  4.4 Mass balance closure (dt_radia)
    pool_start(:,:,:) = zero
    DO iele = 1,nelements

       ! Biomass pool
       DO ipar = 1,nparts
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               ( bm_to_litter(:,:,ipar,iele) + &
               turnover_daily(:,:,ipar,iele) ) * &
               dtradia/one_day * veget_max(:,:)
       ENDDO

       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilitt = 1,nlitt
          DO ilevs = 1,nlevs
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  litter(:,ilitt,:,ilevs,iele) * veget_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon (gC m-2) *  (m2 m-2)
       DO icarb = 1,ncarb
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO

    ENDDO ! # nelements
 
    !! 4.5 Litter dynamics and litter heterothropic respiration

    ! Including: litter update, lignin content, PFT parts, litter decay,
    ! litter heterotrophic respiration, dead leaf soil cover.
    ! Note: there is no vertical discretisation in the soil for litter decay.
    turnover_littercalc(:,:,:,:) = turnover_daily(:,:,:,:) * dtradia/one_day
    bm_to_littercalc(:,:,:,:) = bm_to_litter(:,:,:,:) * dtradia/one_day

    IF (control%ok_functional_allocation) THEN

       ! Litter calculation accounting for wood decomposition, needs to be used
       ! when forestry is used. In earlier versions of the code veget_cov_max was
       ! passed. Veget_cov_max does not account for the fraction of none biological
       ! land use in the pixel. The adavantage of using both veget_cov_max and 
       ! veget_max in stomate_lpj and its subroutines is unclear. Hence, only
       ! veget_max was retained.
       CALL littercalc_prognostic (npts, dtradia/one_day, &
            turnover_littercalc, bm_to_littercalc, &
            veget_max, temp_sol, stempdiag, shumdiag, litterhumdiag, &
            litterpart, litter, dead_leaves, lignin_struc, &
            lignin_wood, &
            deadleaf_cover, resp_hetero_litter, &
            soilcarbon_input_inst, control_temp_inst, control_moist_inst, &
            matrixA, vectorB)

    ELSE

       ! Litter calculation without wood decomposition, this routine was
       ! included for the sole purpose to reproduce the TRUNK version.
       ! In earlier versions of the code veget_cov_max was passed.
       ! Veget_cov_max does not account for the fraction of none biological
       ! land use in the pixel. The adavantage of using both veget_cov_max 
       ! and veget_max in stomate_lpj and its subroutines is unclear. Hence, 
       ! only veget_max was retained.
       CALL littercalc_diagnostic (npts, dtradia/one_day, &
            turnover_littercalc, bm_to_littercalc, &
            veget_max, temp_sol, stempdiag, shumdiag, litterhumdiag, &
            litterpart, litter, dead_leaves, lignin_struc, &
            deadleaf_cover, resp_hetero_litter, &
            soilcarbon_input_inst, control_temp_inst, control_moist_inst, &
            matrixA, vectorB)

    ENDIF


    !---TEMP---
    CALL check_biomass_sync("stomate.f90: After littercalc", npts, biomass, &
         circ_class_biomass, circ_class_n, ind, &
         lsync, bm_sync)   
    
    ! Heterothropic litter respiration during time step ::dtradia @tex $(gC m^{-2})$ @endtex

    resp_hetero_litter(:,:) = resp_hetero_litter(:,:) * dtradia/one_day    

    !! 4.6 Soil carbon dynamics and soil heterotrophic respiration

    ! Note: there is no vertical discretisation in the soil for litter decay.
    CALL soilcarbon (npts, dtradia/one_day, clay, soilcarbon_input_inst, &
         control_temp_inst, control_moist_inst, carbon, resp_hetero_soil, &
         matrixA, veget_max)

    ! Heterothropic soil respiration during time step ::dtradia @tex $(gC m^{-2})$ @endtex 
    resp_hetero_soil(:,:) = resp_hetero_soil(:,:) * dtradia/one_day

    ! Total heterothrophic respiration during time step ::dtradia @tex $(gC m^{-2})$ @endtex
    resp_hetero_radia(:,:) = resp_hetero_litter(:,:) + resp_hetero_soil(:,:)
    resp_hetero_d(:,:) = resp_hetero_d(:,:) + resp_hetero_radia(:,:)

    !! 4.7 Accumulate instantaneous variables (do_slow=.FALSE.) 
    ! Accumulate instantaneous variables (do_slow=.FALSE.) and eventually 
    ! calculate daily mean value (do_slow=.TRUE.) 
    CALL stomate_accu (npts, nlevs, dt_slow, dtradia, &
         & do_slow, control_moist_inst, control_moist_daily)
    CALL stomate_accu (npts, nlevs, dt_slow, dtradia, &
         & do_slow, control_temp_inst, control_temp_daily)

    DO i = 1,ncarb
       CALL stomate_accu (npts, nvm, dt_slow, dtradia, &
            & do_slow, soilcarbon_input_inst(:,i,:), soilcarbon_input_daily(:,i,:))
    ENDDO


    !! 4.8 Mass balance closure (dt_radia)
    !! 5.9.1 Calculate components of the mass balance
    !  Initial values of biomass, litter, soil and harvest pool
    pool_end(:,:,:) = zero
    DO iele = 1,nelements

       ! Litter pool
       DO ilitt = 1,nlitt
          DO ilevs = 1,nlevs
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  litter(:,ilitt,:,ilevs,iele) * veget_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon
       DO icarb = 1,ncarb
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO

    ENDDO ! # nelements

    !! 5.9.2 Calculate mass balance (dt_radia)
    check_intern(:,:,iatm2land,icarbon) = zero
    check_intern(:,:,iland2atm,icarbon) = -un * (resp_hetero_litter(:,:) + &
         resp_hetero_soil(:,:)) * veget_max(:,:)
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern(:,:,:) = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO

    ! Write result of the closure check
    DO ipts = 1,npts
       DO ivm = 1,nvm
          IF (closure_intern(ipts,ivm,icarbon) .LT. min_stomate .AND. &
               closure_intern(ipts,ivm,icarbon) .GT. -min_stomate) THEN
             IF (ld_massbal) WRITE(numout,*) 'Mass balance closure: stomate.f90 (dt_radia)', ipts, ivm
          ELSE
             WRITE(numout,*) 'Error: mass balance is not closed in stomate.f90 (dt_radia)', ipts, ivm
!             WRITE(numout,*) '   Difference, ipts, ivm', ipts, ivm, closure_intern(ipts,ivm,icarbon)
          ENDIF
       ENDDO
    ENDDO

!! 5. Daily processes - performed at the end of the day
    IF (do_slow) THEN

       !! 5.1 Initialize check for mass balance closure  
       !  Initial values of biomass, litter, soil and harvest pool
       pool_start(:,:,:) = zero
       resp_maint_d(:,:) = zero
       resp_growth_d(:,:) = zero

       DO iele = 1,nelements

          ! Biomass pool (gC m-2) * (m2 m-2)
          ! turnover and bm_to_litter should have been set to zero when
          ! entering the daily loop or they have to be ignored in the pool_start
          ! calculation as done here
          DO ipar = 1,nparts
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  (biomass(:,:,ipar,iele) * veget_max(:,:)) 
          ENDDO

          ! Litter pool (gC m-2) *  (m2 m-2) 
          DO ilitt = 1,nlitt
             DO ilevs = 1,nlevs
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     litter(:,ilitt,:,ilevs,iele) * veget_max(:,:)
             ENDDO
          ENDDO

          ! Soil carbon (gC m-2) *  (m2 m-2)
          ! soilcarbon_input_daily should have been set to zero when
          ! entering the daily loop or they are ignored in the pool_start
          ! calculation  
          DO icarb = 1,ncarb
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  carbon(:,icarb,:) * veget_max(:,:)
          ENDDO

       ENDDO ! # nelements

       !  Surface area (m2) of the pixel
       pixel_area(:) = resolution(:,1) * resolution(:,2)

       ! The biomass harvest pool shouldn't be multiplied by veget_max
       ! its units are already in gC pixel-1. The total amount for the 
       ! pixel was stored in harvest_pool. Account for the harvest and
       ! wood product pools. There are no longer PFTs 
       pool_start(:,1,icarbon) = pool_start(:,1,icarbon) + &
            (SUM(SUM(harvest_pool(:,:,:,icarbon),3),2) + &
            SUM(prod_l(:,:),2) + SUM(prod_m(:,:),2) + &
            SUM(prod_s(:,:),2) )/pixel_area(:)     

       
       !! 5.2 Update lai
       ! Use lai from stomate
       ! ?? check if this is the only time control%ok_pheno is used??
       ! ?? Looks like it is the only time. But this variables probably is defined 
       ! in stomate_constants or something, in which case, it is difficult to track.
       IF (control%ok_pheno) THEN
          !! 5.1.1 Update LAI 
          ! Set lai of bare soil to zero
          lai(:,ibare_sechiba) = zero
          ! lai for all PFTs
          DO j = 2, nvm
             DO i = 1,npts

                IF(veget_max(i,j) == zero)THEN
                   ! this vegetation type is not present, so no reason to do the 
                   ! calculation
                   CYCLE
                ENDIF

                lai(i,j) = biomass_to_lai(biomass(i,j,ileaf,icarbon),j)
             ENDDO
          ENDDO
          frac_age(:,:,:) = leaf_frac(:,:,:)
       ELSE 
          ! 5.1.2 Use a prescribed lai
          ! WARNING: code in setlai is identical to the lines above
          ! Update subroutine if LAI has to be forced 
          CALL  setlai(npts,lai,biomass) 
          frac_age(:,:,:) = zero
       ENDIF

       !! 5.3 Calculate long-term "meteorological" and biological parameters

       ! mainly in support of calculating phenology. If ::EndOfYear=.TRUE.
       ! annual values are update (i.e. xx_lastyear).
       ! In earlier versions of the code veget_cov_max was passed.
       ! Veget_cov_max does not account for the fraction of none biological
       ! land use in the pixel. The adavantage of using both veget_cov_max 
       ! and veget_max in stomate_lpj and its subroutines is unclear. Hence, 
       ! only veget_max was retained.

       CALL season &
            &          (npts, dt_days, EndOfYear, &
            &           veget, veget_max, &
            &           humrel_daily, t2m_daily, tsoil_daily, soilhum_daily, lalo, &
            &           precip_daily, npp_daily, biomass, &
            &           turnover_daily, gpp_daily, when_growthinit, &
            &           maxhumrel_lastyear, maxhumrel_thisyear, &
            &           minhumrel_lastyear, minhumrel_thisyear, &
            &           maxgppweek_lastyear, maxgppweek_thisyear, &
            &           gdd0_lastyear, gdd0_thisyear, &
            &           precip_lastyear, precip_thisyear, &
            &           lm_lastyearmax, lm_thisyearmax, &
            &           maxfpc_lastyear, maxfpc_thisyear, fpc_max, &
            &           humrel_month, humrel_week, t2m_longterm, &
            &           tlong_ref, t2m_month, t2m_week, tsoil_month, soilhum_month, &
            &           npp_longterm, turnover_longterm, gpp_week, &
            &           gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
            &           time_lowgpp, time_hum_min, hum_min_dormance, gdd_init_date, &
            &           gdd_from_growthinit, herbivores, humrel_growingseason, rue_longterm, &
            &           vir_humrel_daily, vir_humrel_growingseason)

       !---TEMP---
       CALL check_biomass_sync("stomate.f90: After season", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync) 
 

       !! 5.4 Waterstress

       !  The waterstress factor varies between 0.1 and 1 and is calculated
       !  from ::humrel_growingseason. The latter is only used in the allometric 
       !  allocation and its time integral is determined by tau_sap for trees
       !  (see constantes_mtc.f90 for tau_sap and see pft_constantes.f90 for 
       !  the definition of tau_hum_growingseason). The time integral for 
       !  grasses and crops is a prescribed constant (see constantes.f90). By
       !  having ::wstress_fac working on the turnover, water stress is progated
       !  into LF.
       !  Because the calculated values for ::wstress_fac are too low for this purpose
       !  Sonke Zhaele multiply it by two in the N-branch. This approach maintains
       !  the physiological basis of KF while combining it with a simple 
       !  multiplicative factor for water stress. Clearly after multiplication with 
       !  2, wstress is closer to 1 and will thus result in a KF values closer to 
       !  the physiologically expected KF. 
       !  In this implementation we take the sqrt (this is done in stomate where 
       !  ::humrel_daily is calculated from ::stressed and ::unstressed. The
       !  transformation from the ratio between stressed an unstressed gpp into a 
       !  numerical value that is used in the allocation and turnover is arbitrairy.
       !  A more physiological approach accounting for turgor would be needed to de
       !  fundamentally better.
       !  Note that the current implementation allows for the plants to adapt to drought 
       !  by adjusting its allocation. This is a long-term effect and it is long-term
       !  because ::humrel_growingseason integrates over ::tau_sap. For the moment no
       !  short term effects to drought are implemented. Short-term effects should be 
       !  implmented on mortality (through loss of turgor, heat stress, carbon starvation).
       wstress_season(:,1) = zero
       wstress_month(:,1) = zero
       vir_wstress_fac(:,1) = zero

       DO ivm = 2,nvm

          ! Calculate waterstress
          ! Water stress used in stomate
          !+++CHECK+++
          wstress_season(:,ivm) = MAX(humrel_growingseason(:,ivm), min_water_stress)
          wstress_month(:,ivm) = MAX(humrel_month(:,ivm), min_water_stress)
          vir_wstress_fac(:,ivm) = MAX(vir_humrel_growingseason(:,ivm), min_water_stress)
          !+++++++++++
 
          !+++CHECK+++
          ! The reduction of the leaf longevity should probably depend on
          ! the leaf skin temperature that will become available through
          ! the multi-layer energy budget. For the moment we don't have
          ! water stress on the leaves. A long term adaption could be 
          ! through ::sla
          tau_eff_leaf(:,ivm) = tau_leaf(ivm) * un
          
          ! The reduction of the root longevity depends on the soil moisture
          ! stress which we believe is reasonably well captured by our 
          ! proxy for wstress. We need to produce more roots to take the water
          ! from those layers that have water.
          ! feedback to c-allocation has been switched off
           tau_eff_root(:,ivm) = tau_root(ivm) *  un
          
          ! The reduction of sapwood longevity should depend on the cavitation
          ! which is calculated in hydrol_architecture. Should be linked
          ! once the memory of cavitation is calculated
          tau_eff_sap(:,ivm) = tau_sap(ivm) *  un
          !+++++++++
          
       ENDDO

       ! Add to sitory file
       CALL histwrite_p (hist_id_stomate, 'WSTRESS_SEASON', itime, &
            wstress_season(:,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'WSTRESS_MONTH', itime, &
            wstress_month(:,:), npts*nvm, horipft_index)
      

       !! 5.5 Use all processes included in stomate

       IF (control%ok_stomate) THEN

          !! 5.5.1  Activate stomate processes 
          ! Activate stomate processes (the complete list of processes depends 
          ! on whether the DGVM is used or not). Processes include: climate constraints 
          ! for PFTs, PFT dynamics, Phenology, Allocation, NPP (based on GPP and
          ! authothropic respiration), fire, mortality, vmax, assimilation temperatures,
          ! all turnover processes, light competition, sapling establishment, lai and 
          ! land cover change.
          ! In earlier versions of the code veget_cov_max was passed.
          ! Veget_cov_max does not account for the fraction of none biological
          ! land use in the pixel. The adavantage of using both veget_cov_max 
          ! and veget_max in stomate_lpj and its subroutines is unclear. Hence, 
          ! only veget_max was retained. Also, ::veget_cov_max_new has now been 
          ! replaced by ::veget_max_new.
          
          !---TEMP---
          ! Useful for debugging when running over a region and a single pixel is 
          ! causing problems. this code helps to find the coordinates of that pixel
!!$          DO ipts=1,npts
!!$             WRITE(numout,'(A,I3,10F20.10)') 'lalo stomate:',ipts,lalo(ipts,1:2)
!!$          ENDDO
          !----------

          CALL StomateLpj (npts, dt_days, EndOfYear, EndOfMonth, neighbours, &
               resolution, clay, herbivores, tsurf_daily, tsoil_daily, &
               t2m_daily, t2m_min_daily, litterhum_daily, soilhum_daily, maxhumrel_lastyear, &
               minhumrel_lastyear, gdd0_lastyear, precip_lastyear, humrel_month, humrel_week, &
               humrel_growingseason, tlong_ref, t2m_longterm, t2m_month, t2m_week, tsoil_month, &
               soilhum_month, gdd_m5_dormance,  gdd_from_growthinit, gdd_midwinter, ncd_dormance, &
               ngd_minus5, turnover_longterm, gpp_daily, gpp_week, time_lowgpp, &
               time_hum_min, maxfpc_lastyear, resp_maint_part, PFTpresent, age, &
               fireindex, firelitter, leaf_age, leaf_frac, biomass, &
               ind, adapted, regenerate, senescence, senescence_days, &
               when_growthinit, litterpart, litter, dead_leaves, carbon, &
               black_carbon, lignin_struc, lignin_wood, veget_max, veget, npp_longterm, &
               lm_lastyearmax, veget_lastlight, everywhere, need_adjacent, RIP_time, &
               rprof, npp_daily, turnover_daily, turnover_time, control_moist_inst, &
               control_temp_inst, soilcarbon_input_inst, co2_to_bm_dgvm, co2_fire, resp_hetero_d, &
               resp_maint_d, resp_growth_d, deadleaf_cover, vcmax, t_photo_min, &
               t_photo_opt, t_photo_max, t_photo_stress, bm_to_litter, prod_s, &
               prod_m, prod_l, flux_s, flux_m, flux_l, &
               veget_max_new, cflux_prod_s, cflux_prod_m, cflux_prod_l, carb_mass_total, &
               fpc_max, age_stand, rotation_n, last_cut, forest_managed, &
               forest_managed_lastyear, matrixA, rue_longterm, lab_fac, circ_class_n, &
               circ_class_biomass, KF, circ_class_dist, harvest_pool_bound, harvest_pool, &
               harvest_type, harvest_cut, harvest_area, mai, pai, &
               previous_wood_volume, mai_count, bm_sync, coppice_dens, lai_per_level, &
               laieff_fit, orphan_flux, vir_humrel_growingseason, litter_demand, wstress_season, &
               wstress_month, vir_wstress_fac, tau_eff_leaf,  tau_eff_sap, &
               tau_eff_root, MatrixV, VectorB, VectorU, &
               c_buried, st_dist, k_latosa_adapt)

          !---TEMP---
          CALL check_biomass_sync("stomate.f90: After StomatLPJ", npts, biomass, &
               circ_class_biomass, circ_class_n, ind, &
               lsync, bm_sync)

          !! 5.5.2 Long term adaptation of allocation to water stress
          !  ::wstress season is calculated as the seasonal mean of the
          !  ratio between the stressed and unstressed GPP. If the plant
          !  experiences a short spell of drought, leaves will be killed
          !  (see stomate_turnover). However, when the drought stress is 
          !  maintained during the season, it is assumed that the plant 
          !  will economise its canopy and therefore adjust its allocation
          ! factors to grow less leaves. To avoid that the canopy can only
          ! shrink the plants will try to grow more leaves when they do
          ! not experience any water stress. Extending the LAI, however,
          ! increases the chances that the plant will experience drough 
          ! stress in the future. These feedbacks should stabilize the LAI.
          wstress_adapt(:,:) = zero
          WHERE (wstress_season(:,:) .GT. 0.99)

             ! Increase the leaf allocation by 10% over the whole
             ! year. If there was no water stress during the whole 
             ! year, the following year more C will be allocated
             ! to the leaves.
             wstress_adapt(:,:) = 1.05

          ELSEWHERE

             wstress_adapt(:,:) = wstress_season(:,:)
           
          ENDWHERE
          
          !+++CHECK+++
          ! Don't use this adaptation
!!$          k_latosa_adapt(:,:) = k_latosa_adapt(:,:) * &
!!$               (wstress_adapt(:,:)**(un/365))
          !+++++++++++
        
          DO ivm = 2,nvm

             WHERE ( k_latosa_adapt(:,ivm) .GE. k_latosa_max(ivm) )
                
                k_latosa_adapt(:,ivm) = k_latosa_max(ivm)

             ENDWHERE

          ENDDO

          CALL histwrite_p (hist_id_stomate, 'K_LATOSA_ADAPT', itime, &
               k_latosa_adapt(:,:), npts*nvm, horipft_index)
          
                
          !+++CHECK+++
          ! check units and variable names for new lcchange and product use
          !! 5.3.2 Calculate the total CO2 flux from land use change
!!$          fco2_lu(:) = cflux_prod_s(:) + cflux_prod_m(:) + &
!!$               cflux_prod_l(:) + SUM(harvest_pool(:,:,1,ifm_crop,icut_crop),2)
          !+++++++++++

          
          !! 5.5 Photosynthesis parameters
          !+++OLD+++
!!$          assim_param(:,:,ivcmax) = zero
!!$          assim_param(:,:,ivjmax) = zero
!!$          assim_param(:,:,itmin) = zero
!!$          assim_param(:,:,itopt) = zero
!!$          assim_param(:,:,itmax) = zero
!!$          DO j = 2,nvm
!!$             assim_param(:,j,ivcmax) = vcmax(:,j)
!!$          ENDDO
!!$          DO j = 2, nvm
!!$             assim_param(:,j,ivjmax) = vjmax(:,j)
!!$          ENDDO
!!$          DO j = 2, nvm
!!$             assim_param(:,j,itmin) = t_photo_min(:,j)
!!$          ENDDO
!!$          DO j = 2, nvm
!!$             assim_param(:,j,itopt) = t_photo_opt(:,j)
!!$          ENDDO
!!$          DO j = 2, nvm
!!$             assim_param(:,j,itmax) = t_photo_max(:,j)
!!$          ENDDO
          !+++++++++

          assim_param(:,:,ivcmax) = zero

          DO j = 2,nvm
             assim_param(:,j,ivcmax) = vcmax(:,j)
          ENDDO


          !! 5.6 Update forcing variables for soil carbon
          IF (ldcarbon_write  .AND. TRIM(Cforcing_name) /= 'NONE') THEN
             npp_tot(:) = 0
             DO j=2,nvm
                npp_tot(:) = npp_tot(:) + npp_daily(:,j)
             ENDDO
             ! ::nbyear Number of years saved for carbon spinup
             sf_time = MODULO(REAL(date,r_std)-1,one_year*REAL(nbyear,r_std))
             iatt=FLOOR(sf_time/dt_forcesoil) + 1
             IF (iatt == 0) iatt = iatt_old + 1
             IF ((iatt<iatt_old) .and. (.not. cumul_Cforcing)) THEN
                nforce(:)=0
                soilcarbon_input(:,:,:,:) = zero
                control_moist(:,:,:) = zero
                control_temp(:,:,:) = zero
                npp_equil(:,:) = zero
             ENDIF
             iatt_old = iatt
             ! Update forcing
             nforce(iatt) = nforce(iatt)+1
             soilcarbon_input(:,:,:,iatt) = soilcarbon_input(:,:,:,iatt) + soilcarbon_input_daily(:,:,:)
             control_moist(:,:,iatt) = control_moist(:,:,iatt) + control_moist_daily(:,:)
             control_temp(:,:,iatt) = control_temp(:,:,iatt) + control_temp_daily(:,:)
             npp_equil(:,iatt) = npp_equil(:,iatt) + npp_tot(:)
          ENDIF


          !! 5.8 Check mass balance closure

          !! 5.8.1 Calculate components of the mass balance
          !  Initial values of biomass, litter, soil and harvest pool
          pool_end(:,:,:) = zero
          DO iele = 1,nelements

             ! Biomass pool
             DO ipar = 1,nparts
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     ( (biomass(:,:,ipar,iele) + &
                     bm_to_litter(:,:,ipar,iele) + &
                     turnover_daily(:,:,ipar,iele)) * veget_max(:,:)) 
             ENDDO

             ! Litter pool
             DO ilitt = 1,nlitt
                DO ilevs = 1,nlevs
                   pool_end(:,:,iele) = pool_end(:,:,iele) + &
                        litter(:,ilitt,:,ilevs,iele) * veget_max(:,:)
                ENDDO
             ENDDO

             ! Soil carbon
             DO icarb = 1,ncarb
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     carbon(:,icarb,:)  * veget_max(:,:)
             ENDDO

          ENDDO ! # nelements

          pool_end(:,1,icarbon) = pool_end(:,1,icarbon) + &
            (SUM(SUM(harvest_pool(:,:,:,icarbon),3),2) + &
            SUM(prod_l(:,:),2) + SUM(prod_m(:,:),2) + &
            SUM(prod_s(:,:),2) )/pixel_area(:)

          !! 5.8.2 Calculate mass balance
          !  iatm2land and iland2atm have already been calculated after growth in §4.2         
          IF (control%ok_land_cover_change .AND. EndOfYear) THEN
             check_intern(:,:,iatm2land,icarbon) = &
                  orphan_flux(:,:,igpp,icarbon)  * orphan_flux(:,:,ivegold,icarbon) + &
                  orphan_flux(:,:,ico2bm,icarbon) * orphan_flux(:,:,ivegnew,icarbon)
          ELSE
             check_intern(:,:,iatm2land,icarbon) = (gpp_daily(:,:) + co2_to_bm_dgvm(:,:)) * &
                  veget_max(:,:)
          ENDIF

          check_intern(:,:,iland2atm,icarbon) = -un * (orphan_flux(:,:,irmain,icarbon) + &
            orphan_flux(:,:,irgrow,icarbon)) * orphan_flux(:,:,ivegold,icarbon) * dt_days
          check_intern(:,1,iland2atm,icarbon) = -un * ( SUM(flux_s(:,:),2) + SUM(flux_m(:,:),2) + & 
               SUM(flux_l(:,:),2) ) / pixel_area(:)
          check_intern(:,:,ilat2out,icarbon) = zero
          check_intern(:,:,ilat2in,icarbon) = -un * zero
          check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
          closure_intern(:,:,:) = zero
          DO imbc = 1,nmbcomp
             closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
          ENDDO

          ! Write result of the closure check
          !  Sum over ivm because of LCC and redistribution    
          DO ipts = 1,npts
             IF (SUM(closure_intern(ipts,:,icarbon)) .LT. min_stomate .AND. &
                  SUM(closure_intern(ipts,:,icarbon)) .GT. -min_stomate) THEN
                IF (ld_massbal) WRITE(numout,*) 'Mass balance closure: stomate.f90 (dt_slow)', ipts
             ELSE
                WRITE(numout,*) 'Error: mass balance is not closed in stomate.f90 (dt_slow)'
!                WRITE(numout,*) '   Difference, ipts, ', ipts, SUM(closure_intern(ipts,:,icarbon)) 
             ENDIF
          ENDDO

       ENDIF ! control%ok_stomate


      !! 5.9 Write forcing file if ::ldforcing_write=.TRUE.
      ! Note: if STOMATE is run in coupled mode the forcing file is written
      ! If run in stand-alone mode, the forcing file is read!
      IF ( ldforcing_write .AND. TRIM(forcing_name) /= 'NONE' ) THEN

         !! 5.9.1 Convert GPP to sechiba time steps
         ! GPP is multiplied by coverage to obtain forcing @tex $(gC m^{-2} dt_slow^{-1})$\f \end@tex $(m^2 m^{-2})$ @endtexonly
         ! @tex$ m^{-2}$ @endtex remains in the units because ::veget_cov_max is a fraction, not a 
         ! surface area. In sechiba values are ponderated by surface and frac_no_bio. 
         ! At the beginning of stomate, the units are converted. 
         ! When we use forcesoil we call sechiba_main and so we need the have the same units as in sechiba.
         gpp_daily_x(:,:) = zero
         DO j = 2, nvm             
            gpp_daily_x(:,j) = gpp_daily_x(:,j) + &
                 & gpp_daily(:,j) * dt_slow / one_day * veget_cov_max(:,j)
         ENDDO

         ! Bare soil moisture availability has not been treated
         ! in STOMATE, update it here
         humrel_daily(:,ibare_sechiba) = humrel(:,ibare_sechiba)   

         ! Update index to store the next forcing step in memory
         iisf = iisf+1

         ! How many times have we treated this forcing state
         xn = REAL(nf_cumul(isf(iisf)),r_std)

         !! 5.9.2 Cumulate forcing variables
         ! Cumulate forcing variables (calculate average)
         ! Note: precipitation is multiplied by dt_slow/one_day to be consistent with 
         ! the units in sechiba
         IF (cumul_forcing) THEN
            clay_fm(:,iisf) = (xn*clay_fm(:,iisf)+clay(:))/(xn+1.)
            humrel_daily_fm(:,:,iisf) = &
                 & (xn*humrel_daily_fm(:,:,iisf) + humrel_daily(:,:))/(xn+1.)
            litterhum_daily_fm(:,iisf) = &
                 & (xn*litterhum_daily_fm(:,iisf)+litterhum_daily(:))/(xn+1.)
            t2m_daily_fm(:,iisf) = &
                 & (xn*t2m_daily_fm(:,iisf)+t2m_daily(:))/(xn+1.)
            t2m_min_daily_fm(:,iisf) = &
                 & (xn*t2m_min_daily_fm(:,iisf)+t2m_min_daily(:))/(xn+1.)
            tsurf_daily_fm(:,iisf) = &
                 & (xn*tsurf_daily_fm(:,iisf)+tsurf_daily(:))/(xn+1.)
            tsoil_daily_fm(:,:,iisf) = &
                 & (xn*tsoil_daily_fm(:,:,iisf)+tsoil_daily(:,:))/(xn+1.)
            soilhum_daily_fm(:,:,iisf) = &
                 & (xn*soilhum_daily_fm(:,:,iisf)+soilhum_daily(:,:))/(xn+1.)
            precip_fm(:,iisf) = &
                 & (xn*precip_fm(:,iisf)+precip_daily(:)*dt_slow/one_day)/(xn+1.)
            gpp_daily_fm(:,:,iisf) = &
                 & (xn*gpp_daily_fm(:,:,iisf) + gpp_daily_x(:,:))/(xn+1.)
            veget_fm(:,:,iisf) = &
                 & (xn*veget_fm(:,:,iisf) + veget(:,:) )/(xn+1.)
            veget_max_fm(:,:,iisf) = &
                 & (xn*veget_max_fm(:,:,iisf) + veget_max(:,:) )/(xn+1.)
            lai_fm(:,:,iisf) = &
                 & (xn*lai_fm(:,:,iisf) + lai(:,:) )/(xn+1.)
         ELSE
            ! Here we just calculate the values
            clay_fm(:,iisf) = clay(:)
            humrel_daily_fm(:,:,iisf) = humrel_daily(:,:)
            litterhum_daily_fm(:,iisf) = litterhum_daily(:)
            t2m_daily_fm(:,iisf) = t2m_daily(:)
            t2m_min_daily_fm(:,iisf) =t2m_min_daily(:)
            tsurf_daily_fm(:,iisf) = tsurf_daily(:)
            tsoil_daily_fm(:,:,iisf) =tsoil_daily(:,:)
            soilhum_daily_fm(:,:,iisf) =soilhum_daily(:,:)
            precip_fm(:,iisf) = precip_daily(:)
            gpp_daily_fm(:,:,iisf) =gpp_daily_x(:,:)
            veget_fm(:,:,iisf) = veget(:,:)
            veget_max_fm(:,:,iisf) =veget_max(:,:)
            lai_fm(:,:,iisf) =lai(:,:)
         ENDIF
         nf_cumul(isf(iisf)) = nf_cumul(isf(iisf))+1

         ! 5.9.3 Do we have to write the forcing states?
         IF (iisf == nsfm) THEN

            !! 5.9.3.1 Write these forcing states
            CALL forcing_write(forcing_id,1,nsfm)

            ! determine which forcing states must be read
            isf(1) = isf(nsfm)+1
            IF ( isf(1) > nsft ) isf(1) = 1
            DO iisf = 2, nsfm
               isf(iisf) = isf(iisf-1)+1
               IF (isf(iisf) > nsft)  isf(iisf) = 1
            ENDDO

            ! Read forcing variables - for debug use only
            ! CALL forcing_read(forcing_id,nsfm)
            iisf = 0

         ENDIF

      ENDIF


!!$      !+++CHECK+++
!!$      !! 5.10 Compute daily CO2 flux (AR5 output - not essential)
!!$      !  CO2 flux in @tex $gC m^{-2} s^{-1}$ @endtex (positive towards the atmosphere) 
!!$      !  is sum of: (1) heterotrophic respiration from ground + (2) maintenance 
!!$      !  respiration from the plants + (3) growth respiration from the plants + 
!!$      !  (4) co2 emissions from fire - (5) co2 taken up in the DGVM to establish 
!!$      !  saplings - (6) co2 taken up by photosyntyhesis. CHECK: In case of a LCC, 
!!$      !  these calculations needs to make use of the orphan fluxes
!!$      co2_flux_daily(:,:) = -un * (check_intern(:,:,iatm2land,icarbon) - &
!!$           check_intern(:,:,iland2atm,icarbon))
!!$
!!$      resp_maint_d(:,:) + resp_growth_d(:,:) + resp_hetero_d(:,:) + &
!!$           & co2_fire(:,:) - co2_to_bm_dgvm(:,:) - gpp_daily(:,:)
!!$
!!$      CALL xios_orchidee_send_field("nep",SUM(co2_flux_daily*veget_cov_max,dim=2)/1e3/one_day*contfrac)
!!$
!!$      IF ( hist_id_stom_IPCC > 0 ) THEN
!!$         vartmp(:) = SUM(co2_flux_daily*veget_cov_max,dim=2)/1e3/one_day*contfrac
!!$         CALL histwrite_p (hist_id_stom_IPCC, "nep", itime, &
!!$              vartmp, npts, hori_index)
!!$      ENDIF
!!$
!!$      ! CHECK: Check units of harvest pool and add a variable to account for the
!!$      ! daily harvest flux. What is this used for? NOTE: the structure of
!!$      ! harvest_pool has changed. CHECK: If net_co2_flux_monthly contains soil 
!!$      ! respirations which seems likely Then the loop should go over 
!!$      ! j=1,nvm. Because bare soil can respire after a land cover change!
!!$      
!!$      ! See 5.10 for details on NEP + fire. At the monthly time step also 
!!$      ! harvest and land use change are calculated
!!$      co2_flux_monthly(:,:) = co2_flux_monthly(:,:) + co2_flux_daily(:,:) 
!!$      harvest_above_monthly(:) = harvest_above_monthly(:) + &
!!$           + SUM(harvest_pool(:,:,1,ifm_crop,icut_crop),2)
!!$      cflux_prod_monthly(:) = cflux_prod_monthly(:) + cflux_prod_s(:) + & 
!!$           & cflux_prod_m(:) + cflux_prod_l(:)
!!$      
!!$      
!!$      !! 5.11 Compute monthly CO2 fluxes 
!!$      
!!$      IF ( EndOfMonth ) THEN
!!$         
!!$         IF ( control%ok_stomate ) THEN
!!$            
!!$            
!!$            !! 5.12.1 Write history file for monthly fluxes
!!$            
!!$            CALL histwrite_p (hist_id_stomate, 'CO2FLUX', itime, &
!!$                 co2_flux_monthly, npts*nvm, horipft_index)
!!$            
!!$         ENDIF
!!$         
!!$         ! CHECK: loop should go over j=1,nvm. Because bare soil can respire after
!!$         ! a land cover change! I (=VB) translated the French, but the whole stuff 
!!$         ! does not make sense to me. "If one deletes the montly cumulation, one 
!!$         ! should not forget this change in resolution(:,1)*resolution(:,2)*contfrac(:)"
!!$         ! Original: "Si on supprimer le cumul par mois, il ne faut pas oublier 
!!$         ! cette modif resolution(:,1)*resolution(:,2)*contfrac(:)
!!$         DO j=2, nvm
!!$            co2_flux_monthly(:,j) = co2_flux_monthly(:,j)* &
!!$                 resolution(:,1)*resolution(:,2)*contfrac(:)
!!$         ENDDO
!!$         
!!$         ! Should be supressed, this is post-processing
!!$         ! ?? How does it differ from co2_flux_monthly??
!!$         net_co2_flux_monthly = zero
!!$         DO ji=1,npts
!!$            DO j=2,nvm
!!$                net_co2_flux_monthly = net_co2_flux_monthly + &
!!$                     &  co2_flux_monthly(ji,j)*veget_cov_max(ji,j)
!!$             ENDDO
!!$          ENDDO
!!$          
!!$          !! 5.12.2 Cumulative fluxes of land use cover change, harvest and net biosphere production
!!$          ! Parallel processing, gather the information from different processors. first argument is the lo
!!$          ! local variable, the second argument is the global variable. bcast send it to all processors.
!!$          net_cflux_prod_monthly_sum = &
!!$               &  SUM(cflux_prod_monthly(:)*resolution(:,1)*resolution(:,2)*contfrac(:))*1e-15
!!$          CALL reduce_sum(net_cflux_prod_monthly_sum,net_cflux_prod_monthly_tot)
!!$          CALL bcast(net_cflux_prod_monthly_tot)
!!$          net_harvest_above_monthly_sum = &
!!$               &   SUM(harvest_above_monthly(:)*resolution(:,1)*resolution(:,2)*contfrac(:))*1e-15
!!$          CALL reduce_sum(net_harvest_above_monthly_sum,net_harvest_above_monthly_tot)
!!$          CALL bcast(net_harvest_above_monthly_tot)
!!$          net_co2_flux_monthly = net_co2_flux_monthly*1e-15
!!$          CALL reduce_sum(net_co2_flux_monthly,net_co2_flux_monthly_sum)
!!$          CALL bcast(net_co2_flux_monthly_sum)
!!$          net_biosp_prod_monthly_tot =  &
!!$               & ( net_co2_flux_monthly_sum + net_cflux_prod_monthly_tot + &
!!$               & net_harvest_above_monthly_tot )
!!$          
!!$          WRITE(numout,9010) 'GLOBAL net_cflux_prod_monthly    (Peta gC/month)  = ',net_cflux_prod_monthly_tot
!!$          WRITE(numout,9010) 'GLOBAL net_harvest_above_monthly (Peta gC/month)  = ',net_harvest_above_monthly_tot
!!$          WRITE(numout,9010) 'GLOBAL net_co2_flux_monthly      (Peta gC/month)  = ',net_co2_flux_monthly_sum
!!$          WRITE(numout,9010) 'GLOBAL net_biosp_prod_monthly    (Peta gC/month)  = ',net_biosp_prod_monthly_tot
!!$          
!!$9010      FORMAT(A52,F17.14)
!!$          
!!$          
!!$          ! Reset Monthly values
!!$          co2_flux_monthly(:,:) = zero
!!$          harvest_above_monthly(:) = zero
!!$          cflux_prod_monthly(:)    = zero
!!$
!!$       ENDIF ! Monthly processes - at the end of the month
!!$       !+++++++++++

      !! 5.12 Calculate the daily NBP
      IF (spinup_analytic) THEN

!!$         !+++CHECK+++
!!$         Use orphan fluxes. Harvest pool accumulates over a year and is only set to zero
!!$         at the end of the year. Seems that there is some double counting in this formulation
!!$         nbp_accu(:) = nbp_accu(:) + (-SUM(co2_flux_daily(:,:) * veget_max(:,:),dim=2) - &
!!$              (cflux_prod_s(:) + cflux_prod_m(:) + cflux_prod_l(:)) - &
!!$              SUM(harvest_pool(:,:,1,ifm_crop,icut_crop),2))/1e3
!!$         !+++++++++++

         !  Daily NBP is used in the spin-up. Rather than starting from all the initial
         !  components as done previously we prefer to make use of the aggregated components
         !  calculated to check the mass balance closure. NBP is then the change in all
         !  biomass pools.
         nbp_accu(:) = nbp_accu(:) + SUM((pool_end(:,:,icarbon) - pool_start(:,:,icarbon)),2)

      ENDIF
      
      !! 5.12 Reset daily variables
      humrel_daily(:,:) = zero
      vir_humrel_daily(:,:) = zero
      vir_transpir_supply_daily(:,:) = zero
      transpir_supply_daily(:,:) = zero
      transpir_daily(:,:) = zero
      litterhum_daily(:) = zero
      t2m_daily(:) = zero
      t2m_min_daily(:) = large_value
      tsurf_daily(:) = zero
      tsoil_daily(:,:) = zero
      soilhum_daily(:,:) = zero
      precip_daily(:) = zero
      gpp_daily(:,:) = zero
      resp_maint_part(:,:,:)=zero
      resp_hetero_d=zero
      IF (bavard >= 3) THEN
         WRITE(numout,*) 'stomate_main: daily processes done'
      ENDIF
      
   ENDIF  ! Daily processes - at the end of the day


   !! 6. Outputs from Stomate
   ! co2_flux receives a value from STOMATE only if STOMATE is activated.
   ! Otherwise, the calling hydrological module must do this itself.
   IF ( control%ok_stomate ) THEN
      
      !! 6.1 Respiration and fluxes
      !+++CHECK+++
      ! Not clear what this is about - should the orphan fluxes be used?
      ! This is passed to slow_proc.f90 and then to sechiba.f90
      resp_maint(:,:) = resp_maint_radia(:,:)*veget_cov_max(:,:)
      resp_maint(:,ibare_sechiba) = zero
      resp_growth(:,:)= resp_growth_d(:,:)*veget_cov_max(:,:)*dtradia/one_day
      resp_hetero(:,:) = resp_hetero_radia(:,:)*veget_cov_max(:,:)
      
      !! 6.2 Derived CO2 fluxes
      ! CO2 flux in gC m^{-2} s^{-1} (positive towards the atmosphere) is sum of:
      ! (1) heterotrophic respiration from ground + (2) maintenance respiration 
      ! from the plants + (3) growth respiration from the plants + (4) co2 
      ! emissions from fire - (5) co2 taken up in the DGVM to establish 
      ! saplings - (6) co2 taken up by photosyntyhesis
      co2_flux(:,:) = resp_hetero(:,:) + resp_maint(:,:) + resp_growth(:,:) &
           & + (co2_fire(:,:)-co2_to_bm_dgvm(:,:))*veget_cov_max(:,:)/one_day &
           & - gpp(:,:)
      
      temp_growth(:)=t2m_month(:)-tp_00
      !+++++++++++
      
   ENDIF


    !! 7. Analytical spinup

    IF (spinup_analytic) THEN

       !! 7.1. Update V and U at sechiba time step
       DO m = 2,nvm
          DO j = 1,npts 
             ! V <- A * V
             MatrixV(j,m,:,:) = MATMUL(matrixA(j,m,:,:),MatrixV(j,m,:,:))
             ! U <- A*U + B
             VectorU(j,m,:) = MATMUL(matrixA(j,m,:,:),VectorU(j,m,:)) + vectorB(j,m,:)
          ENDDO ! loop pixels
       ENDDO ! loop PFTS

       !! 7.2. What happened at the end of the year ?
       IF (EndOfYear) THEN
     
          ! 7.2.1 Increase the years counter every EndOfyear
          global_years = global_years + 1 

          ! 7.2.3 Is global_years is a multiple of the period time ?

          ! 3.2.1 When global_years is a multiple of the spinup_period, we calculate :
          !       1) the mean nbp flux over the period. This value is restarted
          !       2) we solve the matrix system by Gauss Jordan method
          !       3) We test if a point is at equilibrium : if yes, we mark the point (ok_equilibrium array)
          !       4) Then we reset the matrix 
          !       5) We erase the carbon_stock calculated by ORCHIDEE by the one found by the method
          IF( MOD(global_years, spinup_period) == 0 ) THEN
             WRITE(numout,*) 'Spinup analytic : Calculate if system is in equlibrium. global_years=',global_years
             ! The number total of days during the forcing period is given by :
             !    spinup_period*365 (we consider only the noleap calendar)
             nbp_flux(:) = nbp_accu(:) / ( spinup_period * 365.)
             ! Reset the values
             nbp_accu(:) = zero

             carbon_stock(:,ibare_sechiba,:) = zero
             ! Prepare the matrix for the resolution
             ! Add a temporary matrix W which contains I-MatrixV
             ! we should take the opposite of matrixV and add the identitiy : we solve (I-MatrixV)*C = VectorU
             MatrixW(:,:,:,:) = moins_un * MatrixV(:,:,:,:)
             DO jv = 1,nbpools
                MatrixW(:,:,jv,jv) =  MatrixW(:,:,jv,jv) + un
             ENDDO
             carbon_stock(:,:,:) = VectorU(:,:,:)

             !
             !  Solve the linear system
             !
             DO m = 2,nvm
                DO j = 1,npts
                   ! the solution will be stored in VectorU : so it should be restarted before
                   ! loop over npts and nvm, so we solved npts*(nvm-1) (9,9) linear systems
                   CALL gauss_jordan_method(nbpools,MatrixW(j,m,:,:),carbon_stock(j,m,:))
                ENDDO ! loop pixels
             ENDDO ! loop PFTS

             ! Reset temporary matrixW
             MatrixW(:,:,:,:) = zero 

             previous_stock(:,:,:) = current_stock(:,:,:)
             current_stock(:,:,:) = carbon_stock(:,:,:)  
             ! The relative error is calculated over the passive carbon pool (sum over the pfts) over the pixel.
             CALL error_L1_passive(npts,nvm, nbpools, current_stock, previous_stock, veget_max, &
                  &                eps_carbon, carbon_eq)   

             !! ok_equilibrium is saved,
             WHERE( carbon_eq(:) .AND. .NOT.(ok_equilibrium(:)) )
                ok_equilibrium(:) = .TRUE.  
             ENDWHERE

             ! Reset matrixV for the pixel to the identity matrix and vectorU to zero
             MatrixV(:,:,:,:) = zero
             VectorU(:,:,:) = zero
             DO jv = 1,nbpools
                MatrixV(:,:,jv,jv) = un
             END DO
             WRITE(numout,*) 'Reset for matrixV and VectorU done'    

             !! Write the values found in the standard outputs of ORCHIDEE
             litter(:,istructural,:,iabove,icarbon) = carbon_stock(:,:,istructural_above)
             litter(:,istructural,:,ibelow,icarbon) = carbon_stock(:,:,istructural_below)
             litter(:,imetabolic,:,iabove,icarbon) = carbon_stock(:,:,imetabolic_above)
             litter(:,imetabolic,:,ibelow,icarbon) = carbon_stock(:,:,imetabolic_below)
             litter(:,iwoody,:,iabove,icarbon) = carbon_stock(:,:,iwoody_above)
             litter(:,iwoody,:,ibelow,icarbon) = carbon_stock(:,:,iwoody_below)
             carbon(:,iactive,:) = carbon_stock(:,:,iactive_pool)
             carbon(:,islow,:) = carbon_stock(:,:,islow_pool)
             carbon(:,ipassive,:) = carbon_stock(:,:,ipassive_pool) 

             !! Add for parallelisation 
             IF (.NOT. ALLOCATED(ok_equilibrium_g)) THEN
                IF(is_root_prc) THEN
                   ALLOCATE(ok_equilibrium_g(nbp_glo),stat=ier)
                   IF (ier /= 0) THEN
                      WRITE(numout,*) ' error in memory allocation for ok_equilibrium_g.'
                      STOP 'stomate_main'
                   ENDIF
                ELSE
                   ALLOCATE(ok_equilibrium_g(0))
                END IF
             END IF ! IF (.NOT. ALLOCATED(ok_equilibrium_g))
             CALL gather(ok_equilibrium,ok_equilibrium_g)

             ! Final step, if all the points are at equilibrium, we can stop the job
             IF(is_root_prc) THEN         
                IF(ALL(ok_equilibrium_g)) THEN
                   WRITE(numout,*) 'Spinup analytic : Equilibrium for carbon pools is reached'
                ELSE
                   WRITE(numout,*) 'Spinup analytic : Equilibrium for carbon pools is not yet reached'
                END IF
             ENDIF
          ENDIF ! ( MOD(global_years,spinup_period) == 0)
       ENDIF ! (EndOfYear)

    ENDIF !(spinup_analytic)

    !! 8. Messages

    IF ( (bavard >= 2).AND.EndOfYear.AND.do_slow) THEN
       WRITE(numout,*) 'stomate: EndOfYear'
    ENDIF
    IF (bavard >= 4) WRITE(numout,*) 'Leaving stomate_main'
    IF (long_print) WRITE (numout,*) ' stomate_main done '

    CALL check_biomass_sync("stomate.f90: end of stomate_main", npts, biomass, &
         circ_class_biomass, circ_class_n, ind, &
         lsync, bm_sync)

  END SUBROUTINE stomate_main


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_init
!!
!>\BRIEF        The routine is called only at the first simulation. At that 
!! time settings and flags are read and checked for internal consistency and 
!! memory is allocated for the variables in stomate.
!!
!! DESCRIPTION  : The routine reads the 
!! following flags from the run definition file:
!! -bavard (level of online diagnostic in stomate)\n
!! -ipd (index of grid point for online diagnostics)\n
!! -ok_herbivores (flag to activate herbivores)\n
!! -treat_expansion (flag to activate PFT expansion across a pixel\n
!! -harvest_agri (flag to harvest aboveground biomass from agricultural PFTs)\n
!! \n
!! Check for inconsistent setting between the following flags:
!! -control%ok_stomate\n
!! -control%ok_dgvm\n
!! -control%ok_co2\n
!! -ldforcing_write\n
!! \n
!! Memory is allocated for all the variables of stomate and new indexing tables 
!! are build. New indexing tables are needed because a single pixel can conatin 
!! several PFTs. The new indexing tables have separate indices for the different 
!! PFTs. Similar index tables are build for land use cover change.\n
!! \n
!! Several global variables and land cover change variables are initialized to 
!! zero.\n
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): Strictly speaking the subroutine has no output 
!! variables. However, the routine allocates memory and builds new indexing 
!! variables for later use.\n 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE stomate_init &
       &  (kjpij, npts, index, ldforcing_write, lalo, &
       &   rest_id_stom, hist_id_stom, hist_id_stom_IPCC)

  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)                    :: kjpij             !! Total size of the un-compressed grid, including 
                                                                      !! oceans (unitless) 
    INTEGER(i_std),INTENT(in)                    :: npts              !! Domain size - number of terrestrial pixels 
                                                                      !! (unitless) 
    INTEGER(i_std),INTENT(in)                    :: rest_id_stom      !! STOMATE's _Restart_ file identifier
    INTEGER(i_std),INTENT(in)                    :: hist_id_stom      !! STOMATE's _history_ file identifier
    INTEGER(i_std),INTENT(in)                    :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file identifier 
    INTEGER(i_std),DIMENSION(npts),INTENT(in):: index                 !! Indices of the terrestrial pixels on the global 
                                                                      !! map 
    REAL(r_std),DIMENSION(npts,2),INTENT(in) :: lalo                  !! Geogr. coordinates (latitude,longitude) (degrees)
    LOGICAL,INTENT(in)                           :: ldforcing_write   !! Flag activates writing of _forcing_ file

   
    !! 0.2 Output variables


    !! 0.3 Modified variables


    !! 0.4 Local variables

    LOGICAL                                      :: l_error           !! Check errors in netcdf call
    INTEGER(i_std)                               :: ier               !! Check errors in netcdf call
    INTEGER(i_std)                               :: ji,j,ipd,idia     !! Indices 
!_ ================================================================================================================================
    
  !! 1. Online diagnostics

    !Config Key   = BAVARD
    !Config Desc  = level of online diagnostics in STOMATE (0-4)
    !Config If    = OK_STOMATE
    !Config Def   = 1
    !Config Help  = With this variable, you can determine
    !Config         how much online information STOMATE
    !Config         gives during the run. 0 means
    !Config         virtually no info.
    !Config Units = [-]
    !
    ! By default, ::bavard is set to 1 in stomate_data
    bavard = 1
    ! Get ::bavard from run definition file
    CALL getin_p('BAVARD', bavard)

    IF ( npts > 0 ) THEN
       !Config  Key  = STOMATE_DIAGPT
       !Config  Desc = Index of grid point for online diagnostics
       !Config If    = OK_STOMATE
       !Config  Def  = 1
       !Config  Help = This is the index of the grid point which
       !               will be used for online diagnostics.
       !Config Units = [-]
       ! By default ::ipd is set to 1
       ipd = 1
       ! Get ::ipd from run definition file
       CALL getin_p('STOMATE_DIAGPT',ipd)
       ipd = MIN( ipd, npts )
       WRITE(numout,*) 'Stomate: '
       WRITE(numout,*) '  Index of grid point for online diagnostics: ',ipd
       WRITE(numout,*) '  Lon, lat:',lalo(ipd,2),lalo(ipd,1)
       WRITE(numout,*) '  Index of this point on GCM grid: ',index(ipd)
       !
    ENDIF
    
  !! 2. Check consistency of flags

    IF ( ( .NOT. control%ok_stomate ) .AND. control%ok_dgvm ) THEN
       WRITE(numout,*) 'Cannot do dynamical vegetation without STOMATE.'
       WRITE(numout,*) 'Inconsistency between ::control%ok_stomate and ::control%ok_dgvm'
       WRITE(numout,*) 'Stop: fatal error'
       STOP
    ENDIF

    IF ((.NOT.control%ok_co2).AND.control%ok_stomate) THEN
       WRITE(numout,*) 'Cannot call STOMATE without GPP.'
       WRITE(numout,*) 'Inconsistency between ::control%ok_stomate and ::control%ok_co2'
       WRITE(numout,*) 'Stop: fatal error'
       STOP
    ENDIF

    IF ( ( .NOT. control%ok_co2 ) .AND. ldforcing_write ) THEN
       WRITE(numout,*)'Cannot write forcing file if photosynthesis is not activated'
       WRITE(numout,*)'Inconsistency between ::ldforcing_write and ::control%ok_co2'
       WRITE(numout,*) 'Stop: fatal error'
       STOP
    ENDIF
    
  !! 3. Communicate settings
    
    WRITE(numout,*) 'stomate first call - overview of the activated flags:'
    WRITE(numout,*) '  Photosynthesis: ', control%ok_co2
    WRITE(numout,*) '  USE STOMATE: ', control%ok_stomate
    WRITE(numout,*) '  USE DGVM: ', control%ok_dgvm
    
  !! 4. Allocate memory for STOMATE's variables

    l_error = .FALSE.

    ALLOCATE(veget_cov_max(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for veget_cov_max. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(veget_cov_max_new(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for veget_cov_max_new. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(ba(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for ba. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(adapted(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for adapted. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(regenerate(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for regenerate. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(humrel_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for humrel_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(vir_humrel_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for vir_humrel_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    vir_humrel_daily(:,:)=zero

     ALLOCATE(stressed_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for stressed_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    stressed_daily = zero

    ALLOCATE(unstressed_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for unstressed_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    unstressed_daily(:,:) = zero

    ALLOCATE(daylight(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for daylight. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    daylight(:,:) = zero

    ALLOCATE(transpir_supply_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for transpir_supply_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    transpir_supply_daily=zero

    ALLOCATE(vir_transpir_supply_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for vir_transpir_supply_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    vir_transpir_supply_daily=zero

    ALLOCATE(transpir_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for transpir_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    transpir_daily=zero

    ALLOCATE(litterhum_daily(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for litterhum_daily. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_daily(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_daily. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_min_daily(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_min_daily. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(tsurf_daily(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for tsurf_daily. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(tsoil_daily(npts,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for tsoil_daily. We stop. We need npts*nbdl words',npts,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(soilhum_daily(npts,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for soilhum_daily. We stop. We need npts*nbdl words',npts,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(precip_daily(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for precip_daily. We stop. We need npts words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gpp_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gpp_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(npp_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for npp_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(turnover_daily(npts,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_daily. We stop. We need npts*nvm*nparts*nelements words', &
       &   npts,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF
    turnover_daily(:,:,:,:) = zero

    ALLOCATE(turnover_littercalc(npts,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_littercalc. We stop. We need npts*nvm*nparts*nelements words', & 
        &  npts,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(humrel_month(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for humrel_month. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(humrel_week(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    ALLOCATE(humrel_growingseason(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    ALLOCATE(vir_humrel_growingseason(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    ALLOCATE(t2m_longterm(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_longterm. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(tlong_ref(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for tlong_ref. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_month(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_month. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_week(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_week. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(tsoil_month(npts,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for tsoil_month. We stop. We need npts*nbdl words',npts,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(soilhum_month(npts,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for soilhum_month. We stop. We need npts*nbdl words',npts,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(fireindex(npts,nvm),stat=ier) 
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for fireindex. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(firelitter(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for firelitter. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxhumrel_lastyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxhumrel_lastyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxhumrel_thisyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxhumrel_thisyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(minhumrel_lastyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for minhumrel_lastyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(minhumrel_thisyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for minhumrel_thisyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxgppweek_lastyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxgppweek_lastyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxgppweek_thisyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxgppweek_thisyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd0_lastyear(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd0_lastyear. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd0_thisyear(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd0_thisyear. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd_init_date(npts,4),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_init_date. We stop. We need npts*4 words',npts,4
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd_from_growthinit(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_from_growthinit. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    gdd_from_growthinit(:,:) = zero

    ALLOCATE(precip_lastyear(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for precip_lastyear. We stop. We need npts*nvm words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(precip_thisyear(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for precip_thisyear. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd_m5_dormance(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_m5_dormance. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    gdd_m5_dormance(:,:) = zero

    ALLOCATE(gdd_midwinter(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_midwinter. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    gdd_midwinter(:,:) = zero

    ALLOCATE(ncd_dormance(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for ncd_dormance. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    ncd_dormance(:,:) = zero

    ALLOCATE(ngd_minus5(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for ngd_minus5. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    ngd_minus5(:,:) = zero

    ALLOCATE(PFTpresent(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for PFTpresent. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(npp_longterm(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for npp_longterm. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lm_lastyearmax(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lm_lastyearmax. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lm_thisyearmax(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lm_thisyearmax. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxfpc_lastyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxfpc_lastyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxfpc_thisyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxfpc_thisyear. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(turnover_longterm(npts,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_longterm. We stop. We need npts*nvm*nparts*nelements words', & 
       &    npts,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gpp_week(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gpp_week. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(senescence(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for senescence. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    senescence(:,:) = .FALSE.

    ALLOCATE(when_growthinit(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for when_growthinit. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(age(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for age. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_hetero_d(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_hetero_d. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_hetero_radia(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_hetero_radia. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_maint_d(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_d. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_growth_d(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_growth_d. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(co2_fire(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for co2_fire. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(co2_to_bm_dgvm(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for co2_to_bm_dgvm. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    co2_to_bm_dgvm(:,:) = zero

    ALLOCATE(veget_lastlight(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for veget_lastlight. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(everywhere(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for everywhere. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(need_adjacent(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for need_adjacent. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(leaf_age(npts,nvm,nleafages),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for leaf_age. We stop. We need npts*nvm*nleafages words', & 
       &      npts,nvm,nleafages
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(leaf_frac(npts,nvm,nleafages),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for leaf_frac. We stop. We need npts*nvm*nleafages words', & 
       &      npts,nvm,nleafages
       STOP 'stomate_init'
    ENDIF

! moved to sechiba
!!$    ALLOCATE(circ_class_n(npts,nvm,ncirc),stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for circ_class_n. We stop. We need npts*nvm*ncirc words', & 
!!$       &      npts,nvm,ncirc
!!$       STOP 'stomate_init'
!!$    ENDIF
!!$    
!!$    ALLOCATE(circ_class_biomass(npts,nvm,ncirc,nparts,nelements),stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for circ_class_biomass.'
!!$       WRITE(numout,*) 'We stop. We need npts*nvm*nparts*ncirc*nelmements words', & 
!!$       &      npts,nvm,ncirc,nparts,nelements
!!$       STOP 'stomate_init'
!!$    ENDIF

!!$    ALLOCATE(biomass(npts,nvm,nparts,nelements),stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for biomass. We stop. We need npts*nvm*nparts*nelements words', &
!!$       &    npts,nvm,nparts,nelements
!!$       STOP 'stomate_init'
!!$    ENDIF
!!$    ALLOCATE(ind(npts,nvm),stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for ind. We stop. We need npts*nvm words',npts,nvm
!!$       STOP 'stomate_init'
!!$    ENDIF

    ALLOCATE(RIP_time(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for RIP_time. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    ALLOCATE(time_lowgpp(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for time_lowgpp. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(time_hum_min(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for time_hum_min. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(hum_min_dormance(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for hum_min_dormance. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(litterpart(npts,nvm,nlitt),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for litterpart. We stop. We need npts*nvm*nlitt words',  &
       &  npts,nvm,nlitt
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(litter(npts,nlitt,nvm,nlevs,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for litter. We stop. We need npts*nlitt*nvm*nlevs*nelements words', & 
       &    npts,nlitt,nvm,nlevs,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(dead_leaves(npts,nvm,nlitt),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for dead_leaves. We stop. We need npts*nvm*nlitt words', & 
       &   npts,nvm,nlitt
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(carbon(npts,ncarb,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for carbon. We stop. We need npts*ncarb*nvm words',npts,ncarb,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(black_carbon(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for black_carbon. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lignin_struc(npts,nvm,nlevs),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lignin_struc. We stop. We need npts*nvm*nlevs words',npts,nvm,nlevs
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lignin_wood(npts,nvm,nlevs),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lignin_wood. We stop. We need npts*nvm*nlevs words',npts,nvm,nlevs
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(turnover_time(npts,nvm,nparts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_time. We stop. We need npts*nvm words',npts,nvm,nparts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(co2_flux_daily(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for co2_flux_daily. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(co2_flux_monthly(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for co2_flux_monthly. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (cflux_prod_monthly(npts), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for cflux_prod_monthly. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF
 
    ALLOCATE (harvest_above_monthly(npts), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for harvest_above_monthly. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(bm_to_litter(npts,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for bm_to_litter. We stop. We need npts*nvm*nparts*nelements words', & 
       &    npts,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(bm_to_littercalc(npts,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for bm_to_littercalc. We stop. We need npts*nvm*nparts*nelements words', &
       &   npts,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(herbivores(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for herbivores. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(hori_index(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for hori_index. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(horipft_index(npts*nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horipft_index. We stop. We need npts*nvm words',npts*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(horican_index(npts*nlevels_tot),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horican_index. We stop. We need npts*nlevels_tot words',&
            npts*nlevels_tot
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_maint_part_radia(npts,nvm,nparts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_part_radia. We stop. We need npts*nvm*nparts words', &
       &  npts,nvm,nparts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_maint_radia(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_radia. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_maint_part(npts,nvm,nparts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_part. We stop. We need npts*nvm*nparts words', &
       &    npts,nvm,nparts
       STOP 'stomate_init'
    ENDIF
    resp_maint_part(:,:,:) = zero

    ALLOCATE (horip_s_index(npts*nshort), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_s_index. We stop. We need npts*10 words',npts,nshort
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (horip_m_index(npts*nmedium), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_m_index. We stop. We need npts*10 words',npts,nmedium
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (horip_l_index(npts*100), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_l_index. We stop. We need npts*100 words',npts,nlong
       STOP 'stomate_init'
    ENDIF
 
    ALLOCATE (horip_ss_index(npts*(nshort+1)), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_ss_index. We stop. We need npts*11 words',npts,nshort+1
       STOP 'stomate_init'
    ENDIF 

    ALLOCATE (horip_mm_index(npts*(nmedium+1)), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_mm_index. We stop. We need npts*11 words',npts,nmedium+1
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (horip_ll_index(npts*(nlong+1)), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_ll_index. We stop. We need npts*101 words',npts,nlong+1
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (prod_s(npts,0:nshort), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for prod_s. We stop. We need npts*(nshort+1) words',npts,nshort+1
       STOP 'stomate_init'
    ENDIF  

    ALLOCATE (prod_m(npts,0:nmedium), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for prod_m. We stop. We need npts*(nmedium+1) words',npts,nmedium+1
       STOP 'stomate_init'
    ENDIF 

    ALLOCATE (prod_l(npts,0:nlong), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for prod_l. We stop. We need npts*(nlong+1) words',npts,nlong+1
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (flux_s(npts,nshort), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_s. We stop. We need npts*nshort words',npts,nshort
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (flux_m(npts,nmedium), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_m. We stop. We need npts*nmedium words',npts,nmedium
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (flux_l(npts,nlong), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_l. We stop. We need npts*nlong words',npts,nlong
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (cflux_prod_s(npts), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for cflux_prod_s. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (cflux_prod_m(npts), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for cflux_prod_m. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (cflux_prod_l(npts), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for cflux_prod_l. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (carb_mass_total(npts), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for carb_mass_total. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (soilcarbon_input_daily(npts,ncarb,nvm), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for soilcarbon_input_daily. We stop. We need npts*ncarb*nvm words', & 
       &    npts,ncarb,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (control_temp_daily(npts,nlevs), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for control_temp_daily. We stop. We need npts*nlevs words',npts,nlevs
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (control_moist_daily(npts,nlevs), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for control_moist_daily. We stop. We need npts*nlevs words',npts,nlevs
       STOP 'stomate_init'
    ENDIF
    ALLOCATE (rue_longterm(npts,nvm), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for rue_longterm. We stop. We need npts*nlevs words',npts,nlevs
       STOP 'stomate_init'
    ENDIF
    rue_longterm(:,:) = un

    ALLOCATE (t_photo_stress(npts,nvm), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t_photo_stress. We stop. We need npts*nlevs words',npts,nlevs
       STOP 'stomate_init'
    ENDIF
    t_photo_stress(:,:)=zero ! Initializing here, but not 100% certain this is right

    ALLOCATE (lab_fac(npts,nvm), stat=ier)
    ! temp fix, initializing it here...the values get set in stomate_growth_fun_all,
    ! but they get used in stomate_resp beforehand, so maybe a code flow problem?
    lab_fac(:,:)=zero

    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lab_fac. We stop. We need npts*nlevs words',npts,nlevs
       STOP 'stomate_init'
    ENDIF
!!$    ALLOCATE (lai_target(npts,nvm), stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for lai_target. We stop. We need npts*nlevs words',npts,nlevs
!!$       STOP 'stomate_init'
!!$    ENDIF
    ALLOCATE (fpc_max(npts,nvm), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for fpc_max. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (senescence_days(npts,nvm), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for senescence_days. We stop. We need npts*nvm words',npts,nvm
       STOP 'stomate_init'
    ENDIF
    senescence_days(:,:) = zero

    ALLOCATE(ok_equilibrium(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for ok_equilibrium. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(carbon_eq(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for carbon_eq. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(nbp_accu(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_accu. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(nbp_flux(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_flux. We stop. We need npts words',npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(matrixA(npts,nvm,nbpools,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for matrixA. We stop. We need npts*nvm*nbpools*nbpools words',  & 
       &     npts, nvm, nbpools, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(vectorB(npts,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for vectorB. We stop. We need npts*nvm*nbpools words',  & 
       &     npts, nvm, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(VectorU(npts,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for VectorU. We stop. We need npts*nvm*nbpools words',  & 
       &     npts, nvm, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(MatrixV(npts,nvm,nbpools,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for MatrixV. We stop. We need npts*nvm*nbpools*nbpools words',  & 
       &     npts, nvm, nbpools, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(MatrixW(npts,nvm,nbpools,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for MatrixW. We stop. We need npts*nvm*nbpools*nbpools words',  & 
       &     npts, nvm, nbpools, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(previous_stock(npts,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for previous_stock. We stop. We need npts*nvm*nbpools words',  & 
       &     npts, nvm, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(current_stock(npts,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for current_stock. We stop. We need npts*nvm*nbpools words',  & 
       &     npts, nvm, nbpools
       STOP 'stomate_init'
    ENDIF
    
    ALLOCATE(forest_managed(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for forest_managed. We stop. We need npts*nvm words',  & 
       &     npts, nvm
       STOP 'stomate_init'
    ENDIF


    ALLOCATE(forest_managed_lastyear(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for forest_managed_lastyear. We stop. We need npts*nvm words',  & 
       &     npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(age_stand(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for age_stand. We stop. We need npts*nvm words',  & 
       &     npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(rotation_n(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for rotation_n. We stop. We need npts*nvm words',  & 
       &     npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(last_cut(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for last_cut. We stop. We need npts*nvm words',  & 
       &     npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(sigma(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for sigma. We stop. We need npts*nvm words',  & 
       &     npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(KF(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for KF. We stop. We need nvm words = ',npts*nvm
       STOP 'stomate_init'
    ENDIF
    KF(:,:) = zero ! Is there a better place in the code for this?

    ALLOCATE(k_latosa_adapt(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for k_latosa_adapt. We stop. We need nvm words = ',npts*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(harvest_pool(npts,nvm,ndia_harvest+1,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_pool. We stop. We need many words = ',&
            npts*nvm*(ndia_harvest+1)*nelements
       STOP 'stomate_init'
    ENDIF
    harvest_pool(:,:,:,:)=zero

    ALLOCATE(harvest_type(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_type. We stop. We need many words = ',&
            npts*nvm
       STOP 'stomate_init'
    ENDIF
    harvest_type(:,:)=zero

    ALLOCATE(harvest_cut(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_cut. We stop. We need many words = ',&
            npts*nvm
       STOP 'stomate_init'
    ENDIF
    harvest_cut(:,:)=zero

    ALLOCATE(harvest_area(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_area. We stop. We need many words = ',&
            npts*nvm
       STOP 'stomate_init'
    ENDIF
    harvest_area(:,:)=zero

    ALLOCATE(harvest_pool_bound(0:ndia_harvest+1),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_pool_bound. ' // &
            'We stop. We need ndia_harvest+2 words = ',&
            ndia_harvest+2
       STOP 'stomate_init'
    ENDIF

    ! Here we can initialize the values of this array, too.  They should never change
    ! over the course of the simulation.
    harvest_pool_bound(ndia_harvest+1) = val_exp
    DO idia = 0,ndia_harvest
       harvest_pool_bound(idia) = max_harvest_dia*REAL(idia,r_std)/REAL(ndia_harvest,r_std)
    ENDDO

    ALLOCATE(mai(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for mai. ' // &
            'We stop. We need npts*nvm words = ',&
            npts*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(pai(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for pai. ' // &
            'We stop. We need npts*nvm words = ',&
            npts*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(previous_wood_volume(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for previous_wood_volume. ' // &
            'We stop. We need npts*nvm words = ',&
            npts*nvm
       STOP 'stomate_init'
    ENDIF
    
    ALLOCATE(mai_count(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for mai_count. We stop. We need npts*nvm words',  & 
       &     npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(bm_sync(npts,nvm,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for bm_sync. We stop. We need npts*nvm*nelements words',  & 
       &     npts, nvm, nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(coppice_dens(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for coppice_dens. ' // &
            'We stop. We need npts*nvm words = ',&
            npts*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(litter_demand(npts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for litter_demand. ' // &
            'We stop. We need npts words = ',&
            npts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(wstress_season(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for wstress_season. ' // &
            'We stop. We need npts*nvm words = ',&
            npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(wstress_month(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for wstress_month. ' // &
            'We stop. We need npts*nvm words = ',&
            npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(vir_wstress_fac(npts,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for vir_wstress_fac. ' // &
            'We stop. We need npts*nvm words = ',&
            npts, nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(c_buried(npts,nnobio),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for c_buried. ' // &
            'We stop. We need npts*nnobio words = ',&
            npts, nnobio
       STOP 'stomate_init'
    ENDIF


    !! 5. File definitions
    
    ! Store history and restart files in common variables
    hist_id_stomate = hist_id_stom
    hist_id_stomate_IPCC = hist_id_stom_IPCC
    rest_id_stomate = rest_id_stom
    
    ! In STOMATE reduced grids are used containing only terrestrial pixels.
    ! Build a new indexing table for the vegetation fields separating 
    ! between the different PFTs. Note that ::index has dimension (npts) 
    ! wheras ::indexpft has dimension (npts*nvm). 

    hori_index(:) = index(:)

    DO j = 1, nvm
       DO ji = 1, npts
          horipft_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nlevels_tot
       DO ji = 1, npts
          horican_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    ! Similar index tables are build for the land cover change variables
    DO j = 1, nshort
       DO ji = 1, npts
          horip_s_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nmedium
       DO ji = 1, npts
          horip_m_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nlong
       DO ji = 1, npts
          horip_l_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nshort+1
       DO ji = 1, npts
          horip_ss_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nmedium+1
       DO ji = 1, npts
          horip_mm_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nlong+1
       DO ji = 1, npts
          horip_ll_index((j-1)*npts+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

  !! 6. Initialization of global and land cover change variables. 

    ! All variables are cumulative variables. bm_to_litter is not and is therefore
    ! excluded
    !   bm_to_litter(:,:,:) = zero
    turnover_daily(:,:,:,:) = zero
    resp_hetero_d(:,:) = zero
    co2_flux_daily(:,:) = zero
    co2_flux_monthly(:,:) = zero
    cflux_prod_monthly(:) = zero
    harvest_above_monthly(:) = zero
    control_moist_daily(:,:) = zero
    control_temp_daily(:,:) = zero
    soilcarbon_input_daily(:,:,:) = zero
    ! Land cover change variables
    prod_s(:,:)  = zero
    prod_m(:,:)  = zero
    prod_l(:,:) = zero
    flux_s(:,:)  = zero
    flux_m(:,:)  = zero
    flux_l(:,:) = zero
    cflux_prod_s(:)  = zero
    cflux_prod_m(:) = zero
    cflux_prod_l(:) = zero
    fpc_max(:,:)=zero
    
  END SUBROUTINE stomate_init


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_clear
!!
!>\BRIEF        Deallocate memory of the stomate variables.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_clear

  !! 1. Deallocate all dynamics variables

    IF (ALLOCATED(veget_cov_max)) DEALLOCATE(veget_cov_max)
    IF (ALLOCATED(veget_cov_max_new)) DEALLOCATE(veget_cov_max_new)
    IF (ALLOCATED(ba)) DEALLOCATE(ba)
   
    IF (ALLOCATED(adapted)) DEALLOCATE(adapted)
    IF (ALLOCATED(regenerate)) DEALLOCATE(regenerate)
    IF (ALLOCATED(humrel_daily)) DEALLOCATE(humrel_daily)
    IF (ALLOCATED(vir_humrel_daily)) DEALLOCATE(vir_humrel_daily)
    IF (ALLOCATED(transpir_supply_daily)) DEALLOCATE(transpir_supply_daily)
    IF (ALLOCATED(vir_transpir_supply_daily)) DEALLOCATE(vir_transpir_supply_daily)
    IF (ALLOCATED(transpir_daily)) DEALLOCATE(transpir_daily)
    IF (ALLOCATED(gdd_init_date)) DEALLOCATE(gdd_init_date)
    IF (ALLOCATED(litterhum_daily)) DEALLOCATE(litterhum_daily)
    IF (ALLOCATED(t2m_daily))  DEALLOCATE(t2m_daily)
    IF (ALLOCATED(t2m_min_daily))  DEALLOCATE(t2m_min_daily)
    IF (ALLOCATED(tsurf_daily))  DEALLOCATE(tsurf_daily)
    IF (ALLOCATED(tsoil_daily)) DEALLOCATE(tsoil_daily)
    IF (ALLOCATED(soilhum_daily)) DEALLOCATE(soilhum_daily)
    IF (ALLOCATED(precip_daily)) DEALLOCATE(precip_daily)
    IF (ALLOCATED(gpp_daily)) DEALLOCATE(gpp_daily)
    IF (ALLOCATED(npp_daily)) DEALLOCATE(npp_daily)
    IF (ALLOCATED(turnover_daily)) DEALLOCATE(turnover_daily)
    IF (ALLOCATED(turnover_littercalc)) DEALLOCATE(turnover_littercalc)
    IF (ALLOCATED(humrel_month)) DEALLOCATE(humrel_month)
    IF (ALLOCATED(humrel_week)) DEALLOCATE(humrel_week)
    IF (ALLOCATED(humrel_growingseason)) DEALLOCATE(humrel_growingseason)
    IF (ALLOCATED(vir_humrel_growingseason)) DEALLOCATE(vir_humrel_growingseason)
    IF (ALLOCATED(t2m_longterm)) DEALLOCATE(t2m_longterm)
    IF (ALLOCATED(tlong_ref)) DEALLOCATE(tlong_ref)
    IF (ALLOCATED(t2m_month)) DEALLOCATE(t2m_month)
    IF (ALLOCATED(t2m_week)) DEALLOCATE(t2m_week)
    IF (ALLOCATED(tsoil_month)) DEALLOCATE(tsoil_month)
    IF (ALLOCATED(soilhum_month)) DEALLOCATE(soilhum_month)
    IF (ALLOCATED(fireindex)) DEALLOCATE(fireindex)
    IF (ALLOCATED(firelitter)) DEALLOCATE(firelitter)
    IF (ALLOCATED(maxhumrel_lastyear)) DEALLOCATE(maxhumrel_lastyear)
    IF (ALLOCATED(maxhumrel_thisyear)) DEALLOCATE(maxhumrel_thisyear)
    IF (ALLOCATED(minhumrel_lastyear)) DEALLOCATE(minhumrel_lastyear)
    IF (ALLOCATED(minhumrel_thisyear)) DEALLOCATE(minhumrel_thisyear)
    IF (ALLOCATED(maxgppweek_lastyear)) DEALLOCATE(maxgppweek_lastyear)
    IF (ALLOCATED(maxgppweek_thisyear)) DEALLOCATE(maxgppweek_thisyear)
    IF (ALLOCATED(gdd0_lastyear)) DEALLOCATE(gdd0_lastyear)
    IF (ALLOCATED(gdd0_thisyear)) DEALLOCATE(gdd0_thisyear)
    IF (ALLOCATED(precip_lastyear)) DEALLOCATE(precip_lastyear)
    IF (ALLOCATED(precip_thisyear)) DEALLOCATE(precip_thisyear)
    IF (ALLOCATED(gdd_m5_dormance)) DEALLOCATE(gdd_m5_dormance)
    IF (ALLOCATED(gdd_from_growthinit)) DEALLOCATE(gdd_from_growthinit)
    IF (ALLOCATED(gdd_midwinter)) DEALLOCATE(gdd_midwinter)
    IF (ALLOCATED(ncd_dormance)) DEALLOCATE(ncd_dormance)
    IF (ALLOCATED(ngd_minus5))  DEALLOCATE(ngd_minus5)
    IF (ALLOCATED(PFTpresent)) DEALLOCATE(PFTpresent)
    IF (ALLOCATED(npp_longterm)) DEALLOCATE(npp_longterm)
    IF (ALLOCATED(lm_lastyearmax)) DEALLOCATE(lm_lastyearmax)
    IF (ALLOCATED(lm_thisyearmax)) DEALLOCATE(lm_thisyearmax)
    IF (ALLOCATED(maxfpc_lastyear)) DEALLOCATE(maxfpc_lastyear)
    IF (ALLOCATED(maxfpc_thisyear)) DEALLOCATE(maxfpc_thisyear)
    IF (ALLOCATED(turnover_longterm)) DEALLOCATE(turnover_longterm)
    IF (ALLOCATED(gpp_week)) DEALLOCATE(gpp_week)
    IF (ALLOCATED(senescence)) DEALLOCATE(senescence)
    IF (ALLOCATED(when_growthinit)) DEALLOCATE(when_growthinit)
    IF (ALLOCATED(age))  DEALLOCATE(age)
    IF (ALLOCATED(resp_hetero_d)) DEALLOCATE(resp_hetero_d)
    IF (ALLOCATED(resp_hetero_radia)) DEALLOCATE(resp_hetero_radia)
    IF (ALLOCATED(resp_maint_d)) DEALLOCATE(resp_maint_d)
    IF (ALLOCATED(resp_growth_d)) DEALLOCATE(resp_growth_d)
    IF (ALLOCATED(co2_fire)) DEALLOCATE(co2_fire)
    IF (ALLOCATED(co2_to_bm_dgvm)) DEALLOCATE(co2_to_bm_dgvm)
    IF (ALLOCATED(veget_lastlight)) DEALLOCATE(veget_lastlight)
    IF (ALLOCATED(everywhere)) DEALLOCATE(everywhere)
    IF (ALLOCATED(need_adjacent)) DEALLOCATE(need_adjacent)
    IF (ALLOCATED(leaf_age)) DEALLOCATE(leaf_age)
    IF (ALLOCATED(leaf_frac)) DEALLOCATE(leaf_frac)
! moved to sechiba
!    IF (ALLOCATED(circ_class_n)) DEALLOCATE(circ_class_n)
!    IF (ALLOCATED(circ_class_biomass)) DEALLOCATE(circ_class_biomass)
!    IF (ALLOCATED(ind)) DEALLOCATE(ind)
!    IF (ALLOCATED(biomass)) DEALLOCATE(biomass)
    IF (ALLOCATED(RIP_time)) DEALLOCATE(RIP_time)
    IF (ALLOCATED(time_lowgpp)) DEALLOCATE(time_lowgpp)
    IF (ALLOCATED(time_hum_min)) DEALLOCATE(time_hum_min)
    IF (ALLOCATED(hum_min_dormance)) DEALLOCATE(hum_min_dormance)
    IF (ALLOCATED(litterpart)) DEALLOCATE(litterpart)
    IF (ALLOCATED(litter)) DEALLOCATE(litter)
    IF (ALLOCATED(dead_leaves)) DEALLOCATE(dead_leaves)
    IF (ALLOCATED(carbon)) DEALLOCATE(carbon)
    IF (ALLOCATED(black_carbon)) DEALLOCATE(black_carbon)
    IF (ALLOCATED(lignin_struc)) DEALLOCATE(lignin_struc)
    IF (ALLOCATED(lignin_wood)) DEALLOCATE(lignin_wood)
    IF (ALLOCATED(turnover_time)) DEALLOCATE(turnover_time)
    IF (ALLOCATED(co2_flux_daily)) DEALLOCATE(co2_flux_daily)
    IF (ALLOCATED(co2_flux_monthly)) DEALLOCATE(co2_flux_monthly)
    IF (ALLOCATED(harvest_above_monthly)) DEALLOCATE (harvest_above_monthly)
    IF (ALLOCATED(cflux_prod_monthly)) DEALLOCATE (cflux_prod_monthly)
    IF (ALLOCATED(bm_to_litter)) DEALLOCATE(bm_to_litter)
    IF (ALLOCATED(bm_to_littercalc)) DEALLOCATE(bm_to_littercalc)
    IF (ALLOCATED(herbivores)) DEALLOCATE(herbivores)
    IF (ALLOCATED(resp_maint_part_radia)) DEALLOCATE(resp_maint_part_radia)
    IF (ALLOCATED(resp_maint_radia)) DEALLOCATE(resp_maint_radia)
    IF (ALLOCATED(resp_maint_part)) DEALLOCATE(resp_maint_part)
    IF (ALLOCATED(hori_index)) DEALLOCATE(hori_index)
    IF (ALLOCATED(horipft_index)) DEALLOCATE(horipft_index)
    IF (ALLOCATED(horican_index)) DEALLOCATE(horican_index)
    IF (ALLOCATED(clay_fm)) DEALLOCATE(clay_fm)
    IF (ALLOCATED(humrel_daily_fm)) DEALLOCATE(humrel_daily_fm)
    IF (ALLOCATED(litterhum_daily_fm))  DEALLOCATE(litterhum_daily_fm)
    IF (ALLOCATED(t2m_daily_fm))  DEALLOCATE(t2m_daily_fm)
    IF (ALLOCATED(t2m_min_daily_fm))  DEALLOCATE(t2m_min_daily_fm)
    IF (ALLOCATED(tsurf_daily_fm)) DEALLOCATE(tsurf_daily_fm)
    IF (ALLOCATED(tsoil_daily_fm)) DEALLOCATE(tsoil_daily_fm)
    IF (ALLOCATED(soilhum_daily_fm))  DEALLOCATE(soilhum_daily_fm)
    IF (ALLOCATED(precip_fm)) DEALLOCATE(precip_fm)
    IF (ALLOCATED(gpp_daily_fm))  DEALLOCATE(gpp_daily_fm)
    IF (ALLOCATED(veget_fm)) DEALLOCATE(veget_fm)
    IF (ALLOCATED(veget_max_fm)) DEALLOCATE(veget_max_fm)
    IF (ALLOCATED(lai_fm))  DEALLOCATE(lai_fm)
    !
    IF (ALLOCATED(ok_equilibrium)) DEALLOCATE(ok_equilibrium)
    IF (ALLOCATED(carbon_eq)) DEALLOCATE(carbon_eq)
    IF (ALLOCATED(matrixA)) DEALLOCATE(matrixA)
    IF (ALLOCATED(vectorB)) DEALLOCATE(vectorB)
    IF (ALLOCATED(MatrixV)) DEALLOCATE(MatrixV)
    IF (ALLOCATED(VectorU)) DEALLOCATE(VectorU)
    IF (ALLOCATED(MatrixW)) DEALLOCATE(MatrixW)
    IF (ALLOCATED(previous_stock)) DEALLOCATE(previous_stock)
    IF (ALLOCATED(current_stock)) DEALLOCATE(current_stock) 
    IF (ALLOCATED(nbp_accu)) DEALLOCATE(nbp_accu)
    IF (ALLOCATED(nbp_flux)) DEALLOCATE(nbp_flux)
    !
    IF (ALLOCATED(sigma)) DEALLOCATE (sigma) 
    IF (ALLOCATED(age_stand)) DEALLOCATE (age_stand)
    IF (ALLOCATED(rotation_n)) DEALLOCATE (rotation_n)
    IF (ALLOCATED(last_cut)) DEALLOCATE (last_cut)
    IF (ALLOCATED(KF)) DEALLOCATE (KF)
    IF (ALLOCATED(k_latosa_adapt)) DEALLOCATE (k_latosa_adapt)
    IF (ALLOCATED(harvest_pool)) DEALLOCATE (harvest_pool)
    IF (ALLOCATED(harvest_type)) DEALLOCATE (harvest_type)
    IF (ALLOCATED(harvest_cut)) DEALLOCATE (harvest_cut)
    IF (ALLOCATED(harvest_area)) DEALLOCATE (harvest_area)
    IF (ALLOCATED(harvest_pool_bound)) DEALLOCATE (harvest_pool_bound)
    IF (ALLOCATED(mai)) DEALLOCATE (mai)
    IF (ALLOCATED(pai)) DEALLOCATE (pai)
    IF (ALLOCATED(previous_wood_volume)) DEALLOCATE (previous_wood_volume)
    IF (ALLOCATED(mai_count)) DEALLOCATE (mai_count)
    IF (ALLOCATED(coppice_dens)) DEALLOCATE (coppice_dens)
    IF (ALLOCATED(litter_demand)) DEALLOCATE (litter_demand)
    IF (ALLOCATED(wstress_season)) DEALLOCATE (wstress_season)
    IF (ALLOCATED(wstress_month)) DEALLOCATE (wstress_month)
    IF (ALLOCATED(vir_wstress_fac)) DEALLOCATE (vir_wstress_fac)
    IF (ALLOCATED(c_buried)) DEALLOCATE (c_buried)
    IF (ALLOCATED(bm_sync)) DEALLOCATE (bm_sync)
    IF (ALLOCATED(forest_managed)) DEALLOCATE (forest_managed)
    IF (ALLOCATED(forest_managed_lastyear)) DEALLOCATE (forest_managed_lastyear)
    IF (ALLOCATED(circ_class_dist)) DEALLOCATE (circ_class_dist)
    IF (ALLOCATED(st_dist)) DEALLOCATE (st_dist)

    IF (is_root_prc) THEN
       IF (ALLOCATED(clay_fm_g)) DEALLOCATE(clay_fm_g)
       IF (ALLOCATED(humrel_daily_fm_g)) DEALLOCATE(humrel_daily_fm_g)
       IF (ALLOCATED(litterhum_daily_fm_g))  DEALLOCATE(litterhum_daily_fm_g)
       IF (ALLOCATED(t2m_daily_fm_g))  DEALLOCATE(t2m_daily_fm_g)
       IF (ALLOCATED(t2m_min_daily_fm_g))  DEALLOCATE(t2m_min_daily_fm_g)
       IF (ALLOCATED(tsurf_daily_fm_g)) DEALLOCATE(tsurf_daily_fm_g)
       IF (ALLOCATED(tsoil_daily_fm_g)) DEALLOCATE(tsoil_daily_fm_g)
       IF (ALLOCATED(soilhum_daily_fm_g))  DEALLOCATE(soilhum_daily_fm_g)
       IF (ALLOCATED(precip_fm_g)) DEALLOCATE(precip_fm_g)
       IF (ALLOCATED(gpp_daily_fm_g))  DEALLOCATE(gpp_daily_fm_g)
       IF (ALLOCATED(veget_fm_g)) DEALLOCATE(veget_fm_g)
       IF (ALLOCATED(veget_max_fm_g)) DEALLOCATE(veget_max_fm_g)
       IF (ALLOCATED(lai_fm_g))  DEALLOCATE(lai_fm_g)
       IF (ALLOCATED(ok_equilibrium_g)) DEALLOCATE(ok_equilibrium_g)
    ENDIF

    IF (ALLOCATED(isf)) DEALLOCATE(isf)
    IF (ALLOCATED(nf_written)) DEALLOCATE(nf_written)
    IF (ALLOCATED(nf_cumul)) DEALLOCATE(nf_cumul)
    IF (ALLOCATED(nforce)) DEALLOCATE(nforce)
    IF (ALLOCATED(control_moist)) DEALLOCATE(control_moist)
    IF (ALLOCATED(control_temp)) DEALLOCATE(control_temp)
    IF (ALLOCATED(soilcarbon_input)) DEALLOCATE(soilcarbon_input)
    IF ( ALLOCATED (horip_s_index)) DEALLOCATE (horip_s_index)
    IF ( ALLOCATED (horip_m_index)) DEALLOCATE (horip_m_index)
    IF ( ALLOCATED (horip_l_index)) DEALLOCATE (horip_l_index)
    IF ( ALLOCATED (horip_ss_index)) DEALLOCATE (horip_ss_index)
    IF ( ALLOCATED (horip_mm_index)) DEALLOCATE (horip_mm_index)
    IF ( ALLOCATED (horip_ll_index)) DEALLOCATE (horip_ll_index)
    IF ( ALLOCATED (prod_s)) DEALLOCATE (prod_s)
    IF ( ALLOCATED (prod_m)) DEALLOCATE (prod_m)
    IF ( ALLOCATED (prod_l)) DEALLOCATE (prod_l)
    IF ( ALLOCATED (flux_s)) DEALLOCATE (flux_s)
    IF ( ALLOCATED (flux_m)) DEALLOCATE (flux_m)
    IF ( ALLOCATED (flux_l)) DEALLOCATE (flux_l)
    IF ( ALLOCATED (cflux_prod_s)) DEALLOCATE (cflux_prod_s)
    IF ( ALLOCATED (cflux_prod_m)) DEALLOCATE (cflux_prod_m)
    IF ( ALLOCATED (cflux_prod_l)) DEALLOCATE (cflux_prod_l)
    IF ( ALLOCATED (soilcarbon_input_daily)) DEALLOCATE (soilcarbon_input_daily)
    IF ( ALLOCATED (control_temp_daily)) DEALLOCATE (control_temp_daily)
    IF ( ALLOCATED (control_moist_daily)) DEALLOCATE (control_moist_daily)
    IF ( ALLOCATED (fpc_max)) DEALLOCATE (fpc_max)

 !! 2. reset l_first

    l_first_stomate=.TRUE.

 !! 3. call to clear functions

    CALL get_reftemp_clear
    CALL season_clear
    CALL stomatelpj_clear
    CALL littercalc_clear
    CALL vmax_clear
 
  END SUBROUTINE stomate_clear


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_var_init
!!
!>\BRIEF        Initialize variables of stomate with a none-zero initial value.
!! Subroutine is called only if ::ok_stomate = .TRUE. STOMATE diagnoses some 
!! variables for SECHIBA : assim_param, deadleaf_cover, etc. These variables can 
!! be recalculated from STOMATE's prognostic variables.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): leaf age (::leaf_age) and fraction of leaves in leaf 
!! age class (::leaf_frac). The maximum water on vegetation available for 
!! interception, fraction of soil covered by dead leaves
!! (::deadleaf_cover) and assimilation parameters (:: assim_param).
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_var_init &
       &  (npts, veget_cov_max, leaf_age, leaf_frac, &
       &   tlong_ref, t2m_month, dead_leaves, &
       &   forest_managed, age_stand, &
       &   deadleaf_cover, assim_param)


  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)                         :: npts            !! Domain size - terrestrial pixels only
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)        :: veget_cov_max   !! Fractional coverage: maximum share of the pixel 
                                                                         !! covered by a PFT (unitless) 
    REAL(r_std),DIMENSION(npts),INTENT(in)            :: tlong_ref       !! "long term" 2 meter reference temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in)            :: t2m_month       !! Air temperature at 2 meter integrated over a 
                                                                         !! month (K) 
    REAL(r_std),DIMENSION(npts,nvm,nlitt),INTENT(in)  :: dead_leaves     !! Metabolic and structural fraction of dead leaves 
                                                                         !! per ground area 
                                                                         !! @tex $(gC m^{-2})$ @endtex 
    INTEGER(i_std),DIMENSION(npts,nvm),INTENT(in)     :: forest_managed  !! forest management flag (0-4, unitless)
    INTEGER(i_std),DIMENSION(npts,nvm),INTENT(in)     :: age_stand       !! Age of stand (years)
    
    !! 0.2 Modified variables

    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(inout) :: leaf_age  !! Age of different leaf classes per PFT (days)
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(inout) :: leaf_frac !! Fraction of leaves in leaf age class per PFT 
                                                                         !! (unitless; 1) 
    
    !! 0.3 Output variables

    REAL(r_std),DIMENSION(npts), INTENT (out)         :: deadleaf_cover  !! Fraction of soil covered by dead leaves 
                                                                         !! (unitless) 
    REAL(r_std),DIMENSION(:,:,:), INTENT(out)         :: assim_param     !! min+max+opt temperatures (K) & vmax for 
                                                                         !! photosynthesis  
                                                                         !! @tex $(\mumol m^{-2} s^{-1})$ @endtex
    ! 0.4 Local variables
   
    REAL(r_std),PARAMETER                             :: dt_0 = zero     !! Dummy time step, must be zero
    REAL(r_std),DIMENSION(npts,nvm)                   :: vcmax           !! Dummy vcmax 
                                                                         !! @tex $(\mu mol m^{-2} s^{-1})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm)                   :: t_photo_min     !! Min temperature for photosynthesis (K)
    REAL(r_std),DIMENSION(npts,nvm)                   :: t_photo_opt     !! Opt temperature for photosynthesis (K)
    REAL(r_std),DIMENSION(npts,nvm)                   :: t_photo_max     !! Max temperature for photosynthesis (K)
    REAL(r_std), DIMENSION(npts,nvm)                  :: tau_dum_leaf    !! Dummy leaf turnover time. Equals the initial
                                                                         !! because it has to be calculated before
                                                                         !! waterstress is known (days)
    INTEGER(i_std)                                    :: j               !! Index (untiless)
    
!_ ================================================================================================================================   

    ! Only if stomate is activated
    IF (control%ok_stomate) THEN
     
  !! 1. photosynthesis parameters

       !! 1.1 vcmax (stomate_vmax.f90)
       IF (control%ok_functional_allocation)THEN

          DO j = 1,nvm
             tau_dum_leaf(:,j) = tau_leaf(j)
          ENDDO
          CALL vmax_prognostic (npts, dt_0, leaf_age, leaf_frac,  &
            &    age_stand, vcmax, tau_dum_leaf)

       ELSE

          CALL vmax_diagnostic (npts, dt_0, leaf_age, leaf_frac,  &
            &    forest_managed, age_stand, &
            &    vcmax)

       ENDIF
       
       !! 1.2 assimilation temperatures (stomate_assimtemp.f90)
       CALL assim_temp(npts, tlong_ref, t2m_month, &
            &                    t_photo_min, t_photo_opt, t_photo_max)
      
       !! 1.3 transform into nvm vegetation types
       assim_param(:,:,ivcmax) = zero
       DO j = 2, nvm
          assim_param(:,j,ivcmax)=vcmax(:,j)
       ENDDO
       
  !! 2. Dead leaf cover (stomate_litter.f90)
       
       CALL deadleaf (npts, veget_cov_max, dead_leaves, deadleaf_cover)     
       
    ENDIF ! ok_stomate = .TRUE.
    
  END SUBROUTINE stomate_var_init


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_accu
!!
!>\BRIEF        Accumulate a variable for the time period specified by 
!! ::dt_tot or calculate the mean value over ::dt_tot.
!! 
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): accumulated or mean variable ::field_out:: 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_accu &
       &  (npts, n_dim2, dt_tot, dt, ldmean, field_in, field_out)
    
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
 
    INTEGER(i_std),INTENT(in)                        :: npts      !! Domain size (unitless)
    INTEGER(i_std),INTENT(in)                        :: n_dim2    !! 2nd dimension (1 or nvm)
    REAL(r_std),INTENT(in)                           :: dt_tot    !! Current time step (days)
    REAL(r_std),INTENT(in)                           :: dt        !! Accumulated time step (days)
    LOGICAL,INTENT(in)                               :: ldmean    !! Flag to calculate the mean over
!!$    LOGICAL,INTENT(in)                               :: ldnight   !! Flag indicating day vs night time
    REAL(r_std),DIMENSION(:,:),INTENT(in)            :: field_in  !! Field that needs to be accumulated
 
    !! 0.2 Output variables
    
    !! 0.3 Modified variables

    REAL(r_std),DIMENSION(:,:),INTENT(inout)         :: field_out !! Accumulated or mean field

    !! 0.4 Local variables

    INTEGER(i_std)                                   :: ipts      !! Index
  
!_ ================================================================================================================================

  !! 1. Accumulate field

    field_out(:,:) = field_out(:,:)+field_in(:,:)*dt


  !! 2. Mean fields

    IF (ldmean) THEN
       field_out(:,:) = field_out(:,:)/dt_tot
    ENDIF

  END SUBROUTINE stomate_accu


!! ================================================================================================================================
!! SUBROUTINE 	: init_forcing
!!
!>\BRIEF        Allocate memory for the variables containing the forcing data.
!! The maximum size of the allocated memory is specified in run definition file
!! (::max_totsize) and needs to be a compromise between charging the memory and 
!! accessing disks to get the forcing data.
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): Strictly speaking the subroutine has no output 
!! variables. However, the routine allocates memory for later use. 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE init_forcing (npts,nsfm,nsft)
    
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    
    INTEGER(i_std),INTENT(in) :: npts     !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std),INTENT(in) :: nsfm     !! Number of time steps that can be stored in memory (unitless)
    INTEGER(i_std),INTENT(in) :: nsft     !! Number of time steps in a year (unitless)

   !! 0.2 Output variables

   !! 0.3 Modified variables

   !! 0.4 Local variables

    LOGICAL                   :: l_error  !! Check errors in netcdf call
    INTEGER(i_std)            :: ier      !! Check errors in netcdf call
!_ ================================================================================================================================
    
  !! 1. Allocate memory

    ! Note ::nvm is number of PFTs and ::nbdl is number of soil layers
    l_error = .FALSE.
    ALLOCATE(clay_fm(npts,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables clay_fm ',npts,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(humrel_daily_fm(npts,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables humrel_daily_fm ',npts,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(litterhum_daily_fm(npts,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables litterhum_daily_fm ',npts,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(t2m_daily_fm(npts,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_daily_fm ',npts,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(t2m_min_daily_fm(npts,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_min_daily_fm ',npts,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(tsurf_daily_fm(npts,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables tsurf_daily_fm ',npts,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(tsoil_daily_fm(npts,nbdl,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables tsoil_daily_fm ',npts,nbdl,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(soilhum_daily_fm(npts,nbdl,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables soilhum_daily_fm ',npts,nbdl,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(precip_fm(npts,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables precip_fm ',npts,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(gpp_daily_fm(npts,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables gpp_daily_fm ',npts,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(veget_fm(npts,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_fm ',npts,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(veget_max_fm(npts,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_max_fm ',npts,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(lai_fm(npts,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables lai_fm ',npts,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(isf(nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables isf ',nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(nf_written(nsft),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables nf_written ',nsft
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(nf_cumul(nsft),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables nf_cumul ',nsft
       STOP 'init_forcing'
    ENDIF
    
  !! 2. Allocate memory for the root processor only (parallel computing)

    ! Where, ::nbp_glo is the number of global continental points
    IF (is_root_prc) THEN
       ALLOCATE(clay_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables clay_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(humrel_daily_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables humrel_daily_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(litterhum_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables litterhum_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(t2m_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(t2m_min_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_min_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(tsurf_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables tsurf_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(tsoil_daily_fm_g(nbp_glo,nbdl,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables tsoil_daily_fm_g ',nbp_glo,nbdl,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(soilhum_daily_fm_g(nbp_glo,nbdl,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables soilhum_daily_fm_g ',nbp_glo,nbdl,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(precip_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables precip_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(gpp_daily_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables gpp_daily_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(veget_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(veget_max_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_max_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(lai_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables lai_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
    ELSE
       ! Allocate memory for co-processors
       ALLOCATE(clay_fm_g(0,nsfm),stat=ier)
       ALLOCATE(humrel_daily_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(litterhum_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(t2m_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(t2m_min_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(tsurf_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(tsoil_daily_fm_g(0,nbdl,nsfm),stat=ier)
       ALLOCATE(soilhum_daily_fm_g(0,nbdl,nsfm),stat=ier)
       ALLOCATE(precip_fm_g(0,nsfm),stat=ier)
       ALLOCATE(gpp_daily_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(veget_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(veget_max_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(lai_fm_g(0,nvm,nsfm),stat=ier)
    ENDIF ! is_root_proc
    
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables'
       STOP 'init_forcing'
    ENDIF

  !! 3. Initilaize variables

    CALL forcing_zero
    
  END SUBROUTINE init_forcing


!! ================================================================================================================================
!! SUBROUTINE 	: forcing_zero
!!
!>\BRIEF        Initialize variables containing the forcing data; variables are 
!! set to zero.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE forcing_zero
    
    clay_fm(:,:) = zero
    humrel_daily_fm(:,:,:) = zero
    litterhum_daily_fm(:,:) = zero
    t2m_daily_fm(:,:) = zero
    t2m_min_daily_fm(:,:) = zero
    tsurf_daily_fm(:,:) = zero
    tsoil_daily_fm(:,:,:) = zero
    soilhum_daily_fm(:,:,:) = zero
    precip_fm(:,:) = zero
    gpp_daily_fm(:,:,:) = zero
    veget_fm(:,:,:) = zero
    veget_max_fm(:,:,:) = zero
    lai_fm(:,:,:) = zero
    
  END SUBROUTINE forcing_zero


!! ================================================================================================================================
!! SUBROUTINE 	: forcing_write
!!
!>\BRIEF        Appends data values to a netCDF file containing the forcing 
!! variables of the general processes in stomate.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): netCDF file
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE forcing_write(forcing_id,ibeg,iend)
    
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)      :: forcing_id  !! File identifer of forcing file, assigned when netcdf is created
    INTEGER(i_std),INTENT(in)      :: ibeg, iend  !! First and last time step to be written

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                 :: iisf        !! Index of isf where isf is the number of time steps that can be 
                                                  !! stored in memory 
    INTEGER(i_std)                 :: iblocks     !! Index of block that is written
    INTEGER(i_std)                 :: nblocks     !! Number of blocks that needs to be written
    INTEGER(i_std)                 :: ier         !! Check errors in netcdf call
    INTEGER(i_std),DIMENSION(0:2)  :: ifirst      !! First block in memory - changes with iblocks
    INTEGER(i_std),DIMENSION(0:2)  :: ilast       !! Last block in memory - changes with iblocks
    INTEGER(i_std),PARAMETER       :: ndm = 10    !! Maximum number of dimensions
    INTEGER(i_std),DIMENSION(ndm)  :: start       !! First block to write
    INTEGER(i_std)                 :: ndim        !! Dimensions of forcing to be added to the netCDF
    INTEGER(i_std),DIMENSION(ndm)  :: count_force !! Number of elements in each dimension  
    INTEGER(i_std)                 :: vid         !! Variable identifer of netCDF
!_ ================================================================================================================================
    
  !! 1. Determine number of blocks of forcing variables that are stored in memory

    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    DO iisf = ibeg, iend
       IF (     (nblocks /= 0) &
            &      .AND.(isf(iisf) == isf(ilast(nblocks))+1)) THEN
          ! Last block found
          ilast(nblocks) = iisf
       ELSE
          ! First block found
          nblocks = nblocks+1
          IF (nblocks > 2)  STOP 'Problem in forcing_write'
          ifirst(nblocks) = iisf
          ilast(nblocks) = iisf
       ENDIF
    ENDDO

  !! 2. Gather distributed variables (parallel computing)

    CALL gather(clay_fm,clay_fm_g)
    CALL gather(humrel_daily_fm,humrel_daily_fm_g)
    CALL gather(litterhum_daily_fm,litterhum_daily_fm_g)
    CALL gather(t2m_daily_fm,t2m_daily_fm_g)
    CALL gather(t2m_min_daily_fm,t2m_min_daily_fm_g)
    CALL gather(tsurf_daily_fm,tsurf_daily_fm_g)
    CALL gather(tsoil_daily_fm,tsoil_daily_fm_g)
    CALL gather(soilhum_daily_fm,soilhum_daily_fm_g)
    CALL gather(precip_fm,precip_fm_g)
    CALL gather(gpp_daily_fm,gpp_daily_fm_g)
    CALL gather(veget_fm,veget_fm_g)
    CALL gather(veget_max_fm,veget_max_fm_g)
    CALL gather(lai_fm,lai_fm_g)
 
 !! 3. Append data to netCDF file
   
    IF (is_root_prc) THEN
       ! The netCDF file has been created earlier in this module, a file ID is available 
       ! and variables and dimensions have already been defined
       DO iblocks = 1, nblocks
          IF (ifirst(iblocks) /= ilast(iblocks)) THEN
             ndim = 2
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(clay_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'clay',vid)
             ier = NF90_PUT_VAR (forcing_id,vid, &
                  &              clay_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(humrel_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'humrel',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            humrel_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(litterhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'litterhum',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            litterhum_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            t2m_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_min_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m_min',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            t2m_min_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsurf_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsurf',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            tsurf_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsoil_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsoil',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            tsoil_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(soilhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'soilhum',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            soilhum_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(precip_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'precip',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            precip_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(gpp_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'gpp',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            gpp_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            veget_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_max_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget_max',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            veget_max_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(lai_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'lai',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            lai_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
          ENDIF
       ENDDO
    ENDIF
    
  !! 4. Adjust flag of forcing file
    nf_written(isf(:)) = .TRUE.

  END SUBROUTINE forcing_write

  
!! ================================================================================================================================
!! SUBROUTINE 	: forcing_read
!!
!>\BRIEF        Read forcing file.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None 
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE forcing_read(forcing_id,nsfm)
   
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)  :: forcing_id           !! File identifer of forcing file, assigned when netcdf is created
    INTEGER(i_std),INTENT(in)  :: nsfm                 !! Number of time steps stored in memory        
    
    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                 :: iisf              !! Index of isf where isf is the number of time steps that can be stored in 
                                                        !! memory 
    INTEGER(i_std)                 :: iblocks           !! Index of block that is written
    INTEGER(i_std)                 :: nblocks           !! Number of blocks that needs to be written
    INTEGER(i_std)                 :: ier               !! Check error of netcdf call
    INTEGER(i_std),DIMENSION(0:2)  :: ifirst            !! First block in memory - changes with iblocks
    INTEGER(i_std),DIMENSION(0:2)  :: ilast             !! Last block in memory - changes with iblocks
    INTEGER(i_std),PARAMETER       :: ndm = 10          !! Maximum number of dimensions
    INTEGER(i_std),DIMENSION(ndm)  :: start             !! First block to write
    INTEGER(i_std)                 :: ndim              !! Dimensions of forcing to be added to the netCDF
    INTEGER(i_std),DIMENSION(ndm)  :: count_force       !! Number of elements in each dimension
    INTEGER(i_std)                 :: vid               !! Variable identifer of netCDF
    LOGICAL, PARAMETER             :: check=.FALSE.     !! Flag for debugging 
    LOGICAL                        :: a_er=.FALSE.      !! Error catching from netcdf file
!_ ================================================================================================================================

    IF (check) WRITE(numout,*) "forcing_read "
    
  !! 1. Set to zero if the corresponding forcing state

    ! has not yet been written into the file  
    DO iisf = 1, nsfm
       IF (.NOT.nf_written(isf(iisf))) THEN
          clay_fm(:,iisf) = zero
          humrel_daily_fm(:,:,iisf) = zero
          litterhum_daily_fm(:,iisf) = zero
          t2m_daily_fm(:,iisf) = zero
          t2m_min_daily_fm(:,iisf) = zero
          tsurf_daily_fm(:,iisf) = zero
          tsoil_daily_fm(:,:,iisf) = zero
          soilhum_daily_fm(:,:,iisf) = zero
          precip_fm(:,iisf) = zero
          gpp_daily_fm(:,:,iisf) = zero
          veget_fm(:,:,iisf) = zero
          veget_max_fm(:,:,iisf) = zero
          lai_fm(:,:,iisf) = zero
       ENDIF
    ENDDO
    
  !! 2. determine blocks of forcing states that are contiguous in memory

    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    
    DO iisf = 1, nsfm
       IF (nf_written(isf(iisf))) THEN
          IF (     (nblocks /= 0) &
               &        .AND.(isf(iisf) == isf(ilast(nblocks))+1)) THEN

             ! element is contiguous with last element found
             ilast(nblocks) = iisf
          ELSE

             ! found first element of new block
             nblocks = nblocks+1
             IF (nblocks > 2)  STOP 'Problem in forcing_read'
             
             ifirst(nblocks) = iisf
             ilast(nblocks) = iisf
          ENDIF
       ENDIF
    ENDDO
    IF (check) WRITE(numout,*) "forcing_read nblocks, ifirst, ilast",nblocks, ifirst, ilast
    
  !! 3. Read variable values

    IF (is_root_prc) THEN
       DO iblocks = 1, nblocks
          IF (check) WRITE(numout,*) "forcing_read iblocks, ifirst(iblocks), ilast(iblocks)",iblocks, &
               ifirst(iblocks), ilast(iblocks)
          IF (ifirst(iblocks) /= ilast(iblocks)) THEN
             a_er=.FALSE.
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(clay_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'clay',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            clay_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(humrel_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'humrel',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            humrel_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(litterhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'litterhum',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              litterhum_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              t2m_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_min_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m_min',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              t2m_min_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsurf_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsurf',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              tsurf_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsoil_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsoil',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              tsoil_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(soilhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'soilhum',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              soilhum_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(precip_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'precip',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              precip_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(gpp_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'gpp',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            gpp_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            veget_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_max_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget_max',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            veget_max_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(lai_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'lai',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            lai_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)
             IF (a_er) THEN
                CALL ipslerr_p (3,'forcing_read', &
                     &        'PROBLEM when read forcing file', &
                     &        '','')
             ENDIF

          ENDIF ! (ifirst(iblocks) /= ilast(iblocks))
       ENDDO ! iblocks
    ENDIF ! is_root_prc

  !! 4. Distribute the variable over several processors

    CALL scatter(clay_fm_g,clay_fm)
    CALL scatter(humrel_daily_fm_g,humrel_daily_fm)
    CALL scatter(litterhum_daily_fm_g,litterhum_daily_fm)
    CALL scatter(t2m_daily_fm_g,t2m_daily_fm)
    CALL scatter(t2m_min_daily_fm_g,t2m_min_daily_fm)
    CALL scatter(tsurf_daily_fm_g,tsurf_daily_fm)
    CALL scatter(tsoil_daily_fm_g,tsoil_daily_fm)
    CALL scatter(soilhum_daily_fm_g,soilhum_daily_fm)
    CALL scatter(precip_fm_g,precip_fm)
    CALL scatter(gpp_daily_fm_g,gpp_daily_fm)
    CALL scatter(veget_fm_g,veget_fm)
    CALL scatter(veget_max_fm_g,veget_max_fm)
    CALL scatter(lai_fm_g,lai_fm)
  
  END SUBROUTINE forcing_read


!! ================================================================================================================================
!! SUBROUTINE 	: setlai
!!
!>\BRIEF        Routine to force the lai in STOMATE. The code in this routine
!! simply CALCULATES lai and is therefore not functional. The routine should be 
!! rewritten if one wants to force lai.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): ::lai
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE setlai(npts,lai,biomass)

  !! 0 Variable and parameter declaration 
  
    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)                                      :: npts     !! Domain size - number of pixels (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in)  :: biomass  !!Biomass per PFT
    
    !! 0.2 Output variables

    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)  :: lai  !! PFT leaf area index @tex $(m^{2} m^{-2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                               :: j    !! index (unitless)
!_ ================================================================================================================================
    
    !! 1. Set lai for bare soil to zero

    lai(:,ibare_sechiba) = zero

    !! 2. Multiply foliage biomass by sla to calculate lai for all PFTs and pixels

    DO j=2,nvm
       lai(:,j) = biomass(:,j,ileaf,icarbon)*sla(j)
    ENDDO
    
  END SUBROUTINE setlai

END MODULE stomate
